<?php
function sellvia_decode_content_urls( $content ){

    $site_url = get_site_url();

    $find_h = '#^http(s)?://#';
    $find_w = '/^www\./';
    $domain = preg_replace( $find_h, '', $site_url );
    $domain = preg_replace( $find_w, '', $domain );

    $content = str_replace( '##siteurl##', $site_url, $content );
    $content = str_replace( '##sitedomain##', $domain, $content );

    return $content;
}

function sellvia_integer( $int ) {
    return preg_replace( '/\D/', '', $int );
}

function sellvia_is_url( $url ) {
    return (bool) preg_match( '|(\/\/)(www\.)?(.)*[\.](.)*$|iu', $url );
}

function sellvia_is_external( $url ) {
    return strripos( $url, $_SERVER[ "SERVER_NAME" ] ) === false;
}

function sellvia_floatvalue( $value ) {

    $value = html_entity_decode( $value, ENT_QUOTES, "UTF-8" );

    $value = preg_replace('/[^0-9,.]/', '', $value, -1 );

    if( preg_match( '/(\d+\,\d+)+\.\d+/', $value ) ){
        $value = str_replace( ',', '', $value );
    }

    $value = str_replace( ',', '.', $value );

    return number_format( floatval( $value ), 2, '.', '' );
}

function sellvia_prepare_image_url( $url ) {

    $url = set_url_scheme( $url, 'https' );

    $split_url = explode( '?', $url );

    return $split_url[ 0 ];
}

function sellvia_prepare_options( $data = [] ) {

    $foo = [];

    foreach( $data as $key => $val )
        $foo[] = [
            'value' => $key,
            'label' => $val
        ];

    return $foo;
}

function sellvia_prepare_var_slug( $str ) {

    return substr( md5( $str ), 0, 22 );
}

function sellvia_get_ip_address() {

    if ( ! empty( $_SERVER['HTTP_CLIENT_IP'] ) ) {

        $ip = $_SERVER['HTTP_CLIENT_IP'];
    }
    elseif ( ! empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {

        $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
    }
    else {
        $ip = $_SERVER['REMOTE_ADDR'];
    }

    return $ip;
}

function sellvia_format_date( $time ) {
    $format = 'M d, Y H:i';
    $string = $time;
    if ( ! preg_match('#([0-9]{1,4})-([0-9]{1,2})-([0-9]{1,2}) ([0-9]{1,2}):([0-9]{1,2}):([0-9]{1,2})#', $string, $matches) )
        return date( $format, 0 );
    $string_time = gmmktime( $matches[4], $matches[5], $matches[6], $matches[2], $matches[3], $matches[1] );
    $string_localtime = gmdate( $format, $string_time );

    return $string_localtime;
}

function sellvia_array_insert( $array, $index, $insert ) {

    return array_slice( $array, 0, $index, true ) + $insert + array_slice( $array, $index, count( $array ) - $index, true);
}

function sellvia_url_remove_get_params( $url ) {

    $split_url   = explode( '?', $url );

    return $split_url[ 0 ];
}


