<?php

function sellvia_init_db() {

    global $wpdb;

    $wpdb->sellvia_activities        = $wpdb->prefix . 'sellvia_activities';
    $wpdb->sellvia_shipping_group    = $wpdb->prefix . 'sellvia_shipping_group';
    $wpdb->sellvia_shipping_relation = $wpdb->prefix . 'sellvia_shipping_relation';
}
add_action( 'init', 'sellvia_init_db', 5 );

function sellvia_html_tags_code() {

    global $allowedposttags;

    if( ! isset( $allowedposttags['iframe'] ) ){
        $allowedposttags['iframe'] = [
            'title' => true,
            'src' => true,
            'width' => true,
            'height' => true,
            'frameborder' => true,
            'allowfullscreen' => true,
        ];
    }
}
add_action('init', 'sellvia_html_tags_code', 10);

function sellvia_init_with_keys(){

    $keys = sellvia_get_api_keys();

    if( empty( $keys['public_key'] ) || empty( $keys['secret_key'] ) ){
        return;
    }

    add_action( 'init', 'sellvia_front_size_chart_init', 20);
    add_action( 'init', 'sellvia_shipping_badge_init', 20);
    add_action( 'init', 'sellvia_front_reviews_init', 20);

    add_action('init', 'sellvia_media_init', 8);

    add_action('init', 'sellvia_shipping_init', 9);

    add_action('init', 'sellvia_nice_attributes_init', 9);

    add_action( 'admin_head', 'sellvia_order_enqueue_scripts' );

    add_action( 'ads_adsorders_description_header', 'sellvia_go_to_catalog_orders');

//    add_action( 'admin_init', 'sellvia_last_update_products_notice', 20);

    if( is_user_logged_in() ) {
        add_action( 'wp_footer', 'sellvia_report_product_html', 0);
    }

    add_action( 'before_delete_post', 'sellvia_delete_product_images', 10, 2 );
    add_action( 'delete_comment', 'sellvia_delete_comment', 10, 2 );

    add_filter( 'mce_css', 'sellvia_plugin_mce_css' );
}
add_action( 'init', 'sellvia_init_with_keys', 5);

function sellvia_last_update_products_notice(){

    $last_update = get_option( 'sellvia_last_update_products', '' );

    if( empty( $last_update ) ){
        $last_update = current_time('timestamp');
        update_option( 'sellvia_last_update_products', $last_update );
    }else{

        $settings = sellvia_parse_args( \sellvia\ProductUpdate\ProductUpdate::$default_setting,  get_option( 'sellvia_update_product_settings', [] ) );

        if(
            ( intval( $last_update ) < ( current_time('timestamp') - 86400 * 3 ) ) &&
            ( $settings['cost'] == 'update' || $settings['stock'] == 'update' )
        ) {

            global $wpdb;

            $self_products = $wpdb->get_results(
                "SELECT
                m.post_id, m.meta_value as sellvia_post_id                
                FROM {$wpdb->prefix}postmeta m
                WHERE m.meta_key = 'sellvia_post_id'",
                ARRAY_A);

            if (!empty($self_products)) {


                add_action('admin_notices', function () {

                    $last_update = get_option( 'sellvia_last_update_products', '' );

                    printf('<div class="%1$s">
                            <div style="display: flex; flex-wrap: nowrap;">
                                <div style="
                                    margin-top:10px;
                                    margin-right: 10px; 
                                    margin-bottom: 10px; 
                                    width: 57px; 
                                    min-width: 57px; 
                                    height: 57px; 
                                    background-color: #5f5f5f; 
                                    background-image:  url(%2$s); 
                                    background-position: center; 
                                    background-size: 45px 45px; 
                                    background-repeat:no-repeat; 
                                    border-radius: 4px;"
                                ></div>
                                <div>
                                    <h4 style="margin-top:10px;margin-bottom:8px;">%3$s</h4>
                                    <p>%4$s</p>
                                </div>
                                <div style="align-self: center; margin-left: auto;">
                                    <a 
                                        href="%5$s"
                                        class="button button-primary button-hero"
                                    >Update products</a>
                                </div>
                            </div>
                        </div>',
                        'notice notice-warning',
                        SELLVIA_URL . '/assets/images/icon_sellvia.svg',
                        __('Sellvia products were last updated', 'sel') . ' ' . human_time_diff( intval( $last_update ), current_time('timestamp')) . ' ago' ,
                        __('We recommend updating products every 3 days', 'sel'),
                        get_admin_url() . 'admin.php?page=sellvia_updates'
                    );
                });
            }
        }
    }
}

function sellvia_order_enqueue_scripts(){

    $screen = get_current_screen();

    $ads = SELLVIA_PLUGIN == 'ADS' &&
        isset( $_GET['page'] ) &&
        isset( $_GET['trans_id'] ) &&
        $_GET['page'] == 'adsorders' &&
        $_GET['trans_id'] > 0;

    $woo = (
            ( in_array( SELLVIA_PLUGIN, ['ADSW', 'WOO'] ) ) &&
            isset( $screen->post_type ) &&
            $screen->post_type == 'shop_order'
        ) &&
        (
            (isset( $_GET['post'] ) && $_GET['post'] > 0) ||
            (isset( $_GET['id'] ) && $_GET['id'] > 0)
        );

    if( $ads || $woo ) {
        wp_enqueue_script( 'sellvia_order_script', SELLVIA_URL . '/assets/js/order.min.js', ['jquery'], SELLVIA_VERSION );
        wp_enqueue_style( 'sellvia_order_style', SELLVIA_URL . '/assets/css/order.min.css', [], SELLVIA_VERSION );
    }
}

add_action( 'current_screen', function(){

    if( in_array( SELLVIA_PLUGIN, ['ADSW', 'WOO'] ) ){
        $screen = get_current_screen();

        if( isset( $screen->id ) && ( $screen->id === 'shop_order' || $screen->id === 'edit-shop_order' ) ){
            add_action( 'admin_footer', 'sellvia_go_to_catalog_orders');
            add_action( 'admin_footer', 'sellvia_go_to_catalog_orders_script');
        }
    }
} );

function sellvia_media_init(){

    $external_media = new \sellvia\Media\cdnMedia();
}

function sellvia_front_size_chart_init(){

    $front_reviews = new \sellvia\front\SizeChart();
}

function sellvia_shipping_badge_init(){

    $shipping_badge = new \sellvia\ShippingBadge\ShippingBadge();
}

function sellvia_front_reviews_init(){

    $front_reviews = new \sellvia\front\Reviews();
}

function sellvia_shipping_init(){

    if( in_array( SELLVIA_PLUGIN, ['ADS'] ) ){
        new \sellvia\Shipping\ShippingAds();
    }

    if( in_array( SELLVIA_PLUGIN, ['ADSW', 'WOO'] ) ){
        new \sellvia\Shipping\ShippingWoo();
    }
}

function sellvia_nice_attributes_init(){
    if( in_array( SELLVIA_PLUGIN, ['WOO'] ) ) {
        $nice_attributes = new \sellvia\front\NiceAttributes();
    }
}

function sellvia_tracking_init(){

    if( in_array( SELLVIA_PLUGIN, ['ADS'] ) ){
        $tracking = new \sellvia\Tracking\TrackingAds();
    }

    if( in_array( SELLVIA_PLUGIN, ['ADSW', 'WOO'] ) ){
        $tracking = new \sellvia\Tracking\TrackingWoo();
    }
}
//add_action( 'init', 'sellvia_tracking_init');

function sellvia_hide_core_update_notifications_from_users() {

    $screen = get_current_screen();

    if( isset( $screen->id) && in_array( $screen->id, [
            'toplevel_page_sellvia',
            'sellvia_page_sellvia_academy',
            'sellvia_page_sellvia_products',
            'sellvia_page_sellvia_presets',
            'sellvia_page_sellvia_fast_shipping_badge',
            'sellvia_page_sellvia_settings',
            'sellvia_page_sellvia_updates',
            'sellvia_page_sellvia_license',
        ] ) ){
        remove_all_actions('admin_notices');
        remove_all_actions('all_admin_notices');
    }

}
add_action( 'admin_head', 'sellvia_hide_core_update_notifications_from_users', 10000 );

function sellvia_view_on_sellvia( $actions, $post ){

    if( !current_user_can( 'edit_posts' ) && 'product' != $post->post_type ) {
        return $actions;
    }

    $sellvia_post_id = get_post_meta( $post->ID, 'sellvia_post_id', true  );

    if( empty( $sellvia_post_id ) )
        return $actions;

    $actions = array_merge( $actions, [
        'view_on_sellvia' => sprintf( '<a target="_blank" href="%1$s">%2$s</a>',
            esc_url( SELLVIA_CATALOG_PATH . 'product/' . $sellvia_post_id ),
            __( 'View on Sellvia', 'sel' )
        ),
    ] );

    return $actions;
}
add_filter( 'post_row_actions', 'sellvia_view_on_sellvia', 15, 2 );

function sellvia_admin_bar( $wp_admin_bar, $post_id ){

    $sellvia_post_id = get_post_meta( $post_id, 'sellvia_post_id', true  );

    if( $sellvia_post_id ) {

        $url = esc_url( SELLVIA_CATALOG_PATH . 'product/' . $sellvia_post_id );

        $args = [
            'id'    => 'view-on-sellvia',
            'title' =>
                '<span style="width: 16px; font-size: 16px; margin-right: 3px" class="ab-icon"><img style="max-width: 100%;" src="'. SELLVIA_URL . '/assets/images/icon_sellvia.svg' . '" /></span>
                <span class="ab-label">'.__( 'View on Sellvia', 'sel' ).'</span>',
            'href'  => $url,
            'meta'  => [
                'class'  => 'sellvia-button',
                'target' => '_blank'
            ]
        ];

        $wp_admin_bar->add_node( $args );

        $args = [
            'id'    => 'report-on-sellvia',
            'title' =>
                '<span style="width: 16px; font-size: 16px; margin-right: 3px" class="ab-icon"><img style="max-width: 100%;" src="'. SELLVIA_URL . '/assets/images/icon_sellvia.svg' . '" /></span>
                <span class="ab-label">'.__( 'Report product', 'sel' ).'</span>',
            'href' => '#',

            'meta'  => [
                'class'  => 'sellvia-button',
                'onclick' => 'document.dispatchEvent( new Event("report_product"));',
            ]
        ];

        $wp_admin_bar->add_node( $args );
    }
}

function sellvia_button_in_bar_admin( $wp_admin_bar ) {

    if( !is_admin() ) {
        return null;
    }

    $screen = get_current_screen();

    if( isset( $screen->id ) && $screen->id != 'product' ) {
        return false;
    }

    global $post;

    sellvia_admin_bar( $wp_admin_bar, $post->ID );

}
add_action( 'admin_bar_menu', 'sellvia_button_in_bar_admin', 80 );

function sellvia_button_in_bar_front( $wp_admin_bar ){

    if( is_singular( 'product' ) && !is_admin() ) {

        global $wp_the_query;

        $current_object = $wp_the_query->get_queried_object();

        if( empty( $current_object ) || !current_user_can( 'edit_post', $current_object->ID ) )
            return;

        sellvia_admin_bar( $wp_admin_bar, $current_object->ID );
    }
}
add_action( 'admin_bar_menu', 'sellvia_button_in_bar_front', 80 );

function sellvia_is_sellvia_product_filter( $post_type, $which ){

    if( $post_type == 'product' ) {

        $is_sellvia_product = $_GET['is_sellvia_product'] ?? 0;

        echo '<select name="is_sellvia_product" id="is_sellvia_product">';
        echo '<option value="">' . __( 'All products', 'msp' ) . '</option>';

        printf('<option value="%1$s" %3$s>%2$s</option>',
            1,
            __( 'Only from Sellvia', 'sel'),
            intval( $is_sellvia_product ) === 1  ? 'selected="selected"': ''
        );

        echo '</select>';
    }
}
add_action( 'restrict_manage_posts', 'sellvia_is_sellvia_product_filter', 100, 2);

function sellvia_is_sellvia_product( $pieces ) {

    $screen = get_current_screen();


    if( ! isset( $screen->post_type ) || $screen->post_type != 'product' ) {
        return $pieces;
    }
    if( ! isset( $_GET[ 'is_sellvia_product' ] )  || !$_GET[ 'is_sellvia_product' ] ){
        return $pieces;
    }

    $is_sellvia_product = $_GET[ 'is_sellvia_product' ];

    if( $screen->base === 'edit' && intval( $is_sellvia_product ) === 1 ) {
        global $wpdb;
        $pieces[ 'join'] .= 'INNER JOIN '. $wpdb->postmeta . ' spm ON '.$wpdb->posts.'.ID = spm.post_id ';
        $pieces[ 'where' ] = $pieces[ 'where' ] . " AND spm.meta_key = 'sellvia_post_id'";
    }

    return $pieces;
}

function sellvia_admin_init_posts_clauses() {

    add_filter( 'posts_clauses', 'sellvia_is_sellvia_product', 20, 1 );
}
add_action( 'admin_init', 'sellvia_admin_init_posts_clauses' );

function sellvia_columns( $columns ) {

    return sellvia_array_insert( $columns, 7, [
        'is_sellvia_product' => '<img width="24" height="24" title="'. __('Sellvia product', 'sel') .'" src="'. SELLVIA_URL . '/assets/images/sellvia_icon_white.svg" />' . '<span class="title">' . __('Sellvia product', 'sel') . '</span>'
    ] );
}
add_filter( 'manage_product_posts_columns', 'sellvia_columns', 100, 1 );

function  sellvia_is_sellvia_product_column_value( $column_name, $id ) {

    if( $column_name == 'is_sellvia_product' ){

        $sellvia_post_id = get_post_meta( $id, 'sellvia_post_id', true);

        if( !empty($sellvia_post_id) && is_numeric($sellvia_post_id) ){

            $url = esc_url( SELLVIA_CATALOG_PATH . 'product/' . $sellvia_post_id );

            echo '<div>
                <a target="_blank" href="'. $url .'">
                    <img title="'. __('View on Sellvia', 'sel') .'" width="24" height="24" src="'. SELLVIA_URL . '/assets/images/sellvia_icon_white.svg" />
                </a>
            </div>';
        }

//        $sellvia_marketing_url = get_post_meta( $id, 'sellvia_marketing_url', true );
//
//
//        if( !empty( $sellvia_marketing_url ) && sellvia_is_url( $sellvia_marketing_url ) ){
//
//            echo '<div class="marketing">
//                <a :href="' . esc_url( $sellvia_marketing_url ) . '" target="_blank" title="View marketing materials">
//                    <span class="marketing_icon"></span>
//                </a>
//            </div>';
//        }

    }
}
add_action( 'manage_product_posts_custom_column', 'sellvia_is_sellvia_product_column_value', 100, 2 );


function sellvia_is_sellvia_product_post_class( $classes, $class, $post_id ){

    if ( ! $post_id || get_post_type( $post_id ) !== 'product' ) {
        return $classes;
    }

    $sellvia_post_id = get_post_meta( $post_id, 'sellvia_post_id', true);

    if( !empty( $sellvia_post_id ) && is_numeric($sellvia_post_id) ){
        $classes[] = 'sellvia_product';
    }
    return $classes;
}
add_filter('post_class', 'sellvia_is_sellvia_product_post_class', 10,3);

function sellvia_custom_admin_css() {
    echo '<style>
    .column-is_sellvia_product {
      width: 30px;
      line-height: 0!important;
    } 
    .column-is_sellvia_product span{
        line-height: 0;
    }
    .column-is_sellvia_product .title{
        display: none;
    }    
  </style>';
}
add_action('admin_head', 'sellvia_custom_admin_css');

function sellvia_go_to_catalog_orders(){

    echo '<div id="sellvia_go_to_catalog_orders" style="display: inline-flex;align-items: center;margin-left: 10px;">';
        echo '<img width="14" height="14" src="'. SELLVIA_URL . '/assets/images/sellvia_icon_white.svg" />';
        echo '<a href="' . SELLVIA_ACCOUNT_PATH. 'me/orders" style="margin-left: 3px" target="_blank">' . __( 'Sellvia orders', 'sel').'</a>';
    echo '</div>';
}

function sellvia_go_to_catalog_orders_script(){
    printf(
        "<script>
            jQuery(function($){
            	$('.wp-header-end').before( $('#sellvia_go_to_catalog_orders') );
            })
        </script>"
    );
}

function sellvia_enqueue_front(){

    if( is_single() && get_post_type() == 'product' ) {
        wp_enqueue_script('sellvia_front_product_js', SELLVIA_URL . '/assets/js/front/product.min.js', ['jquery'], SELLVIA_VERSION, true);
        wp_enqueue_style('sellvia_front_product_css', SELLVIA_URL . '/assets/css/front/product.min.css', [], SELLVIA_VERSION );

        if( is_user_logged_in() && current_user_can('level_9') ){
            wp_enqueue_script( 'sellvia_main_script', SELLVIA_URL . '/assets/js/main.min.js', [], SELLVIA_VERSION  );
            wp_enqueue_style( 'sellvia_style', SELLVIA_URL . '/assets/css/style.min.css', [], SELLVIA_VERSION );
        }
    }else{
        if(is_front_page()){
            $theme  = wp_get_theme();
            $theme_name = $theme->get( 'Name' );
            if($theme_name == "Picasso"){
                wp_enqueue_script('sellvia_front_product_js', SELLVIA_URL . '/assets/js/front/product.min.js', ['jquery'], SELLVIA_VERSION, true);
                wp_enqueue_style('sellvia_front_product_css', SELLVIA_URL . '/assets/css/front/product.min.css', [], SELLVIA_VERSION );
            }
        }
    }
}
add_action( 'wp_enqueue_scripts', 'sellvia_enqueue_front' );

function sellvia_delete_product_images( $post_id, $post ){

    if( SELLVIA_PLUGIN == 'WOO' ){

        $attachments = get_children( [ 'post_type' => 'attachment', 'post_parent' => $post_id ] );
        if( $attachments ) {
            foreach( $attachments as $attachment ) wp_delete_attachment( $attachment->ID, true );
        }

    }
}

function sellvia_delete_comment( $comment_id, $comment ){

    $images = get_comment_meta( $comment_id, 'images', true );


    if ( $images === false ) {
        return;
    }

    $image_ids = maybe_unserialize( $images );

    if (!is_array( $image_ids ) || empty( $image_ids )) {
        return;
    }

    foreach ($image_ids as $key => $val) {
        if( is_numeric( $val ) ){
            wp_delete_attachment( $val, true );
        }
    }
}

function sellvia_plugin_mce_css( $mce_css ) {

    if ( ! empty( $mce_css ) )
        $mce_css .= ',';

    $mce_css .= set_url_scheme( SELLVIA_URL . '/assets/css/front/product_admin.min.css' );

    return $mce_css;
}

// Register new status
function sellvia_register_placed_order_status() {
    register_post_status( 'wc-sellvia-placed', array(
        'label'                     => 'Placed on Sellvia',
        'public'                    => true,
        'show_in_admin_status_list' => true,
        'show_in_admin_all_list'    => true,
        'exclude_from_search'       => false,
        'label_count'               => _n_noop( 'Waiting call (%s)', 'Waiting call (%s)' )
    ) );
}
add_action( 'init', 'sellvia_register_placed_order_status' );

// Add custom status to order status list
function sellvia_add_wait_call_to_order_statuses( $order_statuses ) {
    $new_order_statuses = array();
    foreach ( $order_statuses as $key => $status ) {
        $new_order_statuses[ $key ] = $status;
        if ( 'wc-processing' === $key ) {
            $new_order_statuses['wc-sellvia-placed'] = 'Placed on Sellvia';
        }
    }
    return $new_order_statuses;
}
add_filter( 'wc_order_statuses', 'sellvia_add_wait_call_to_order_statuses' );

function sellvia_set_bulk_footer() {

    global $typenow;

    if( $typenow != 'product' )
        return;

    ?>
    <script type="text/javascript">
        jQuery(document).ready(function ($) {
            $('<option>').val('sellvia_reset_product').text('<?php _e( 'Sellvia Reset', 'sel' ) ?>')
                .appendTo("select[name='action'], select[name='action2']");
        });
    </script>
    <?php
}
add_action( 'admin_footer-edit.php', 'sellvia_set_bulk_footer' );

function sellvia_reset_product_script(){

    $screen = get_current_screen();

    if( isset( $screen->id ) && $screen->id === 'edit-product' ) {
        wp_enqueue_script( 'sellvia_main_script', SELLVIA_URL . '/assets/js/main.min.js', [], SELLVIA_VERSION  );
        wp_enqueue_style( 'sellvia_style', SELLVIA_URL . '/assets/css/style.min.css', [], SELLVIA_VERSION );
    }
}
add_action('admin_head', 'sellvia_reset_product_script' );

add_action( 'before_woocommerce_init', function() {
    if ( class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
        \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', SELLVIA_PATH . 'sellvia.php', true );
    }
} );