<?php
/**
 * Author: Vitaly Kukin
 * Date: 08.02.2017
 * Time: 8:30
 */

/**
 * Output the variable product add to cart area.
 */
function woocommerce_variable_add_to_cart() {

    global $product;

    $product_id = $product->get_id();

    // Enqueue variation scripts
    wp_enqueue_script( 'wc-add-to-cart-variation' );

    // Get Available variations?
    $get_variations = true;//sizeof( $product->get_children() ) <= apply_filters( 'woocommerce_ajax_variation_threshold', 30, $product );

    $available_variations = $get_variations ? $product->get_available_variations() : false;
    $attributes           = $product->get_variation_attributes();
    //$selected_attributes  = $product->get_default_attributes();

    $attribute_keys = array_keys( $attributes );

    $attr_img_override = get_post_meta( $product_id, 'adsw-override-global-attribute-image', true );
    $attr_img    = get_post_meta( $product_id, 'adsw-attribute-image', true );
    $drop_attrib = get_post_meta( $product_id, 'adsw-attributes', true );
    $drop_attrib = ($drop_attrib) ? maybe_unserialize( $drop_attrib ) : [];
    $foo         = [];
    $foo_key     = '';
    $a           = 1;
    $i           = 0;

    foreach ( $attributes as $attribute => $val ) {

        if( ! $attr_img && $i == 0 ) {
            $foo_key = $attribute;
            $foo = $val;
        } elseif( $product_id . '-' . str_replace( 'pa_', '', $attribute, $a ) == $attr_img ) {
            $foo_key = $attribute;
            $foo = $val;
        }

        $i++;
    }

    if( count( $attributes ) && $foo_key == '' ) {
        $foo_key = key( $attributes );
        $foo = $attributes[ $foo_key ];
        $attr_img = $product_id . '-' . str_replace( 'pa_', '', $foo_key, $a );
    }

    if( ! $attr_img_override )
        $attr_img = $product_id . '-' . str_replace( 'pa_', '', $foo_key, $a );

    if( count( $foo ) == 0 ) {
        $foo = array_values( $attributes )[0];
    }

    $ex_attr = [];

    $available_attributes = [];

    foreach ( $available_variations as $var ) {

        $db_thumbnail = get_post_meta($var['variation_id'], '_thumbnail_id');
        $db_thumbnail_id = !empty($db_thumbnail) ? $db_thumbnail[0] : '';
        if (
            $var[ 'variation_is_visible' ] == 1 &&
            $var[ 'variation_is_active' ] == 1 &&
            $var[ 'is_purchasable' ] == 1 &&
            isset( $var[ 'attributes' ] ) &&
            count( $var[ 'attributes' ] ) > 0 ) {

            foreach ( $var[ 'attributes' ] as $k => $v ) {

                if( ! isset( $available_attributes[ $k ] ) || ! in_array( $v, $available_attributes[ $k ] ) ){
                    $available_attributes[ $k ][] = $v;
                }

                if ( $k == 'attribute_' . $foo_key && isset( $v ) && in_array( $v, $foo ) && $db_thumbnail_id !== "0" ) {

                    $ex_attr[ $v ] = [
                        'url' => isset( $var[ 'image' ][ 'thumb_src' ] ) ? $var[ 'image' ][ 'thumb_src' ] : false,
                        'image_id' => isset( $var[ 'image_id' ] ) ? $var[ 'image_id' ] : ''
                    ];

                }
            }
        }
    }

    $variations_json = wp_json_encode( $available_variations );
    $variations_attr = function_exists( 'wc_esc_json' ) ? wc_esc_json( $variations_json ) : _wp_specialchars( $variations_json, ENT_QUOTES, 'UTF-8', true );

    do_action( 'woocommerce_before_add_to_cart_form' ); ?>

    <form class="variations_form cart" method="post" enctype='multipart/form-data' data-product_id="<?php echo absint( $product->get_id() ); ?>" data-product_variations="<?php echo $variations_attr; // WPCS: XSS ok. ?>">
        <?php do_action( 'woocommerce_before_variations_form' ); ?>

        <?php if ( empty( $available_variations ) && false !== $available_variations ) : ?>
            <p class="stock out-of-stock"><?php _e( 'This product is currently out of stock and unavailable.', 'woocommerce' ); ?></p>
        <?php else : ?>
            <table class="variations" cellspacing="0">
                <tbody>

                <?php foreach ( $attributes as $attribute_name => $options ) : ?>

                    <?php $sanit_attribute_name = sanitize_title( $attribute_name ); ?>
                    <tr>
                        <td class="label"><label for="<?php echo $sanit_attribute_name; ?>"><?php echo wc_attribute_label( $attribute_name ); ?></label></td>
                        <td class="value">
                            <?php
                            $selected = isset( $_REQUEST[ 'attribute_' . $sanit_attribute_name ] ) ? wc_clean( urldecode( $_REQUEST[ 'attribute_' . $sanit_attribute_name ] ) ) : $product->get_variation_default_attribute( $attribute_name );
                            wc_dropdown_variation_attribute_options( [
                                'options'              => $options,
                                'attribute'            => $attribute_name,
                                'product'              => $product,
                                'selected'             => $selected,
                                'images'               => $ex_attr,
                                'image_key'            => $attr_img,
                                'drop_attrib'          => $drop_attrib,
                                'available_attributes' => $available_attributes,
                            ] );
                            echo end( $attribute_keys ) === $attribute_name ? apply_filters( 'woocommerce_reset_variations_link', '<a class="reset_variations" href="#">' . __( 'Clear', 'woocommerce' ) . '</a>' ) : '';
                            ?>
                        </td>
                    </tr>
                <?php endforeach; ?>
                </tbody>
            </table>

            <div class="single_variation_wrap">
                <?php
                /**
                 * woocommerce_before_single_variation Hook.
                 */
                do_action( 'woocommerce_before_single_variation' );

                /**
                 * woocommerce_single_variation hook. Used to output the cart button and placeholder for variation data.
                 * @since 2.4.0
                 * @hooked woocommerce_single_variation - 10 Empty div for variation data.
                 * @hooked woocommerce_single_variation_add_to_cart_button - 20 Qty and cart button.
                 */
                do_action( 'woocommerce_single_variation' );

                /**
                 * woocommerce_after_single_variation Hook.
                 */
                do_action( 'woocommerce_after_single_variation' );
                ?>
            </div>
        <?php endif; ?>

        <?php do_action( 'woocommerce_after_variations_form' ); ?>
    </form>

    <?php
    do_action( 'woocommerce_after_add_to_cart_form' );
}

/**
 * Output a list of variation attributes for use in the cart forms.
 *
 * @param array $args
 * @since 2.4.0
 */
function wc_dropdown_variation_attribute_options( $args = []) {

    $foo = [
        'images'      => isset( $args[ 'images' ] ) && is_array( $args[ 'images' ] ) ? $args[ 'images' ] : [],
        'image_key'   => isset( $args[ 'image_key' ] ) ? trim( $args[ 'image_key' ] ) : '',
        'drop_attrib' => isset( $args[ 'drop_attrib' ] ) && is_array( $args[ 'drop_attrib' ] ) ? $args[ 'drop_attrib' ] : [],
    ];

    $foo = sellvia_parse_args( $foo, $args );

    $nice_attributes = sellvia_get_nice_attributes_settings();
    $variations_color_apply = $nice_attributes[ 'variations_color_apply' ] == 1;

    $variation_image_size_css = '';

    if ( is_numeric( $nice_attributes[ 'custom_width' ] ) ) {
        $variation_image_size_css .= 'width: ' . $nice_attributes[ 'custom_width' ] . 'px;';
    }
    if ( is_numeric( $nice_attributes[ 'custom_height' ] ) ) {
        $variation_image_size_css .= ' height: ' . $nice_attributes[ 'custom_height' ] . 'px;';
    }


    $adsw_product_settings = get_option( 'adsw-product-settings' );
    $adsw_product_settings = maybe_unserialize( $adsw_product_settings );

    $args = wp_parse_args(
        apply_filters(
            'woocommerce_dropdown_variation_attribute_options_args', $args ),
        [
            'options'          => false,
            'attribute'        => false,
            'product'          => false,
            'selected' 	       => false,
            'name'             => '',
            'id'               => '',
            'class'            => '',
            'show_option_none' => __( 'Choose an option', 'woocommerce' ),
            'images'           => [],
            'image_key'        => '',
            'drop_attrib'      => []
        ]
    );


    $options              = $args[ 'options' ];
    $product              = $args[ 'product' ];
    $attribute            = $args[ 'attribute' ];
    $name                 = $args[ 'name' ] ? $args[ 'name' ] : 'attribute_' . sanitize_title( $attribute );
    $id                   = $args[ 'id' ] ? $args[ 'id' ] : sanitize_title( $attribute );
    $class                = $args[ 'class' ];
    $drop_attrib          = is_array( $args[ 'drop_attrib' ] ) ? $args[ 'drop_attrib' ] : [];
    $available_attributes = $args[ 'available_attributes' ];

    $product_id  = get_the_ID();

    $niceAttributesColor = get_post_meta( $product_id, 'adsw-niceAttributesColor', true );
    $niceAttributesColor =  ! empty( $niceAttributesColor ) ? $niceAttributesColor : [];
    $a = 1;
    $slug_key = str_replace( 'pa_', $product_id . '-', $attribute, $a );

    if ( empty( $options ) && ! empty( $product ) && ! empty( $attribute ) ) {
        $attributes = $product->get_variation_attributes();
        $options    = $attributes[ $attribute ];

    }

    $has_select = in_array( $attribute, $drop_attrib ) ?:false;

    $html = '<div class="adsw-attribute-option">';
    $sel = sprintf(
        '<select id="%s" class="%s" name="%s" data-attribute_name="attribute_%s" style="%s">',
        esc_attr( $id ), esc_attr( $class ), esc_attr( $name ), esc_attr( sanitize_title( $attribute ) ),
        $has_select ? '' : 'display:none'
    );

    if ( $args['show_option_none'] ) {
        $sel .= '<option value="">' . esc_html( $args['show_option_none'] ) . '</option>';
    }

    $attr_img_override = get_post_meta( $product_id, 'adsw-override-global-attribute-image', true );

    if($attr_img_override){
        $nice_attributes[ 'text_only' ] = 0;
    }
    if ( ! empty( $options ) ) {
        if ( $product && taxonomy_exists( $attribute ) ) {

            $terms = wc_get_product_terms( $product_id, $attribute, [ 'fields' => 'all' ] );

            foreach ( $terms as $term ) {

                if ( in_array( $term->slug, $options ) && in_array( $term->slug, $available_attributes['attribute_'.$attribute] ) ) {

                    $active   = sanitize_title( $args['selected'] ) == $term->slug ? 'active' : '';
                    $selected = $active != '' ? 'selected="selected"' : '';
                    $sel .= '<option value="' . esc_attr( $term->slug ) . '" ' . $selected . '>' . esc_html( apply_filters( 'woocommerce_variation_option_name', $term->name ) ) . '</option>';

                    if( ! $has_select ) {

                        if( $slug_key == $args['image_key'] && $nice_attributes[ 'text_only' ] != 1 ) {

                            if( isset( $args[ 'images' ][ $term->slug ] ) && sellvia_is_url( $args[ 'images' ][ $term->slug ][ 'url' ] ) ) {
                                $variation_image_style = $variation_image_size_css;
                                $border_radius = '';
                                if ( $nice_attributes[ 'round_variations' ] == 1 ) {
                                    $border_radius = ' border-radius: 50%;';
                                    $variation_image_style .= $border_radius;
                                }
                                if ( strlen( $variation_image_style ) ) {
                                    $variation_image_style = ' style="' . $variation_image_style . '"';
                                }

                                $img =  wp_get_attachment_image( $args['images'][ $term->slug ]['image_id'], 'shop_thumbnail', false, array(
                                    'title' => $term->name,
                                    'alt' => $term->name,
                                    'style' => $border_radius
                                ));
                                $thumbnail = '<i ' . $variation_image_style . '>' . $img . '</i>';

                                $html .= sprintf(
                                    '<span class="meta-item-img sku-set %s" data-value="%s" data-image_id="%s" title="%s">%s</span>',
                                    $active, esc_attr( $term->slug ),
                                    $args['images'][ $term->slug ][ 'image_id' ],
                                    esc_html( apply_filters( 'woocommerce_variation_option_name', $term->name ) ),
                                    $thumbnail
                                );
                            } else {
                                if ( $variations_color_apply && isset( $adsw_product_settings[ 'attr_options' ][ $attribute ][ 'product_color' ][ $term->term_id ] )
                                    && ! in_array( $attribute, $niceAttributesColor ) ) {

                                    $variation_image_style = 'background: ' . $adsw_product_settings[ 'attr_options' ][ $attribute ][ 'product_color' ][ $term->term_id ] . '; ' . $variation_image_size_css;
                                    if ( $nice_attributes[ 'round_variations' ] == 1 ) {
                                        $variation_image_style .= ' border-radius: 50%;';
                                    }
                                    $thumbnail = '<i style="' . $variation_image_style . '"></i>';
                                    $html .= sprintf(
                                        '<span class="meta-item-img sku-set %s" data-value="%s">%s</span>',
                                        $active, esc_attr( $term->slug ),
                                        $thumbnail
                                    );
                                } else {

                                    $html .= sprintf(
                                        '<span class="meta-item-text sku-set %s" data-value="%s">%s</span>',
                                        $active, esc_attr( $term->slug ),
                                        esc_html( apply_filters( 'woocommerce_variation_option_name', $term->name ) )
                                    );
                                }
                            }
                        } else {

                            $html .= sprintf(
                                '<span class="meta-item-text sku-set %s" data-value="%s">%s</span>',
                                $active, esc_attr( $term->slug ),
                                esc_html( apply_filters( 'woocommerce_variation_option_name', $term->name ) )
                            );
                        }
                    }
                }
            }
        } else {
            foreach ( $options as $option ) {

                $selected = sanitize_title( $args[ 'selected' ] ) === $args[ 'selected' ] ? selected( $args[ 'selected' ], sanitize_title( $option ), false ) : selected( $args[ 'selected' ], $option, false );
                $sel .= '<option value="' . esc_attr( $option ) . '" ' . $selected . '>' . esc_html( apply_filters( 'woocommerce_variation_option_name', $option ) ) . '</option>';

                $active = sanitize_title( $args[ 'selected' ] ) === $args[ 'selected' ] ? sanitize_title( $option ) : $args[ 'selected' ];

                $html .= sprintf(
                    '<span class="meta-item-text sku-set %s" data-value="%s">%s</span>',
                    $active == $option ? 'active' : '',
                    esc_attr( $option ),
                    esc_html( apply_filters( 'woocommerce_variation_option_name', $option ) )
                );
            }
        }
    }

    $sel .= '</select>';

    $html .= '</div>';
    $html .= $sel;

    echo apply_filters( 'woocommerce_dropdown_variation_attribute_options_html', $html, $args );
}
