<?php

/**
 * Api keys
 */
function sellvia_prepare_api_keys( $args ){

    $default = [
        'public_key' => '',
        'secret_key' => '',
    ];

    return sellvia_parse_args( $default, $args );
}

function sellvia_get_api_keys(){

    $keys = get_option('sellvia-api-keys');

    return sellvia_prepare_api_keys( $keys );
}

/**
 * CDN Images
 */
function sellvia_prepare_images_settings( $args = [] ){

    $default = [
        'webp'             => 0,
        'feature_images'   => 0,
        'gallery_images'   => 0,
        'variation_images' => 0,
        'content_images'   => 0,
        'reviews_images'   => 0,
    ];

    return array_map(
        function ( $i ){
            return intval( $i );
        },
        sellvia_parse_args( $default, $args )
    );
}

function sellvia_get_images_settings(){

    $settings = get_option( 'sellvia-images-settings', [] );

    return sellvia_prepare_images_settings( $settings );
}

/**
 * Size Charts
 */
function sellvia_prepare_size_chart_settings( $args = [] ){

    $default = [
        'enable' => 1,
    ];

    return sellvia_parse_args( $default, $args );
}

function sellvia_get_size_chart_settings(){

    $keys = get_option( 'sellvia-size-chart-settings', [] );

    return sellvia_prepare_size_chart_settings( $keys );
}

/**
 * Shipping badge
 */
function sellvia_prepare_shipping_badge_settings( $args = [] ){

    $default = [
        'enable'               => 1,
        'add_processing_time'  => 1,
        'processing_time'      => 1,
        'add_arrives_by'       => 1,
        'free'                 => 1,
        'main_color'           => '#018ed3',
        'primary_text_color'   => '#444444',
        'secondary_text_color' => '#27bf2f',
    ];

    return sellvia_parse_args( $default, $args );
}

function sellvia_get_shipping_badge_settings(){

    $settings = get_option('sellvia-shipping-badge-settings', []);

    return sellvia_prepare_shipping_badge_settings( $settings );
}

/**
 * Shipping
 */
function sellvia_prepare_shipping_settings( $args ){

    $default = [
        'enable' => 0,
        'time' => __ ( '1-3 business days', 'sel' ),
    ];

    return sellvia_parse_args( $default, $args );
}

function sellvia_get_shipping_settings(){

    $keys = get_option('sellvia_shipping_settings');

    return sellvia_prepare_shipping_settings( $keys );
}

/**
 * Price model
 */
function sellvia_prepare_price_model_settings( $args ){

    $default = [
        'enable' => 0,
        'value' => 100,
    ];

    return sellvia_parse_args( $default, $args );
}

function sellvia_get_price_model_settings(){

    $keys = get_option('sellvia_price_model_settings');

    return sellvia_prepare_price_model_settings( $keys );
}

/**
 * Fulfillment
 */
function sellvia_prepare_fulfillment_settings( $args ){

    $default = [
        'auto' => 0,
    ];

    return sellvia_parse_args( $default, $args );
}

function sellvia_get_fulfillment_settings(){

    $keys = get_option('sellvia_fulfillment_settings');

    return sellvia_prepare_fulfillment_settings( $keys );
}

/**
 * Reviews
 */
function sellvia_prepare_review_settings( $args ){

    $default = [
        'enable' => 1,
        'enable_popup' => 1,
    ];

    return sellvia_parse_args( $default, $args );
}

function sellvia_get_review_settings(){

    $keys = get_option('sellvia-review-settings');

    return sellvia_prepare_review_settings( $keys );
}

/**
 * Nice Attributes
 */
function sellvia_prepare_nice_attributes_settings( $args ){

    $default = [
        'enable' => 1,
        'text_only' => 0,
        'custom_css' => '',
        'custom_width' => 50,
        'custom_height' => 50,
        'round_variations' => 0,
        'variations_color_apply' => 0,
    ];

    return sellvia_parse_args( $default, $args );
}

function sellvia_get_nice_attributes_settings(){

    $keys = get_option('sellvia-nice-attributes-settings');

    return sellvia_prepare_nice_attributes_settings( $keys );
}

function sellvia_get_nice_attr( $var = '' ) {

    $args = maybe_unserialize( SELLVIA_NICE );

    if( ! $args ) return false;

    return isset( $args[ $var ] ) ? $args[ $var ] : false;
}