<?php
/**
 * Author: Vitaly Kukin
 * Date: 17.10.2016
 * Time: 9:27
 */

function sellvia_updparam() {
	
    return [
        'api_url'     => 'https://sr01.sellvia.com/addons/update?addon_key=sellvia',
        'plugin_slug' => 'sellvia',
    ];
}

/**
 * Take over the update check
 *
 * @param $checked_data
 * @param bool $upd
 * @param bool $wplang
 * @return mixed
 */
function sellvia_check_update( $checked_data, $upd = false, $wplang = false ) {

    global $wp_version;

    $foo         = sellvia_updparam();
    $api_url     = $foo['api_url'];
    $plugin_slug = $foo['plugin_slug'];

    if ( empty( $checked_data->checked ) )
        return $checked_data;

    $args = [
        'slug'    => $plugin_slug,
        'version' => SELLVIA_VERSION,
        'site'    => get_bloginfo('url'),
    ];
    
    $request_string = [
        'body'        => [
            'action'  => 'basic_check',
            'request' => serialize( $args )
        ],
        'user-agent'  => 'WordPress/' . $wp_version . '; ' . get_bloginfo('url'),
        'sslverify'   => false,
    ];

    $path_slug = $plugin_slug . '/' . $plugin_slug . '.php';
    // Start checking for an update
    $raw_response = wp_remote_post( $api_url, $request_string );
 
	if ( ! is_wp_error( $raw_response ) && ( $raw_response['response']['code'] == 200 ) ) {
        $response = maybe_unserialize( $raw_response['body'] );
    }

    if ( isset( $response ) && is_object( $response ) && ! empty( $response ) ) {
        $checked_data->response[ $path_slug ] = $response;
    } elseif( $upd && isset( $response ) && is_array( $response ) && ! empty( $response ) ) {
	    $checked_data->response[ $path_slug ] = $response;
    }

    return $checked_data;
}
add_filter( 'pre_set_site_transient_update_plugins', 'sellvia_check_update' );

/**
 * Take over the Plugin info screen
 *
 * @param $def
 * @param $action
 * @param $args
 *
 * @return bool|mixed|WP_Error
 */
function sellvia_plugin_api_call( $def, $action, $args ) {

    global $wp_version;

    $foo         = sellvia_updparam();
    $api_url     = $foo['api_url'];
    $plugin_slug = $foo['plugin_slug'];

    if ( ! isset( $args->slug ) || ( $args->slug != $plugin_slug ) ) {
        return $def;
    }
	
	$path_slug = $plugin_slug . '/' . $plugin_slug . '.php';
    
    // Get the current version
    $plugin_info     = get_site_transient('update_plugins');
    $current_version = $plugin_info->checked[ $path_slug ];
    $args->version   = $current_version;
    $args->site      = get_bloginfo('url');

    $request_string = [
        'body'        => [
            'action'  => $action,
            'request' => serialize( $args )
        ],
        'user-agent'  => 'WordPress/' . $wp_version . '; ' . get_bloginfo('url'),
        'sslverify'   => false,
    ];

    $request = wp_remote_post( $api_url, $request_string );

    if ( is_wp_error( $request ) ) {
        $res = new WP_Error(
        	'plugins_api_failed', '
        	An Unexpected HTTP Error occurred during the API request.' . $request->get_error_message()
        );
    } else {

        $res = maybe_unserialize( $request['body'] );

        if ( $res === false ) {
            $res = new WP_Error( 'plugins_api_failed', 'An unknown error occurred', $request['body'] );
        }
    }

    return $res;
}
add_filter( 'plugins_api', 'sellvia_plugin_api_call', 10, 3 );
