<?php

namespace sellvia\ImportProduct;

use sellvia\Media\MediaAds;
use sellvia\Shipping\ShippingAds;

class ImportProductAds{

    public $post_id = null;

    public $data = [];

    private $product = [];
    private $product_meta = [];
    private $ali_meta = [];

    private $mnsb_product_options = [];

    public $message = '';

    public $price_model = [];

    public function __construct( ){

        $this->initSettings();

        $this->initFilters();
    }

    protected function initSettings(){

        $this->price_model = sellvia_get_price_model_settings();
    }

    protected function initFilters(){

        add_filter( 'sellvia_import_price', function( $price ) {

            if( floatval( $this->price_model['value'] ) === 100 ) {
                return $price;
            }

            return sellvia_floatvalue( ( floatval( $this->price_model['value'] ) / 100 ) * floatval( $price ) );
        } );
    }

    public function importProduct( $data, $product_cat = [] ){

        global $wpdb;

        $pid  = $wpdb->get_var(
            "SELECT m.post_id FROM {$wpdb->prefix}postmeta m
                    WHERE m.meta_key = 'sellvia_post_id' AND m.meta_value = '{$data['sellvia_post_id']}'"
        );

        if (!empty($pid)) {
            return[
                'error' => 'This product already exist'
            ];
        }

        $args = [
            'comment_status' => 'open',
            'ping_status'    => 'closed',
            'post_author'    => 1,
            'post_status'    => $data['product_status'],
            'post_title'     => $data['product_title'],
            'post_content'   => $data['product_content'],
            'post_type'      => 'product',
            'post_excerpt'   => $data['product_short'] ? trim( htmlspecialchars_decode( $data['product_short'] ) ) : ''
        ];

        $response = wp_insert_post($args, true);

        wp_update_post([
            'ID' => $response,
            'post_status' => 'publish',
        ]);

        $this->post_id = $response;
        $this->data['post_id'] = $response;
        $this->data['imageUrl'] = $data['imageUrl'];
        $this->data['product_title'] = $data['product_title'];


        if(!empty($data['comments']) && is_array($data['comments'])){
            foreach ($data['comments'] as &$item){
                $item['done'] = 0;
            }
        }

        $this->data['comments'] = $data['comments'];

        $data['prices'] = self::preparePrices( $data['prices'] );

        $this->product = [
            'post_id' => $response,
            'imageUrl' => $data['imageUrl'],
            'price' => $data['prices']['price'],
            'priceMax' => $data['prices']['priceMax'],
            'salePrice' => $data['prices']['salePrice'],
            'salePriceMax' => $data['prices']['salePriceMax'],
            'discount' => $data['prices']['price'] > $data['prices']['salePrice'] ?
                sellvia_floatvalue(100 - 100 * $data['prices']['salePrice'] / $data['prices']['price']) : 0,
            'countReview' => 0,
            'packageType' => $data['packageType'],
            'quantity' => $data['quantity'],
            'evaluateScore' => 0,
            'promotionVolume' => $data['purchaseVolume'],
            'currency' => ADS_CUR_DEF,
        ];

        $if_ads_products = $wpdb->get_var(
            $wpdb->prepare("SELECT post_id FROM {$wpdb->prefix}ads_products WHERE post_id = %s", $response)
        );

        if (!empty($if_ads_products)) {
            $wpdb->update($wpdb->prefix . 'ads_products', $this->product, [ 'post_id' => $response ]);
        } else {
            $wpdb->insert($wpdb->prefix . 'ads_products', $this->product );
        }

        $sizes = [];

        if( isset( $data['size_chart'] ) && !empty( $data['size_chart'] ) ){

            $sizes = self::prepareSizes( $data['size_chart'] );

            if( !empty( $sizes ) ) {
                update_post_meta( $response, 'sellvia_sizeAttr', $data['size_chart'] );
            }
        }

        $this->product_meta = [
            'post_id'  => $response,
            'gallery'  => $data['gallery'],
            'shipping' => '',
            'lotNum'   => $data['lotNum'],
            'pack'     => $data['pack'],
            'sku'      => $data['sku'],
            'skuAttr'  => self::prepareSkuAttr( $data['skuAttr'] ),
            'sizeAttr' => maybe_serialize( $sizes ),
            'video'    => $data['video'],
        ];

        $if_product_meta = $wpdb->get_var(
            $wpdb->prepare("SELECT post_id FROM {$wpdb->prefix}ads_products_meta WHERE post_id = %s", $response)
        );

        if (!empty($if_product_meta)) {
            $wpdb->update($wpdb->prefix . 'ads_products_meta', $this->product_meta, [ 'post_id' => $response ]);
        } else {
            $wpdb->insert($wpdb->prefix . 'ads_products_meta', $this->product_meta );
        }

        $if_ali_meta = $wpdb->get_var(
            $wpdb->prepare("SELECT post_id FROM {$wpdb->prefix}ads_ali_meta WHERE post_id = %s", $response)
        );

        if( isset( $data['productId'] ) ) {
            $this->ali_meta = [
                'post_id'          => $response,
                'product_id'       => $data['productId'],
                'origPrice'        => $data['origPrice'],
                'origPriceMax'     => $data['origPriceMax'],
                'origSalePrice'    => $data['origSalePrice'],
                'origSalePriceMax' => $data['origSalePriceMax'],
                'productUrl'       => $data['productUrl'],
                'feedbackUrl'      => $data['feedbackUrl'],
                'storeUrl'         => $data['storeUrl'],
                'storeName'        => $data['storeName'],
                'storeRate'        => $data['storeRate'],
                'skuOriginaAttr'   => $data['skuOriginaAttr'],
                'skuOriginal'      => $data['skuOriginal'],
                'currencyCode'     => $data['currencyCode'],
                'needUpdate'       => 0,
            ];

            if (!empty($if_ali_meta)) {
                $wpdb->update($wpdb->prefix . 'ads_ali_meta', $this->ali_meta, [ 'post_id' => $response ]);
            } else {
                $wpdb->insert($wpdb->prefix . 'ads_ali_meta', $this->ali_meta );
            }
        }

        if (!empty($if_ali_meta)) {
            $wpdb->update($wpdb->prefix . 'ads_ali_meta', [
                'needUpdate'       => 0,
            ], [ 'post_id' => $response ]);
        }

        $attr = @unserialize($data['attributes']);
        sellvia_customQuery($attr, 'ads_attributes', $response);


        $this->data['images'] = maybe_unserialize( $data['images'] );
        $this->data['comments'] = $this->prepareComments( $this->data['comments'] );

        //Shipping
        $wh_shipping = isset( $data['wh_shipping'] ) && !empty( $data['wh_shipping'] ) ? $data['wh_shipping'] : [];
        ShippingAds::save_product_shipping( $response, maybe_unserialize( $data['skuAttr'] ), $wh_shipping );

        if( isset( $data['preset_categories'] ) && is_array( $data['preset_categories'] )){

            $preset_categories = [];

            foreach ( $data['preset_categories'] as $pr_cat ){
                $preset_categories[] = [
                    'term_id' => $pr_cat['id'],
                    'name' => $pr_cat['title'],
                    'slug' => $pr_cat['slug'],
                    'taxonomy' => 'product_cat',
                    'parent' => !empty( $pr_cat['parent_id'] ) ? $pr_cat['parent_id'] : 0,
                ];
            }

            $this->_taxonomy_install_type( 'product_cat', $preset_categories );
        }

        if( isset( $data['pr_categories'] ) && is_array( $data['pr_categories'] )){

            $pr_categories = [];

            foreach ( $data['pr_categories'] as $pr_cat ){
                $pr_categories[] = [
                    'term_id' => $pr_cat['id'],
                    'name' => $pr_cat['title'],
                    'slug' => $pr_cat['slug'],
                    'taxonomy' => 'product_cat',
                    'parent' => !empty( $pr_cat['parent_id'] ) ? $pr_cat['parent_id'] : 0,
                ];
            }

            $this->procced_terms( $this->data['post_id'], [ 'product_cat' => $pr_categories ] );
        }

        if( $product_cat ) {

            $koo = [];
            foreach( $product_cat as $k ) {
                $terms = $this->get_parents_terms( $k );

                if( $terms )
                    $koo = array_merge( $terms, $koo );
            }
            $koo = array_unique( $koo );

            wp_set_post_terms( $this->data['post_id'], $koo, 'product_cat' );
        }

        if( isset( $data['marketing_url'] ) && !empty( $data['marketing_url'] ) && sellvia_is_url( $data['marketing_url'] ) ){

            update_post_meta( $response, 'sellvia_marketing_url', $data['marketing_url'] );
        }

        //MNSB Plugin
        if( $wpdb->get_var( $wpdb->prepare( "SHOW TABLES LIKE %s", $wpdb->prefix.'mnsb_product_options' ) ) === $wpdb->prefix.'mnsb_product_options'
            && !empty( $data['ai'] ) ){

            $this->mnsb_product_options = [
                'post_id'      => $response,
                'main_keyword' => $data['ai']['main_keyword'],
                'keywords'     => $data['ai']['keywords'],
                'gallery'      => $data['ai']['gallery'],
            ];

            $if_mnsb_product_options = $wpdb->get_var(
                $wpdb->prepare("SELECT post_id FROM {$wpdb->prefix}mnsb_product_options WHERE post_id = %s", $response)
            );

            if ( !empty( $if_mnsb_product_options ) ) {
                $wpdb->update($wpdb->prefix . 'mnsb_product_options', $this->mnsb_product_options, [ 'post_id' => $response ]);
            } else {
                $wpdb->insert($wpdb->prefix . 'mnsb_product_options', $this->mnsb_product_options );
            }

        }else{

            $this->data['images'] = array_filter($this->data['images'], function ($it){
                return $it['place'] !== 'ai_gallery';
            } );
        }

        update_post_meta( $response, 'sellvia_post_id', $data['sellvia_post_id'] );
        update_post_meta( $response, 'sellvia_need_update', 1 );
    }

    public function process_images(){

        if( empty( $this->data['images'] ) || empty( $this->post_id ) ){
            return [];
        }

        global $wpdb;

        $media = new MediaAds();

        $options = $media->getOptions();

        $images_to_upload = [];

        $gallery = isset( $this->product_meta['gallery'] ) ? maybe_unserialize( $this->product_meta['gallery'] ) : [];

        $sku = isset( $this->product_meta['sku'] ) ? maybe_unserialize( $this->product_meta['sku'] ) : [];

        $ai_gallery = !empty( $this->mnsb_product_options['gallery'] ) ? maybe_unserialize( $this->mnsb_product_options['gallery'] ) : [];

        foreach ( $this->data['images'] as $image ) {

            if( $image['place'] == 'thumb' ) {

                if ($options['feature_images'] === 1) {

                    $images_to_upload[] = $image;

                } else {

                    $img_id = $media->create_external_attachment($this->post_id, $image['url']);

                    if ($img_id == false)
                        continue;

                    set_post_thumbnail($this->post_id, $img_id);

                    $wpdb->update($wpdb->prefix . 'ads_products',
                        array(
                            'imageUrl' => $image['url'],
                        ),
                        array('post_id' => $this->post_id)
                    );
                }
            } elseif( $image['place'] == 'gallery') {

                if ( $options['gallery_images'] === 1 ) {

                    $images_to_upload[] = $image;

                } else {

                    if ( $gallery && is_array( $gallery ) ) {

                        $img_id = $media->create_external_attachment($this->post_id, $image['url']);

                        if ($img_id == false)
                            continue;

                        foreach ($gallery as $k => $v) {
                            if ( $v == $image['url'] ) {
                                $gallery[$k] = $img_id;
                            }
                        }
                    }
                }

            } elseif ( $image['place'] == 'sku') {

                if ( $options['variation_images'] === 1 ) {

                    $images_to_upload[] = $image;

                } else {

                    if ($sku && is_array($sku)) {

                        $img_id = $media->create_external_attachment($this->post_id, $image['url']);

                        if ($img_id == false)
                            continue;

                        foreach ($sku as $k => &$v) {
                            if ($v['img'] == $image['url']) {
                                $v['img'] = $img_id;
                            }
                        }
                    }
                }
            } elseif ($image['place'] == 'content'){

                if( $options['content_images'] === 1 ){
                    $images_to_upload[] = $image;
                }

            } elseif ($image['place'] == 'short' ){

                if( $options['content_images'] === 1 ){
                    $images_to_upload[] = $image;
                }

            } elseif ( $image['place'] == 'ai_gallery' ){

//                if ( $options['gallery_images'] === 1 ) {

                $images_to_upload[] = $image;

//                } else {
//
//                    if ( $ai_gallery && is_array( $ai_gallery ) ) {
//
//                        $img_id = $media->create_external_attachment($this->post_id, $image['url']);
//
//                        if ($img_id == false)
//                            continue;
//
//                        foreach ($ai_gallery as $k => $v) {
//                            if ( $v == $image['url'] ) {
//                                $ai_gallery[$k] = $img_id;
//                            }
//                        }
//                    }
//                }
            }
        }

        if( isset( $this->product_meta['gallery'] ) ) {
            $wpdb->update(
                $wpdb->prefix . 'ads_products_meta',
                [
                    'gallery' => maybe_serialize($gallery)
                ],
                [
                    'post_id' => $this->post_id
                ]
            );
        }

        if( isset( $this->product_meta['sku'] ) ) {
            $wpdb->update(
                $wpdb->prefix . 'ads_products_meta',
                [
                    'sku' => maybe_serialize($sku)
                ],
                [
                    'post_id' => $this->post_id
                ]
            );
        }

        if( isset( $this->mnsb_product_options['gallery'] ) && !empty( $this->mnsb_product_options['gallery'] ) ) {
            $wpdb->update(
                $wpdb->prefix . 'mnsb_product_options',
                [
                    'gallery' => maybe_serialize($ai_gallery)
                ],
                [
                    'post_id' => $this->post_id
                ]
            );
        }

        return $images_to_upload;
    }

    //TODO переписать на новый импорт картинок
    public function resetProduct( $post_id, $data, $settings = [] ){

        global $wpdb;

        $data['images'] = maybe_unserialize( $data['images'] );

        $this->post_id = $post_id;
        $this->data['post_id'] = $post_id;

        $this->data['images'] = [];

        $textActivities = [];

        if( isset( $settings['featured'] ) && intval( $settings['featured'] ) === 1 ){

            $product = [
                'post_id' => $post_id,
                'imageUrl' => $data['imageUrl'],
            ];

            $wpdb->update($wpdb->prefix . 'ads_products', $product, [ 'post_id' => $post_id ]);

            $this->data['images'] = array_merge( $this->data['images'], array_filter( $data['images'], function ($it){
                return $it['place'] === 'thumb';
            } ) );

            $textActivities[] = __( 'featured image', 'sel');
        }

        if( isset( $settings['gallery'] ) && intval( $settings['gallery'] ) === 1 ){

            $product_meta = [
                'post_id' => $post_id,
                'gallery'  => $data['gallery'],
            ];

            $wpdb->update($wpdb->prefix . 'ads_products_meta', $product_meta, [ 'post_id' => $post_id ]);

            $this->data['images'] = array_merge( $this->data['images'], array_filter( $data['images'], function ($it){
                return $it['place'] === 'gallery';
            } ) );

            $textActivities[] = __( 'gallery', 'sel');
        }

        if( isset( $settings['description'] ) && intval( $settings['description'] ) === 1 ){

            $post = [
                'ID' => $post_id,
                'post_content' => $data['product_content'],
            ];

            $response = wp_update_post( $post );

            if( $response ) {
                $this->data['images'] = array_merge($this->data['images'], array_filter($data['images'], function ($it) {
                    return $it['place'] === 'content';
                }));
            }

            $textActivities[] = __( 'description', 'sel');
        }

        if( isset( $settings['short_description'] ) && intval( $settings['short_description'] ) === 1 ){

            $post = [
                'ID' => $post_id,
                'post_excerpt' => !empty( $data['product_short'] ) ? trim( htmlspecialchars_decode( $data['product_short'] ) ) : '',
            ];

            $response = wp_update_post( $post );

            if( $response ) {
                $this->data['images'] = array_merge( $this->data['images'], array_filter($data['images'], function ($it) {
                    return $it['place'] === 'short';
                }));
            }

            $textActivities[] = __( 'short', 'sel');
        }

        if( isset( $settings['reviews'] ) && intval( $settings['reviews'] ) === 1 ){

            $this->deleteComments( $post_id );

            $this->data['comments'] = $this->prepareComments( $data['comments'] );

            $textActivities[] = __( 'reviews', 'sel');
        }

        if( isset( $settings['title'] ) && intval( $settings['title'] ) === 1 ){

            $post = [
                'ID' => $post_id,
                'post_title' => $data['product_title'],
            ];

            $response = wp_update_post( $post );

            $textActivities[] = __( 'title', 'sel');
        }

        if( isset( $settings['variant'] ) && intval( $settings['variant'] ) === 1 ){

            $data['prices'] = self::preparePrices( $data['prices'] );

            $product = [
                'post_id' => $post_id,
                'price' => $data['prices']['price'],
                'priceMax' => $data['prices']['priceMax'],
                'salePrice' => $data['prices']['salePrice'],
                'salePriceMax' => $data['prices']['salePriceMax'],
                'discount' => $data['prices']['price'] > $data['prices']['salePrice'] ?
                    sellvia_floatvalue(100 - 100 * $data['prices']['salePrice'] / $data['prices']['price']) : 0,
                'quantity' => $data['quantity'],
                'currency' => ADS_CUR_DEF,
            ];

            $wpdb->update($wpdb->prefix . 'ads_products', $product, [ 'post_id' => $post_id ]);

            $product_meta = [
                'post_id'  => $post_id,
                'sku'      => $data['sku'],
                'skuAttr'  => self::prepareSkuAttr( $data['skuAttr'] ),
            ];

            $wpdb->update($wpdb->prefix . 'ads_products_meta', $product_meta, [ 'post_id' => $post_id ]);

            $this->data['images'] = array_merge( $this->data['images'], array_filter( $data['images'], function ($it){
                return $it['place'] === 'sku';
            } ) );

            $textActivities[] = __( 'variants', 'sel');
            $textActivities[] = __( 'stock and pricing', 'sel');
        }

        //Shipping
        $wh_shipping = isset( $data['wh_shipping'] ) && !empty( $data['wh_shipping'] ) ? $data['wh_shipping'] : [];
        ShippingAds::save_product_shipping( $post_id, maybe_unserialize( $data['skuAttr'] ), $wh_shipping );

        $plural1 = __( 'have', 'sel' );
        $plural2 = __( 'were', 'sel' );

        if( count( array_filter( $settings, function ( $setting ) { return $setting === 1; } ) ) == 1
            && ( $settings['featured'] == 1 || $settings['gallery'] == 1 || $settings['description'] == 1 ) ){
            $plural1 = __( 'has', 'sel' );
            $plural2 = __( 'was', 'sel' );
        }

        if( !empty($textActivities) ) {
            $this->message = __('Product', 'sel') . ' ' . implode(', ', $textActivities) . ' ' . $plural1 . __(' been reset.', 'sel');
            $textActivities = __('Product', 'sel') . ' ' . implode(', ', $textActivities) . ' ' . $plural2 . ' ' . __('reset.', 'sel');

            do_action('sellvia_log',
                [
                    'type' => 'reset',
                    'details' => $textActivities
                ],
                $textActivities,
                'product',
                'info',
                $post_id,
                null,
                ''
            );
        }else{
            $this->message = __('Check options to reset', 'sel');
        }

        return true;
    }

    private function _taxonomy_install_type( $taxonomy_type, $all_data ){

        if( !empty( $all_data ) ) {

            foreach ($all_data as $term_data) {

                $this->_process_term_data($taxonomy_type, $term_data);
            }
        }
    }

    private function _process_term_data($taxonomy_type, $term) {

        if(taxonomy_exists($taxonomy_type)) {
            $term_exists = term_exists( $term['slug'], $taxonomy_type );
            $term_id     = is_array( $term_exists ) ? $term_exists['term_id'] : $term_exists;
            if ( ! $term_id ) {
                if(!empty( $term['parent'] )){
                    // see if we have imported this yet?
                    $term['parent'] = $this->_imported_term_id( $term['parent'] );
                }

                $t = wp_insert_term( $term['name'], $taxonomy_type, $term );
                if ( ! is_wp_error( $t ) ) {
                    $term_id = $t['term_id'];
                    //do_action( 'wp_import_insert_term', $t, $term, $post_id, $post );
                } else {
                    // todo - error
                }
            }
            $this->_imported_term_id($term['term_id'], $term_id);
        }

    }

    public function procced_terms($post_id, $terms_array){
        if ( ! empty( $terms_array ) ) {

            $terms_to_set = array();

            foreach ( $terms_array as $term_slug => $terms ) {

                foreach($terms as $term) {

                    $taxonomy    =  $term['taxonomy'];

                    if(taxonomy_exists($taxonomy)) {

                        $term_exists = term_exists( $term['slug'], $taxonomy );
                        $term_id     = is_array( $term_exists ) ? $term_exists['term_id'] : $term_exists;
                        if ( ! $term_id ) {
                            if(!empty( $term['parent'] )){
                                // see if we have imported this yet?
                                $term['parent'] = $this->_imported_term_id($term['parent']);
                            }

                            $t = wp_insert_term( $term['name'], $taxonomy, $term );
                            if ( ! is_wp_error( $t ) ) {
                                $term_id = $t['term_id'];
                                //do_action( 'wp_import_insert_term', $t, $term, $post_id, $post );
                            } else {
                                // todo - error
                                continue;
                            }
                        }
                        $this->_imported_term_id($term['term_id'], $term_id);
                        $terms_to_set[ $taxonomy ][] = intval( $term_id );
                    }
                }
            }
            foreach ( $terms_to_set as $tax => $ids ) {
                wp_set_post_terms( $post_id, $ids, $tax );
            }
        }
    }

    private function _imported_term_id( $original_term_id , $new_term_id = false ){
        $terms = get_transient('sellvia_importtermids');
        if(!is_array($terms))$terms = [];
        if($new_term_id){
            $terms[$original_term_id] = $new_term_id;
            set_transient('sellvia_importtermids', $terms, 60 * 60 * 24 );
        }else if($original_term_id && isset($terms[$original_term_id])){
            return $terms[$original_term_id];
        }
        return false;
    }

    private function get_parents_terms( $term_id, $list = [] ) {

        $term = get_term_by( 'id', $term_id, 'product_cat' );

        if( ! $term )
            return count( $list ) > 0 ? $list : false;

        $list[] = $term_id;

        if( $term->parent != 0 )
            return $this->get_parents_terms( $term->parent, $list );

        return $list;
    }

    public function prepareComments( $comments = [] ){

        if( !empty($comments) ) {

            $date_now_timestamp = date_i18n('U');

            $comments_dates = array_map( function ($item) {
                return strtotime($item['comment_date']);
            }, $comments);

            $date_max = max($comments_dates);

            $delta = $date_now_timestamp - $date_max;

            $comments = array_map( function ($item) use ($delta) {
                $new_date = date_i18n('Y-m-d H:i:s', strtotime($item['comment_date']) + $delta);
                $item['comment_date'] = $new_date;
                $item['comment_date_gtm'] = $new_date;
                return $item;
            }, $comments);
        }

        return $comments;
    }

    private function deleteComments( $post_id = NULL ){

        if(empty($post_id)){
            return;
        }

        if ( $comments = get_comments( array( 'post_id' => $post_id ) ) ) {
            foreach ( $comments as $comment ) {
                wp_delete_comment( $comment->comment_ID, true );
            }
        }
    }

    public static function preparePrices( $prices ) {

        if( !empty( $prices ) ) {
            foreach ( $prices as &$price ) {
                $price = apply_filters('sellvia_import_price', $price);
            }
        }

        return $prices;
    }

    public static function prepareSkuAttr( $skuAttr ) {

        $skuAttr = maybe_unserialize( $skuAttr );

        if( !empty( $skuAttr ) ) {
            foreach ( $skuAttr as &$item ) {
                $item['price'] = apply_filters('sellvia_import_price', $item['price'] );
                $item['salePrice'] = apply_filters('sellvia_import_price',  $item['salePrice'] );
            }
        }

        return maybe_serialize( $skuAttr );
    }

    public static function prepareSizes( $data ){

        if( empty($data) || empty( $data['measures'] ) || empty( $data['sizes'] ) || empty( $data['value'] ) ) return false;

        $sizes = [
            'title' => [
                0 => __( "Size", "sel" )
            ],
            'list' => []
        ];

        foreach ( $data['sizes'] as $s_key => $s_val ){
            $size = [];

            $size[0] = $s_val;

            $count = 1;

            foreach ( $data['measures'] as $m_key => $m_val){
                $size[$count] = $data['value'][$m_key][$s_key] ?? '';
                $sizes['title'][$count] = $m_val;
                $count++;
            }

            $sizes['list'][] = $size;
        }

        return $sizes;
    }
}
