<?php

namespace sellvia\ImportProduct;

use sellvia\Media\MediaWoo;
use sellvia\Shipping\ShippingWoo;

class ImportProductWoo{

    public $data = [];

    public  $post_id = false;
    private $post;
    private $product;
    private $attribute    = [];
    private $meta         = [];
    private $alimeta      = [];
    private $params       = [];
    private $old_attributes;

    private $mnsb_product_options = [];

    public $images = [
        'gallery'    => [],
        'sku' => [],
        'content' => [],
        'short' => [],
    ];

    private $uploadedImages = [];

    private $uploadedImagesLinkKey = [];

    public $message = '';

    public $price_model = [];

    public function __construct( ){

        $this->initSettings();

        $this->initFilters();
    }

    protected function initSettings(){

        $this->price_model = sellvia_get_price_model_settings();
    }

    protected function initFilters(){

        add_filter( 'sellvia_import_price', function( $price ) {

            if( floatval( $this->price_model['value'] ) === 100 ) {
                return $price;
            }

            return sellvia_floatvalue( ( floatval( $this->price_model['value'] ) / 100 ) * floatval( $price ) );
        } );
    }

    public  function importProduct( $data, $product_cat = [] ){

        $this->data = $data;

        global $wpdb;

        $pid  = $wpdb->get_var(
            "SELECT m.post_id FROM {$wpdb->prefix}postmeta m
                    WHERE m.meta_key = 'sellvia_post_id' AND m.meta_value = '{$data['sellvia_post_id']}'"
        );

        if (!empty($pid)) {
            return[
                'error' => 'This product already exist'
            ];
        }

        $status = $this->prepare();

        if ( is_array( $status ) ) {
            return[
                'error' => 'Something went wrong on woo product prepare'
            ];
        }

        $this->publish();

        if ( is_array( $this->post_id ) ) {
            return[
                'error' => 'Something went wrong on woo product publish'
            ];
        }

        if( isset( $data['preset_categories'] ) && is_array( $data['preset_categories'] )){

            $preset_categories = [];

            foreach ( $data['preset_categories'] as $pr_cat ){
                $preset_categories[] = [
                    'term_id' => $pr_cat['id'],
                    'name' => $pr_cat['title'],
                    'slug' => $pr_cat['slug'],
                    'taxonomy' => 'product_cat',
                    'parent' => !empty( $pr_cat['parent_id'] ) ? $pr_cat['parent_id'] : 0,
                ];
            }

            $this->_taxonomy_install_type( 'product_cat', $preset_categories );
        }

        if( isset( $data['pr_categories'] ) && is_array( $data['pr_categories'] )){

            $pr_categories = [];

            foreach ( $data['pr_categories'] as $pr_cat ){
                $pr_categories[] = [
                    'term_id' => $pr_cat['id'],
                    'name' => $pr_cat['title'],
                    'slug' => $pr_cat['slug'],
                    'taxonomy' => 'product_cat',
                    'parent' => !empty( $pr_cat['parent_id'] ) ? $pr_cat['parent_id'] : 0,
                ];
            }
            $this->procced_terms( $this->post_id, [ 'product_cat' => $pr_categories ] );
        }

        if( $this->post_id && !empty( $product_cat ) ) {

            $koo = [];
            foreach( $product_cat as $k ) {
                $terms = $this->get_parents_terms( $k );

                if( $terms )
                    $koo = array_merge( $terms, $koo );
            }
            $koo = array_unique( $koo );

            wp_set_post_terms( $this->post_id, $koo, 'product_cat' );
        }
    }

    public function prepare(  $action = 'import' ){

        $this->action = $action;

        $this->post[ 'post_title' ]   =  $this->data[ 'product_title' ];
        $this->post[ 'post_content' ] =  trim( htmlspecialchars_decode( $this->data[ 'product_content' ] ) );

        $this->post['post_excerpt'] = !empty( $this->data['product_short'] ) ? trim( htmlspecialchars_decode( $this->data['product_short'] ) ) : '';

        $this->product[ 'quantity' ]     = intval( $this->data[ 'quantity' ] );
        $this->product[ 'package' ]      = $this->preparePackaging( $this->data[ 'pack' ] );
        $this->product[ 'sizeAttr' ]     = isset( $this->data[ 'sizeAttr' ] ) ? $this->filterSizeAttr( $this->data[ 'sizeAttr' ] ) : [];

        $this->data['prices'] = self::preparePrices( $this->data['prices'] );

        $this->preparePostMeta(
            [
                'quantity' => $this->product[ 'quantity' ],
                'orders'   => intval( $this->data[ 'purchaseVolume' ] ),
            ],
            [
                'price'     => $this->data['prices'][ 'price' ],
                'salePrice' => $this->data['prices'][ 'salePrice' ]
            ]
        );

        if( isset( $this->data[ 'productId' ] ) ){

            $this->alimeta[ 'product_id' ]       = sellvia_integer( $this->data[ 'productId' ] );
            $this->alimeta[ 'origPrice' ]        = $this->data[ 'origPrice' ];
            $this->alimeta[ 'origPriceMax' ]     = $this->data[ 'origPriceMax' ];
            $this->alimeta[ 'origSalePrice' ]    = $this->data[ 'origSalePrice' ];
            $this->alimeta[ 'origSalePriceMax' ] = $this->data[ 'origSalePriceMax' ];
            $this->alimeta[ 'productUrl' ]       = esc_url( $this->data[ 'productUrl' ] );
            $this->alimeta[ 'feedbackUrl' ]      = esc_url( $this->data[ 'feedbackUrl' ] );
            $this->alimeta[ 'skuOriginaAttr' ]   = $this->data[ 'skuOriginaAttr' ];
            $this->alimeta[ 'skuOriginal' ]      = $this->data[ 'skuOriginal' ];

            $this->alimeta[ 'storeUrl' ]         = esc_url( $this->data[ 'storeUrl' ] );
            $this->alimeta[ 'storeName' ]        = esc_attr( $this->data[ 'storeName' ] );
            $this->alimeta[ 'storeRate' ]        = esc_attr( $this->data[ 'storeRate' ] );
            $this->alimeta[ 'currencyCode' ]     = $this->data['currencyCode'] ?? 'USD';
        }

        $this->alimeta[ 'needUpdate' ]       = 0;

        $this->product[ 'sku' ] = maybe_unserialize( $this->data[ 'sku' ] );

        $this->data['skuAttr'] = self::prepareSkuAttr( $this->data['skuAttr'] );

        $skuAttr = maybe_unserialize( $this->data[ 'skuAttr' ] );
        $skuAttr_meta = maybe_unserialize( $this->data[ 'skuAttr_meta' ] );

        if( !empty( $skuAttr ) && is_array( $skuAttr ) && count( $skuAttr ) > 0 ){

            foreach ( $skuAttr as $key => $val ){

                $this->product[ 'skuAttr' ][ $key ] = $this->prepareVariationMeta( $val, $val[ 'quantity' ] , $key, $skuAttr_meta[ $key ]['sku_number'] ?? '' );

                $koo      = explode( ';', $key );

                foreach( $koo as $k ) {
                    if( isset( $this->product[ 'sku' ][ $k ] ) ) {

                        $thumb = $this->product[ 'sku' ][ $k ][ 'img' ];

                        if( sellvia_is_url( $thumb ) ) {

                            if( array_key_exists( $thumb, $this->uploadedImagesLinkKey ) ){
                                $thumb = $this->uploadedImagesLinkKey[ $thumb ];
                                $args[ $key ][ '_thumbnail_id' ] = $thumb;
                            }

                            $this->images[ 'sku' ][ $key ] = $thumb;
                        }
                    }
                }

            }
        } else {
            $this->product[ 'skuAttr' ] = [];
        }

        $params = isset( $this->data[ 'attributes' ] ) && !empty( $this->data[ 'attributes' ] ) ? maybe_unserialize( $this->data[ 'attributes' ] ) : [] ; // item specifics

        if( !empty( $params ) && is_array( $params ) && count( $params ) > 0 ){
            foreach ( $params as $param ) {
                if( isset( $param[ 'attr_name' ] ) && isset( $param[ 'attr_value' ] ) )
                    $this->params[] = [
                        'name'  => $param[ 'attr_name' ],
                        'value' => $param[ 'attr_value' ]
                    ];
            }
        }
        $this->images['thumb'] = $this->data['imageUrl'];
        $this->images['gallery'] = maybe_unserialize( $this->data['gallery'] );
        $this->data['comments'] = $this->prepareComments( $this->data['comments'] );

        return true;
    }

    private function preparePackaging( $data ) {

        $foo = [
            '_weight' => '',
            '_length' => '',
            '_width'  => '',
            '_height' => '',
            '_sku'    => $this->data[ 'sellvia_post_id' ],
        ];

        $weight = isset( $data[ 'packaging' ][ 'weight' ] ) ? $data[ 'packaging' ][ 'weight' ] : 0;
        $size   = isset( $data[ 'packaging' ][ 'size' ] ) ? $data[ 'packaging' ][ 'size' ] : 0;

        $default_w = [
            'kg'  => 1,
            'g'   => 1000,
            'lbs' => 0.453592,
            'oz'  => 0.0283495
        ];

        $weight_woo = get_option( 'woocommerce_weight_unit', 'kg' );
        if( isset( $default_w[ $weight_woo ] ) )
            $weight_woo = 'kg';

        $default_d = [
            'cm' => 1,
            'm'  => 0.01,
            'mm' => 0.001,
            'in' => 0.393701,
            'yd' => 0.0109361
        ];

        $dimension_woo = get_option( 'woocommerce_dimension_unit', 'cm' );
        if( isset( $default_d[ $dimension_woo ] ) )
            $dimension_woo = 'cm';

        if( $weight != 0 ) {
            $_w = explode( ' ', $weight );
            $_w = sellvia_floatvalue( $_w[ 0 ] );
            $foo[ '_weight' ] = round( $_w * $default_w[ $weight_woo ], 2 );
        }

        if( $size != 0 ) {
            $size = explode( ' (', $size );
            $size = trim( $size[0] );
            $size = explode( ' x ', $size );

            $foo['_length'] = round( sellvia_integer( $size[ 0 ] ) * $default_d[ $dimension_woo ], 2 );
            $foo['_width']  = round( sellvia_integer( $size[ 1 ] ) * $default_d[ $dimension_woo ], 2 );
            $foo['_height'] = round( sellvia_integer( $size[ 2 ] ) * $default_d[ $dimension_woo ], 2 );
        }

        return $foo;
    }

    private function filterSizeAttr($sizeAttr){

        if(!isset($sizeAttr['title']) || !isset($sizeAttr['list'])){
            return [];
        }

        $sizeAttr['title'] = array_map( function( $i ) { return esc_attr($i);}, $sizeAttr['title']);
        $sizeAttr['list'] = array_map( function( $i ) { return array_map( function($x){ return esc_attr($x); }, $i);}, $sizeAttr['list']);

        return $sizeAttr;
    }

    private function preparePostMeta( $data, $prices ) {

        $regular_price = $prices['price'] == 0 ? $prices['salePrice'] : $prices['price'];
        $sale_price = ! empty( $prices[ 'salePrice' ] ) && $prices[ 'price' ] > $prices[ 'salePrice' ] ? $prices[ 'salePrice' ] : '';

        $meta = [
            '_regular_price'  => $regular_price,
            '_sale_price'     => $sale_price,
            '_price'          => $sale_price != '' ? $sale_price : $regular_price,
            '_stock'          => intval( sellvia_floatvalue( $data['quantity'] ) ), //общее количество товара с учётом всех вариаций
            '_stock_status'   => intval( sellvia_floatvalue( $data['quantity'] ) ) > 0 ? 'instock' : 'outofstock',
            '_height'         => $this->product[ 'package' ]['_height'],
            '_length'         => $this->product[ 'package' ]['_length'],
            '_width'          => $this->product[ 'package' ]['_width'],
            '_weight'         => $this->product[ 'package' ]['_weight'],
            '_sku'            => $this->product[ 'package' ]['_sku'],
            'total_sales'     => intval( sellvia_floatvalue( $data[ 'orders' ] ) ),
            'adswSizeAttr'    => $this->product[ 'sizeAttr' ],
            'sellvia_post_id' => $this->data[ 'sellvia_post_id' ],
            'sellvia_need_update' => 1,
        ];

        if( isset( $this->data['size_chart'] ) && !empty( $this->data['size_chart'] ) ){

            $sizes = ImportProductAds::prepareSizes( $this->data['size_chart'] );

            if( $sizes ) {
                $meta['adswSizeAttr'] = $sizes;
                $meta['sellvia_sizeAttr'] = $this->data['size_chart'];
            }
        }

        if( isset( $this->data[ 'marketing_url'] ) && !empty( $this->data['marketing_url'] ) && sellvia_is_url( $this->data['marketing_url'] ) ){
            $meta[ 'sellvia_marketing_url' ]  = $this->data['marketing_url'];
        }

        $this->meta =  $this->attachStaticPostMeta( $meta );
    }

    private function attachStaticPostMeta( $meta = [] ) {

        $static = [
            '_backorders'            => 'no',  //не продавать если сток 0
            '_crosssell_ids'         => serialize([]),
            '_upsell_ids'            => serialize([]),
            '_download_expiry'       => -1,
            '_download_limit'        => -1,
            '_downloadable'  		 => 'no',
            '_featured'      		 => 'no',
            '_manage_stock'          => 'yes',  //отнимать сток при покупке
            '_sale_price_dates_from' => '',
            '_sale_price_dates_to'   => '',
            '_sold_individually'     => 'no',
            '_virtual'       		 => 'no',
            '_wc_review_count'		 => 0,
            '_wc_average_rating'     => 0,
            '_wc_rating_count'		 => serialize([]),
            '_visibility'    		 => 'visible',
            '_purchase_note' 		 => '',
            '_product_version'       => '7.7.0',
            '_tax_class'             => '',
            '_tax_status'            => 'taxable',

        ];

        return array_merge( $meta, $static );
    }

    private function prepareVariationMeta( $prices, $quantity = 0, $key_skuAttr, $sku = '' ){
;
        $regular_price = $prices['price'] == 0 ? $prices['salePrice'] : $prices['price'];
        $sale_price = ! empty( $prices[ 'salePrice' ] ) && $prices[ 'price' ] > $prices[ 'salePrice' ] ? $prices[ 'salePrice' ] : '';

        $meta = [
            '_regular_price' => $regular_price,
            '_sale_price'    => $sale_price,
            '_price'         => $sale_price != '' ? $sale_price : $regular_price,
            '_stock'         => $quantity,
            '_stock_status'  => $quantity > 0 ? 'instock' : 'outofstock',
            '_sku'           => $sku
        ];

        $meta[ 'adswSKU' ] = $key_skuAttr;

        return $this->attachStaticVariationMeta( $meta );

    }

    private function attachStaticVariationMeta( $meta ) {

        $static = [
            '_sale_price_dates_from' => '',
            '_sale_price_dates_to'   => '',
            '_thumbnail_id'          => '',
            '_virtual'               => 'no',
            '_weight'                => $this->product[ 'package' ][ '_weight' ],
            '_length'                => $this->product[ 'package' ][ '_length' ],
            '_width'                 => $this->product[ 'package' ][ '_width' ],
            '_height'                => $this->product[ 'package' ][ '_height' ],
            '_backorders'            => 'no',
            '_downloadable'          => 'no',
            '_download_expiry'       => -1,
            '_download_limit'        => -1,
            '_variation_description' => '',
            '_manage_stock'          => 'yes',
            '_product_version'       => '7.7.0',
            '_sold_individually'     => 'no',
            '_tax_class'             => 'parent',
            '_tax_status'            => 'taxable',
        ];

        return array_merge( $meta, $static );
    }

    public function publish() {

        global $wpdb;

        $post_id = $this->post();

        if( is_array( $post_id ) ) return $post_id;

        //MNSB Plugin
        if( $this->post_id
            && $wpdb->get_var( $wpdb->prepare( "SHOW TABLES LIKE %s", $wpdb->prefix.'mnsb_product_options' ) ) === $wpdb->prefix.'mnsb_product_options'
            && !empty( $this->data['ai'] )
        ){

            $this->mnsb_product_options = [
                'post_id'      => $this->post_id,
                'main_keyword' => $this->data['ai']['main_keyword'],
                'keywords'     => $this->data['ai']['keywords'],
                'gallery'      => $this->data['ai']['gallery'],
            ];

            $if_mnsb_product_options = $wpdb->get_var(
                $wpdb->prepare("SELECT post_id FROM {$wpdb->prefix}mnsb_product_options WHERE post_id = %s", $this->post_id)
            );

            if ( !empty( $if_mnsb_product_options ) ) {
                $wpdb->update($wpdb->prefix . 'mnsb_product_options', $this->mnsb_product_options, [ 'post_id' => $this->post_id ]);
            } else {
                $wpdb->insert($wpdb->prefix . 'mnsb_product_options', $this->mnsb_product_options );
            }

            $this->images['ai_gallery'] = maybe_unserialize( $this->data['ai']['gallery'] );
        }

        $this->images = $this->prepareImages( $post_id, $this->images );

        $this->images = array_merge( $this->images, $this->linkContentImages() );

        $this->images = array_merge( $this->images, $this->linkShortImages() );

        $this->updatePostMeta($this->meta);

        if( !empty( $this->alimeta ) ) {
            // создаём запись в таблице ali_meta
            $this->updateAliMeta();
        }

        if( isset( $this->product[ 'sku' ] ) && !empty( $this->product[ 'sku' ] ) && isset( $this->product[ 'skuAttr' ] ) && !empty( $this->product[ 'skuAttr' ] ) ) {

            $this->saveSKU();

            $this->saveVariations();

            $this->variationAttributes( $this->attribute );

            $this->setProductType( 'variable', 'product_type' );

            \WC_Product_Variable::sync( $this->post_id );

        } else {
            $this->setProductType( 'simple', 'product_type' );
        }

        $this->setProductMetaLookup( $this->meta, $this->post_id );
        wc_delete_product_transients( $this->post_id );

        $this->productAttributes(); //item specifics

        //Shipping
        $wh_shipping = isset( $this->data['wh_shipping'] ) && !empty( $this->data['wh_shipping'] ) ? $this->data['wh_shipping'] : [];
        ShippingWoo::save_product_shipping( $this->post_id, maybe_unserialize( $this->data['skuAttr'] ), $wh_shipping );
    }

    private function post() {

        global $user_ID;

        $args = [
            'comment_status' => 'open',
            'ping_status' 	 => 'closed',
            'post_author' 	 => $user_ID,
            'post_status' 	 => $this->data['product_status'],
            'post_title' 	 => $this->post[ 'post_title' ],
            'post_content'	 => $this->post[ 'post_content' ],
            'post_type' 	 => 'product',
            'post_excerpt'   => $this->post['post_excerpt'],
        ];

        $response = wp_insert_post( $args, true );

        wp_update_post([
            'ID' => $response,
            'post_status' => 'publish',
        ]);

        if( is_wp_error( $response ) ) return [ 'error' => $response->get_error_message() ];

        $this->setPostId( $response );

        return $response;
    }

    protected function updatePostMeta( $args, $post_id = false ) {

        if( ! $post_id )
            $post_id = $this->post_id;

        if ( ! $args || empty( $args ) ) {
            return false;
        }

        foreach( $args as $key => $val ) {
            update_post_meta( $post_id, $key, $val );
        }

        return true;
    }

    private function updateAliMeta(){

        if ( ! $this->alimeta ) {
            return false;
        }

        global $wpdb;

        $post_id = $wpdb->get_var(
            $wpdb->prepare(
                "SELECT post_id FROM {$wpdb->prefix}adsw_ali_meta WHERE post_id = %d",
                $this->post_id
            )
        );

        if( ! empty( $post_id ) ){

            $wpdb->update(
                $wpdb->prefix . 'adsw_ali_meta',
                $this->alimeta,
                [ 'post_id' => $this->post_id ]
            );
        } else {
            $this->alimeta['post_id'] = $this->post_id;
            $wpdb->insert(
                $wpdb->prefix . 'adsw_ali_meta',
                $this->alimeta
            );
        }
    }

    private function saveSKU() {

        if( !isset( $this->product[ 'sku' ] ) || !is_array( $this->product[ 'sku' ] ) ) return false;

        $attrimg = false;

        foreach ( $this->product[ 'sku' ] as $key => $val ) {

            $taxonomy = $this->addAttribute( $val[ 'prop_title' ] );

            $slug = isset( $val[ 'make_unique' ] ) && $val[ 'make_unique' ] ? sanitize_title( $val[ 'title' ] ) . '-' . $val[ 'sku_id' ] : sanitize_title( $val[ 'title' ] );

            $term = $this->updateTerm( $val[ 'title' ], $slug, $taxonomy );

            if( ! $attrimg ) {

                if( sellvia_is_url( $val[ 'img' ] ) ) {

                    $attrimg = true;

                    update_post_meta( $this->post_id, 'adsw-attribute-image', $this->post_id . '-' . str_replace ( 'pa_', '', $taxonomy ) );
                }
            }

            if( ! isset( $term[ 'error' ] ) )
                wp_set_post_terms( $this->post_id, [ $term ], $taxonomy, true );
            else
                error_log( print_r( $term, true ) );

            $term = get_term_by( 'id', $term, $taxonomy );

            if( $term && ! is_wp_error( $term ) )
                $slug = $term->slug;

            foreach ( $this->product[ 'skuAttr' ] as $k => &$v ){

                $keys = explode( ';', $k );

                if( in_array( $key, $keys ) ){
                    $v[ 'attribute_' . $taxonomy ] = $slug;
                }
            }

            if( ! in_array( $taxonomy, $this->attribute ) ) {

                $this->attribute[] = $taxonomy;
            }
        }
    }

    private function saveVariations(){

        $i = 0;

        foreach ( $this->product[ 'skuAttr' ] as $key => $val ){

            $i++;

            $id = wp_insert_post( [
                'post_title'    => 'Variation #' . $i . ' of ' . esc_attr( strip_tags( $this->post[ 'post_title' ] ) ),
                'post_name'     => 'product-' . $this->post_id . '-variation-' . $i,
                'post_status'   => 'publish',
                'post_parent'   => $this->post_id,
                'post_type'     => 'product_variation',
                'guid'          =>  home_url() . '/?product_variation=product-' . $this->post_id . '-variation-' . $i
            ] );

            $this->updatePostMeta( $val, $id );
            $this->setProductMetaLookup( $val, $id );
        }
    }

    private function variationAttributes( $attributes, $visible = 0 ) {

        $i = 0;

        $product_attributes = [];

        foreach ( $attributes as $attribute ) {

            $product_attributes[ $attribute ] = [
                'name'         => $attribute, // set attribute name
                'value'        => '', // set attribute value
                'position'     => $i,
                'is_visible'   => $visible,
                'is_variation' => 1,
                'is_taxonomy'  => 1
            ];

            $i++;
        }

        update_post_meta( $this->post_id, '_product_attributes', $product_attributes );
    }

    private function productAttributes() {

        if( !empty( $this->params ) && is_array( $this->params ) ) {

            $product_attributes = [];

            $attrib = get_post_meta( $this->post_id, '_product_attributes', true );

            if( $attrib ) $product_attributes = $attrib;

            $i = count( $product_attributes );

            foreach( $this->params as $key => $val ) {

                $slug = sanitize_title( $val['name'] );
                $product_attributes[ $slug ] = [
                    'name'         => str_replace( ':', '', $val[ 'name' ] ), // set attribute name
                    'value'        => $val[ 'value' ], // set attribute value
                    'position'     => $i,
                    'is_visible'   => 1,
                    'is_variation' => 0,
                    'is_taxonomy'  => 0
                ];

                $i++;
            }

            update_post_meta( $this->post_id, '_product_attributes', $product_attributes );
        }
    }

    private function setProductMetaLookup( $args, $post_id = false ){

        global $woocommerce;

        if( version_compare( $woocommerce->version, '3.6.0', "<" ) )
            return false;

        if( ! $post_id )
            return false;


        if ( ! $args || empty( $args ) ) {
            return false;
        }

        global $wpdb;

        $table = $wpdb->prefix . 'wc_product_meta_lookup';

        $update_data = [
            'product_id'     => $post_id,
            'sku'            => $args[ '_sku' ],
            'virtual'        => isset( $args[ '_virtual' ] ) && 'yes' === $args[ '_virtual' ] ? 1 : 0,
            'downloadable'   => isset( $args[ '_downloadable' ] ) && 'yes' === $args[ '_downloadable' ] ? 1 : 0,
            'min_price'      => $args[ '_sale_price' ] != '' ? $args[ '_sale_price' ] : $args[ '_regular_price' ],
            'max_price'      => $args[ '_sale_price' ] != '' ? $args[ '_sale_price' ] : $args[ '_regular_price' ],
            'onsale'         => $args[ '_sale_price' ] != '' ? 1 : 0,
            'stock_quantity' => \wc_stock_amount( $args[ '_stock' ] ),
            'stock_status'   => $args[ '_stock_status' ],
            'rating_count'   => 0,
            'average_rating' => NULL,
            'total_sales'    => isset( $args[ 'total_sales' ] ) ? $args[ 'total_sales' ] : NULL,
        ];

        $wpdb->replace(
            $table,
            $update_data
        );
        wp_cache_set( 'lookup_table', $update_data, 'object_' . $post_id );

        return true;
    }

    private function setPostId( $post_id ) {

        $this->post_id = intval( $post_id );
    }

    private function setProductType( $slug, $taxonomy ){

        $term = get_term_by( 'slug', $slug, $taxonomy );

        if( $term ) {
            wp_delete_object_term_relationships( $this->post_id, $taxonomy );
            wp_set_post_terms( $this->post_id, $slug, $taxonomy, true );
        }
    }

    private function prepareImages( $post_id, $images ) {

        $foo = [];

        if( count( $images[ 'gallery' ] ) ) foreach ( $images[ 'gallery' ] as $k => $image ) {

            $type = 'gallery';

            if( $k == 0 ){

                if( !isset( $images['thumb'] ) || $images['thumb'] == $image[$k] ){
                    $type = 'thumb';
                }else{
                    $foo[] = [
                        'url'     => $images['thumb'],
                        'place'   => 'thumb',
                        'id'      => $post_id,
                        'post_id' => 0
                    ];
                }
            }

            $foo[] = [
                'url'     => $image,
                'place'   => $type,
                'id'      => $post_id,
                'post_id' => 0
            ];
        }

        if( count( $images[ 'sku' ] ) ) foreach ( $images[ 'sku' ] as $k => $image ) {

            $foo[] = [
                'url'     => str_replace( [ '_640x640.jpg', '_640x640.webp' ], '', $image ),
                'place'   => 'sku',
                'post_id'      => 0,
                'id' => $k
            ];
        }

        if( isset( $images[ 'ai_gallery' ] ) && count( $images[ 'ai_gallery' ] ) ){
            foreach ( $images[ 'ai_gallery' ] as $k => $image ) {

                $foo[] = [
                    'url'     => str_replace( [ '_640x640.jpg', '_640x640.webp' ], '', $image ),
                    'place'   => 'ai_gallery',
                    'post_id'      => 0,
                    'id' => $post_id
                ];
            }
        }

        return $foo;
    }

    private function linkContentImages() {

        $content =  $this->post[ 'post_content' ];
        $images  = $this->searchImgInContent( $content );
        $foo = [];

        if( $images ) foreach( $images as $url ) {

            if( ! $this->isExternal( $url ) )
                continue;

            if( $this->is_url_img($url) ) {
                $foo[] = [
                    'url'     => $url,
                    'place'   => 'content',
                    'post_id' => 0,
                    'id' => $this->post_id
                ];
            }
        }
        return $foo;
    }

    private function linkShortImages() {

        if( empty( $this->post[ 'post_excerpt' ] ) ){
            return [];
        }

        $excerpt =  $this->post[ 'post_excerpt' ];
        $images  = $this->searchImgInContent( $excerpt );
        $foo = [];

        if( $images ) foreach( $images as $url ) {

            if( ! $this->isExternal( $url ) )
                continue;

            if( $this->is_url_img($url) ) {
                $foo[] = [
                    'url'     => $url,
                    'place'   => 'short',
                    'post_id' => 0,
                    'id' => $this->post_id
                ];
            }
        }
        return $foo;
    }

    private function searchImgInContent( $content ) {
        return preg_match_all( '/<img.*src=["\']([^""\'\']*)["\'].*\/>/Uuim', $content, $match ) ? $match[ 1 ] : false;
    }

    private function isExternal( $url ) {
        return strripos( $url, $_SERVER[ "SERVER_NAME" ] ) === false;
    }

    private function is_url_img( $url ){
        return preg_match( '/[^\?]+\.(jpg|jpeg|gif|png|webp)\b/i', $url, $matches );
    }

    private function addAttribute( $attribute_name ) {

        $permalinks = get_option( 'woocommerce_permalinks' );

        $has = $this->hasAttribute( $attribute_name );

        if( ! $has ) {

            $label = __( 'Item', 'sel' );
            $name  = wc_attribute_taxonomy_name( sellvia_prepare_var_slug( $attribute_name ) );

            $taxonomy_data = [
                'hierarchical'          => true,
                'update_count_callback' => '_update_post_term_count',
                'labels'                => [
                    'name'              => sprintf( _x( 'Product %s', 'Product Attribute', 'woocommerce' ), $attribute_name ),
                    'singular_name'     => $attribute_name,
                    'search_items'      => sprintf( __( 'Search %s', 'woocommerce' ), $label ),
                    'all_items'         => sprintf( __( 'All %s', 'woocommerce' ), $label ),
                    'parent_item'       => sprintf( __( 'Parent %s', 'woocommerce' ), $label ),
                    'parent_item_colon' => sprintf( __( 'Parent %s:', 'woocommerce' ), $label ),
                    'edit_item'         => sprintf( __( 'Edit %s', 'woocommerce' ), $label ),
                    'update_item'       => sprintf( __( 'Update %s', 'woocommerce' ), $label ),
                    'add_new_item'      => sprintf( __( 'Add New %s', 'woocommerce' ), $label ),
                    'new_item_name'     => sprintf( __( 'New %s', 'woocommerce' ), $label )
                ],
                'show_ui'            => true,
                'show_in_quick_edit' => false,
                'show_in_menu'       => false,
                'show_in_nav_menus'  => false,
                'meta_box_cb'        => false,
                'query_var'          => true,
                'sort'               => false,
                'public'             => true,
                'rewrite'            => [
                    'slug'         => empty( $permalinks[ 'attribute_base' ] ) ? '' :
                        trailingslashit( $permalinks[ 'attribute_base' ] ) . sellvia_prepare_var_slug( $attribute_name ),
                    'with_front'   => false,
                    'hierarchical' => true
                ],
                'capabilities'       => [
                    'manage_terms' => 'manage_product_terms',
                    'edit_terms'   => 'edit_product_terms',
                    'delete_terms' => 'delete_product_terms',
                    'assign_terms' => 'assign_product_terms',
                ]
            ];

            register_taxonomy(
                $name,
                'product',
                $taxonomy_data
            );

            $this->insertWooTax(
                [
                    'attribute_label'   => wc_clean( stripslashes( $attribute_name ) ),
                    'attribute_name'    => sellvia_prepare_var_slug( $attribute_name ),
                    'attribute_type'    => 'select',
                    'attribute_orderby' => 'menu_order',
                    'attribute_public'  => 0
                ]
            );

            return $name;

        }

        return \wc_attribute_taxonomy_name( $has );
    }

    private function hasAttribute( $name ) {

        global $wpdb;

        $row = $wpdb->get_var(
            $wpdb->prepare( "SELECT attribute_name FROM {$wpdb->prefix}woocommerce_attribute_taxonomies WHERE attribute_label = %s", $name
            )
        );

        return empty( $row ) ? false : $row;
    }

    private function updateTerm( $title, $slug, $taxonomy ) {

        $term = get_term_by( 'slug', $slug, $taxonomy );

        if( $term )
            return $term->term_id;

        $result = wp_insert_term( $title, $taxonomy, [ 'slug' => $slug ] );

        if ( is_wp_error( $result ) )
            return [ 'error' => $result->get_error_message() ];

        return $result['term_id'];
    }

    private function insertWooTax( $attribute ) {

        global $wpdb;

        $wpdb->insert( $wpdb->prefix . 'woocommerce_attribute_taxonomies', $attribute );

        do_action( 'woocommerce_attribute_added', $wpdb->insert_id, $attribute );

        flush_rewrite_rules();
        delete_transient( 'wc_attribute_taxonomies' );
    }

    private function _taxonomy_install_type( $taxonomy_type, $all_data ){

        if( !empty( $all_data ) ) {

            foreach ($all_data as $term_data) {

                $this->_process_term_data($taxonomy_type, $term_data);
            }
        }
    }

    private function _process_term_data($taxonomy_type, $term) {

        if(taxonomy_exists($taxonomy_type)) {
            $term_exists = term_exists( $term['slug'], $taxonomy_type );
            $term_id     = is_array( $term_exists ) ? $term_exists['term_id'] : $term_exists;
            if ( ! $term_id ) {
                if(!empty( $term['parent'] )){
                    // see if we have imported this yet?
                    $term['parent'] = $this->_imported_term_id( $term['parent'] );
                }

                $t = wp_insert_term( $term['name'], $taxonomy_type, $term );
                if ( ! is_wp_error( $t ) ) {
                    $term_id = $t['term_id'];
                    //do_action( 'wp_import_insert_term', $t, $term, $post_id, $post );
                } else {
                    // todo - error
                }
            }
            $this->_imported_term_id($term['term_id'], $term_id);
        }

    }

    public function procced_terms($post_id, $terms_array){
        if ( ! empty( $terms_array ) ) {

            $terms_to_set = array();

            foreach ( $terms_array as $term_slug => $terms ) {

                foreach($terms as $term) {

                    $taxonomy    =  $term['taxonomy'];

                    if(taxonomy_exists($taxonomy)) {

                        $term_exists = term_exists( $term['slug'], $taxonomy );
                        $term_id     = is_array( $term_exists ) ? $term_exists['term_id'] : $term_exists;
                        if ( ! $term_id ) {
                            if(!empty( $term['parent'] )){
                                // see if we have imported this yet?
                                $term['parent'] = $this->_imported_term_id($term['parent']);
                            }

                            $t = wp_insert_term( $term['name'], $taxonomy, $term );
                            if ( ! is_wp_error( $t ) ) {
                                $term_id = $t['term_id'];
                                //do_action( 'wp_import_insert_term', $t, $term, $post_id, $post );
                            } else {
                                // todo - error
                                continue;
                            }
                        }
                        $this->_imported_term_id($term['term_id'], $term_id);
                        $terms_to_set[ $taxonomy ][] = intval( $term_id );
                    }
                }
            }
            foreach ( $terms_to_set as $tax => $ids ) {
                wp_set_post_terms( $post_id, $ids, $tax );
            }
        }
    }

    private function _imported_term_id( $original_term_id , $new_term_id = false ){
        $terms = get_transient('sellvia_importtermids');
        if(!is_array($terms))$terms = [];
        if($new_term_id){
            $terms[$original_term_id] = $new_term_id;
            set_transient('sellvia_importtermids', $terms, 60 * 60 * 24 );
        }else if($original_term_id && isset($terms[$original_term_id])){
            return $terms[$original_term_id];
        }
        return false;
    }

    private function get_parents_terms( $term_id, $list = [] ) {

        $term = get_term_by( 'id', $term_id, 'product_cat' );

        if( ! $term )
            return count( $list ) > 0 ? $list : false;

        $list[] = $term_id;

        if( $term->parent != 0 )
            return $this->get_parents_terms( $term->parent, $list );

        return $list;
    }

    public function prepareComments( $comments = [] ){

        if( !empty($comments) ) {

            $date_now_timestamp = date_i18n('U');

            $comments_dates = array_map( function ($item) {
                return strtotime($item['comment_date']);
            }, $comments);

            $date_max = max($comments_dates);

            $delta = $date_now_timestamp - $date_max;

            $comments = array_map( function ($item) use ($delta) {
                $new_date = date_i18n('Y-m-d H:i:s', strtotime($item['comment_date']) + $delta);
                $item['comment_date'] = $new_date;
                $item['comment_date_gtm'] = $new_date;
                return $item;
            }, $comments);
        }

        return $comments;
    }

    public function process_images(){

        if( empty( $this->images ) || empty( $this->post_id ) ){
            return [];
        }

        global $wpdb;

        $media = new MediaWoo();

        $options = $media->getOptions();

        $images_to_upload = [];

        $gallery = [];

        foreach ( $this->images as $image ) {

            if( $image['place'] == 'thumb' ) {

                if ($options['feature_images'] === 1) {

                    $images_to_upload[] = $image;

                } else {

                    $img_id = $media->create_external_attachment($this->post_id, $image['url']);

                    if ($img_id == false)
                        continue;

                    set_post_thumbnail( $this->post_id, $img_id );
                }

            } elseif( $image['place'] == 'gallery') {

                if ( $options['gallery_images'] === 1 ) {

                    $images_to_upload[] = $image;

                } else {

                    $img_id = $media->create_external_attachment($this->post_id, $image['url']);

                    if ($img_id == false)
                        continue;

                    $gallery[] = $img_id;
                }
            } elseif ( $image['place'] == 'sku') {

                if ( $options['variation_images'] === 1 ) {

                    $images_to_upload[] = $image;

                } else {

                    $var_ids = implode( ',', $this->getVariablesID( $this->post_id ) );

                    $variation_post_id = $wpdb->get_var(
                        $wpdb->prepare(
                            "SELECT post_id FROM {$wpdb->postmeta}
								 WHERE post_id IN ({$var_ids})
								 	AND meta_key = 'adswSKU'
								 	AND meta_value = %s",
                            $image['id']
                        )
                    );

                    if( !empty($variation_post_id) ) {

                        $img_id = $media->create_external_attachment($this->post_id, $image['url']);

                        if ($img_id == false)
                            continue;

                        \update_post_meta( $variation_post_id, '_thumbnail_id', $img_id );
                    }
                }
            } elseif ( $image['place'] == 'content' ){

                if( $options['content_images'] === 1 ){
                    $images_to_upload[] = $image;
                }
            } elseif ($image['place'] == 'short' ){

                if( $options['content_images'] === 1 ){
                    $images_to_upload[] = $image;
                }

            } elseif ( $image['place'] == 'ai_gallery' ){

//                if ( $options['gallery_images'] === 1 ) {

                $images_to_upload[] = $image;

//                } else {
//
//                    if ( $ai_gallery && is_array( $ai_gallery ) ) {
//
//                        $img_id = $media->create_external_attachment($this->post_id, $image['url']);
//
//                        if ($img_id == false)
//                            continue;
//
//                        foreach ($ai_gallery as $k => $v) {
//                            if ( $v == $image['url'] ) {
//                                $ai_gallery[$k] = $img_id;
//                            }
//                        }
//                    }
//                }
            }
        }

        \update_post_meta( $this->post_id, '_product_image_gallery', implode( ',', $gallery ) );

        return $images_to_upload;
    }

    public function resetProduct( $post_id, $data, $settings = [] ){

        $this->data = $data;

        $status = $this->prepare( 'reset' );

        if ( is_array( $status ) ) {
            return[
                'error' => 'Something went wrong on woo product prepare'
            ];
        }

        $this->setPostId( $post_id );

        $this->images = $this->prepareImages( $this->post_id , $this->images );

        $this->images = array_merge( $this->images, $this->linkContentImages() );

        $this->images = array_merge( $this->images, $this->linkShortImages() );

        $this->data['post_id'] = $this->post_id;

        $this->data['images'] = [];

        $textActivities = [];

        if( isset( $settings['featured'] ) && intval( $settings['featured'] ) === 1 ){

            $this->data['images'] = array_merge( $this->data['images'], array_filter( $this->images, function ($it){
                return $it['place'] === 'thumb';
            } ) );

            $textActivities[] = __( 'featured image', 'sel');
        }

        if( isset( $settings['gallery'] ) && intval( $settings['gallery'] ) === 1 ){

            \update_post_meta( $this->post_id, '_product_image_gallery', implode(',', [] ) );

            $this->data['images'] = array_merge( $this->data['images'], array_filter( $this->images, function ($it){
                return $it['place'] === 'gallery';
            } ) );

            $textActivities[] = __( 'gallery', 'sel');
        }

        if( isset( $settings['description'] ) && intval( $settings['description'] ) === 1 ){

            $post = [
                'ID' => $post_id,
                'post_content' => $data['product_content'],
            ];

            $response = wp_update_post( $post );

            if( $response ) {
                $this->data['images'] = array_merge($this->data['images'], array_filter( $this->images, function ($it) {
                    return $it['place'] === 'content';
                }));
            }

            $textActivities[] = __( 'description', 'sel');
        }

        if( isset( $settings['short_description'] ) && intval( $settings['short_description'] ) === 1 ){

            $post = [
                'ID' => $post_id,
                'post_excerpt' => !empty( $data['product_short'] ) ? trim( htmlspecialchars_decode( $data['product_short'] ) ) : '',
            ];

            $response = wp_update_post( $post );

            if( $response ) {
                $this->data['images'] = array_merge( $this->data['images'], array_filter($data['images'], function ($it) {
                    return $it['place'] === 'short';
                }));
            }

            $textActivities[] = __( 'short', 'sel');
        }

        if( isset( $settings['title'] ) && intval( $settings['title'] ) === 1 ){

            $post = [
                'ID' => $post_id,
                'post_title' => $data['product_title'],
            ];

            $response = wp_update_post( $post );

            $textActivities[] = __( 'title', 'sel');
        }

        if( isset( $settings['reviews'] ) && intval( $settings['reviews'] ) === 1 ){

            $this->deleteComments( $post_id );

            $textActivities[] = __( 'reviews', 'sel');
        }

        if( isset( $settings['variant'] ) && intval( $settings['variant'] ) === 1 ){

            $this->deleteVariations();

            $meta = [
                '_regular_price'  => $this->meta['_regular_price'],
                '_sale_price'     => $this->meta['_sale_price'],
                '_price'          => $this->meta['_price'],
                '_stock'          => $this->meta['_stock'],
                '_stock_status'   => $this->meta['_stock_status'],
                '_height'         => $this->meta['_height'],
                '_length'         => $this->meta['_length'],
                '_width'          => $this->meta['_width'],
                '_weight'         => $this->meta['_weight'],
                '_sku'            => $this->meta['_sku'],
                'adswSizeAttr'    => $this->meta['adswSizeAttr'],
            ];

            $this->updatePostMeta( $meta );

            if( isset( $this->product[ 'sku' ] ) && !empty( $this->product[ 'sku' ] ) && isset( $this->product[ 'skuAttr' ] ) && !empty( $this->product[ 'skuAttr' ] ) ) {

                $this->saveSKU();

                $this->saveVariations();

                $this->variationAttributes( $this->attribute );

                $this->setProductType( 'variable', 'product_type' );

                \WC_Product_Variable::sync( $this->post_id );

                $textActivities[] = __( 'variations', 'sel');

            } else {
                $this->setProductType( 'simple', 'product_type' );
            }

            $new_attributes = get_post_meta($this->post_id, '_product_attributes', true);
            $old = is_array($this->old_attributes) ? $this->old_attributes : [];
            $foo = array_merge($old, $new_attributes);
            update_post_meta($this->post_id, '_product_attributes', $foo);

            $this->setProductMetaLookup($this->meta, $this->post_id);
            wc_delete_product_transients($this->post_id);

            $this->data['images'] = array_merge( $this->data['images'], array_filter( $this->images, function ($it){
                return $it['place'] === 'sku';
            } ) );

            $textActivities[] = __( 'stock and pricing', 'sel');
        }

        //Shipping
        $wh_shipping = isset( $this->data['wh_shipping'] ) && !empty( $this->data['wh_shipping'] ) ? $this->data['wh_shipping'] : [];
        ShippingWoo::save_product_shipping( $this->post_id, maybe_unserialize( $this->data['skuAttr'] ), $wh_shipping );

        $plural1 = __( 'have', 'sel' );
        $plural2 = __( 'were', 'sel' );

        if( count( array_filter( $settings, function ( $setting ) { return $setting === 1; } ) ) == 1
            && ( $settings['featured'] == 1 || $settings['gallery'] == 1 || $settings['description'] == 1 ) ){
            $plural1 = __( 'has', 'sel' );
            $plural2 = __( 'was', 'sel' );
        }

        if( !empty($textActivities) ) {
            $this->message = __('Product', 'sel') . ' ' . implode(', ', $textActivities) . ' ' . $plural1 . __(' been reset.', 'sel');
            $textActivities = __('Product', 'sel') . ' ' . implode(', ', $textActivities) . ' ' . $plural2 . ' ' . __('reset.', 'sel');

            do_action('sellvia_log',
                [
                    'type' => 'reset',
                    'details' => $textActivities
                ],
                $textActivities,
                'product',
                'info',
                $this->post_id,
                null,
                ''
            );
        }else{
            $this->message = __('Check options to reset', 'sel');
        }

        $this->images = $this->data['images'];

        return true;
    }

    private function getVariablesID( $post_id ) {

        global $wpdb;

        $result = $wpdb->get_results(
            $wpdb->prepare( "SELECT ID FROM {$wpdb->posts} WHERE post_parent = %d AND post_type = 'product_variation'", $post_id )
        );

        $foo = [];
        if( $result ) foreach ( $result as $res )
            $foo[] = $res->ID;

        return $foo;
    }

    private function deleteVariations() {

        $product = wc_get_product( $this->post_id );

        $variation_ids = $product->get_children();

        if( $variation_ids && count( $variation_ids ) )
            foreach ( $variation_ids as $variation_id ) {
                if ( 'product_variation' === get_post_type( $variation_id ) ) {
                    $variation = wc_get_product( $variation_id );
                    $variation->delete( true );
                }
            }

        $attributes = $product->get_attributes();

        $remove = [];

        foreach( $attributes as $attribute ) {

            if( $attribute->get_variation() ) {

                $terms = wp_get_post_terms( $this->post_id, $attribute->get_name() );
                $remove[] = $attribute->get_name();
                if( $terms && ! is_wp_error( $terms ) ) {

                    $del = [];

                    foreach( $terms as $term ) {
                        $del[] = $term->term_id;
                    }

                    if( count( $del ) )
                        wp_remove_object_terms( $this->post_id, $del, $attribute->get_name() );
                }
            }
        }

        $attributes = get_post_meta( $this->post_id, '_product_attributes', true );

        if( $attributes && ! empty( $attributes ) ) {

            $foo = [];

            foreach( $attributes as $attribute ) {
                if( ! in_array( $attribute[ 'name' ], $remove ) )
                    $foo[] = $attribute;
            }
            $this->old_attributes = $foo;
            update_post_meta( $this->post_id, '_product_attributes', $foo );
        }
    }

    private function deleteComments( $post_id = NULL ){

        if(empty($post_id)){
            return;
        }

        if ( $comments = get_comments( array( 'post_id' => $post_id ) ) ) {
            foreach ( $comments as $comment ) {
                wp_delete_comment( $comment->comment_ID, true );
            }
        }
    }

    public static function preparePrices( $prices ) {

        if( !empty( $prices ) ) {
            foreach ( $prices as &$price ) {
                $price = apply_filters('sellvia_import_price', $price);
            }
        }

        return $prices;
    }

    public static function prepareSkuAttr( $skuAttr ) {

        $skuAttr = maybe_unserialize( $skuAttr );

        if( !empty( $skuAttr ) ) {
            foreach ( $skuAttr as &$item ) {
                $item['price'] = apply_filters('sellvia_import_price', $item['price'] );
                $item['salePrice'] = apply_filters('sellvia_import_price',  $item['salePrice'] );
            }
        }

        return maybe_serialize( $skuAttr );
    }
}
