<?php

namespace sellvia\Media;

class Media {

    protected $options = [];

    public function __construct(){

        $this->options = sellvia_get_images_settings();
    }

    public function getOptions(){

        return $this->options;
    }

    public function is_external_attachment( $attachment_id ){

        $tmp = get_post_meta( $attachment_id, '_wp_sellvia_attached_file', true );

        return $tmp && intval($tmp) === 1;
    }

    public function create_external_attachment( $post_id, $url, $name = null, $alt = null, $check_duplicate = true ){

        $url  = is_ssl() ? set_url_scheme( $url, 'https' ) : $url;
        $url = \sellvia_url_remove_get_params( $url );

        //TODO переписать нормально на $wpdb->get_col()
        if( $check_duplicate ) {

            global $wpdb;
            $res = $wpdb->get_col(
                "SELECT m.post_id FROM {$wpdb->prefix}postmeta m
                            WHERE m.meta_key = '_wp_attached_file' AND m.meta_value = '{$url}'");

            if( !empty( $res ) )
                return $res[0];
        }

        $ext = $this->getExtUrlImg( $url );

        if ( $name != '' ) {
            $filename = $post_id . '-' . sanitize_title( $name ) .'.'. $ext;
        } else {
            $filename = $post_id . '-' .  $this->hashName( $url ) .'.'. $ext;
            $name     = basename( $filename );
        }

        $wp_filetype = wp_check_filetype( basename( $filename ), null );

        $attachment = array(
            'post_mime_type' => $wp_filetype['type'],
            'post_title' => $name,
            'post_content' => '',
            'post_status' => 'inherit'
        );

        $attach_id = wp_insert_attachment( $attachment, $url, $post_id );

        if (!$attach_id)
            return false;

        $this->set_attachment_metadata($attach_id, $url);

        update_post_meta( $attach_id, '_sellvia_external_image_url', $url );

        if( !empty( $alt ) ){
            update_post_meta( $attach_id, '_wp_attachment_image_alt', $alt );
        }

        return $attach_id;
    }

    public function localize_attachment( $image_id ){

        $tmp = get_post_meta( $image_id, '_wp_sellvia_attached_file', true );

        if ($tmp && intval($tmp) === 1) {

            $url = get_post_meta( $image_id, '_sellvia_external_image_url', true );

            $url = $this->getFullUrlImg( $url );

            if ( ! $this->is_url_img( $url ) ) {
                return false;
            }

            $filename = get_the_title( $image_id );

            $wp_filetype =  wp_check_filetype( basename( $filename ), null );

            if( ! $wp_filetype ) {
                return false;
            }

            $uploaddir  = wp_upload_dir();
            $uploadfile = $uploaddir[ 'path' ] . '/' . $filename;

            if ( ! file_exists( $uploadfile ) ) {


                $contents = $this->getContentData( $url );

                if( $contents == false )
                    return false;

                $savefile = @fopen( $uploadfile, 'w' );

                if( $savefile ) {

                    fwrite( $savefile, $contents );
                    fclose( $savefile );
                } else {
                    return false;
                }
            }

            if ( !file_exists( $uploadfile ) ) {
                return false;
            }

            if ( filesize( $uploadfile ) == 0 ) {
                unlink( $uploadfile );
                return false;
            }

            //call upload action to give a chance to plugins like Resize Image After Upload to handle the new image
            do_action('wp_handle_upload', [ 'file' => $uploadfile, 'url' => wp_get_attachment_url( $image_id ), 'type' => $wp_filetype[ 'type' ] ] );

            // Trigger possible updates on CDN and other plugins
            update_attached_file( $image_id, $uploadfile);

            // Make thumb and/or update metadata
            wp_update_attachment_metadata( $image_id, wp_generate_attachment_metadata( $image_id, $uploadfile ) );

            delete_post_meta( $image_id, '_wp_sellvia_attached_file' );
            delete_post_meta( $image_id, '_sellvia_external_image_url' );
        }
    }

    private function set_attachment_metadata($attach_id, $url) {

        update_post_meta($attach_id, '_wp_sellvia_attached_file', 1);

        $pi = pathinfo( $url );

        $image_sizes = [];

        //размеры из cdn
        if( !empty( $pi[ 'extension' ] ) ){

            $images_settings = sellvia_get_images_settings();

            $crop_url = $url;

            if( $pi[ 'extension' ] === 'webp' && intval( $images_settings['webp'] ) === 1 ){
                $crop_url =rtrim( $url, '-full.webp' );
            }

            $image_sizes[ 'ads-thumb' ]    = [ 'url' => $crop_url . '-50x50.'   . $pi['extension'], 'width' => 50, 'height' => 50 ];
            $image_sizes[ 'thumbnail' ]    = [ 'url' => $crop_url . '-150x150.' . $pi['extension'], 'width' => 150, 'height' => 150 ];
            $image_sizes[ 'ads-medium' ]   = [ 'url' => $crop_url . '-220x220.' . $pi['extension'], 'width' => 220, 'height' => 220 ];
            $image_sizes[ 'ads-big' ]      = [ 'url' => $crop_url . '-350x350.' . $pi['extension'], 'width' => 350, 'height' => 350 ];
            $image_sizes[ 'ads-large' ]    = [ 'url' => $crop_url . '-640x640.' . $pi['extension'], 'width' => 640, 'height' => 640 ];
        }

        $image_sizes[ 'large' ] = [ 'url' => $url, 'width' => 800, 'height' => 800 ];

        $attach_data = array(
            'file' => 0,
            'width' => 0,
            'height' => 0,
            'sizes' => [],
            'image_meta' => [
                'aperture' => '0',
                'credit' => '',
                'camera' => '',
                'caption' => '',
                'created_timestamp' => '0',
                'copyright' => '',
                'focal_length' => '0',
                'iso' => '0',
                'shutter_speed' => '0',
                'title' => '',
                'orientation' => '0',
                'keywords' => [],
            ],
        );

        $attach_data = array_replace_recursive( $attach_data, [
            'file' => $image_sizes['large']['url'],
            'width' => $image_sizes['large']['width'],
            'height' => $image_sizes['large']['height'],
        ] );

        $wp_sizes = $this->get_image_sizes();

        foreach ( $wp_sizes as $size => $props ) {

            $found_size = $this->_choose_image_size_from_cdn( $props, $image_sizes );

            if ( !empty( $found_size ) ) {
                $wp_filetype = wp_check_filetype( basename( $found_size[ 'url' ] ), null );
                $attach_data[ 'sizes' ][ "$size" ] = [
                    'file'      => basename( $found_size[ 'url' ] ),
                    'width'     => $found_size[ 'width' ],
                    'height'    => $found_size[ 'height' ],
                    'mime-type' => $wp_filetype[ 'type' ],
                ];
            }
        }

        wp_update_attachment_metadata( $attach_id, $attach_data );
    }

    public function downloadReviewImage( $commentId, $post_id ){

        $images = get_comment_meta( $commentId, 'images', true);
        if (count($images) == 0) {
            return false;
        }

        $urls = maybe_unserialize($images);

        if (!is_array($urls) || empty($urls)) {
            return false;
        }

        foreach ($urls as $key => $url) {
            $attach = $this->attachmentImage($post_id, $url, 'full');
            if ($attach !== false) {
                $urls[$key] = $attach['id'];
            }
        }

        return update_comment_meta( $commentId, 'images', $urls);
    }

    private function getExtUrlImg( $url ){

        $parsed = parse_url( $url );
        $filename = basename( $parsed[ 'path' ] );
        $ext = explode( '.', $filename );

        return $ext ? array_pop( $ext ) : '';
    }

    private function hashName( $name ){

        return substr( md5( $name ), 0, 6 );
    }

    public function get_image_sizes() {

        global $_wp_additional_image_sizes;

        $sizes = [];

        foreach ( get_intermediate_image_sizes() as $_size ) {

            if ( in_array( $_size, [ 'thumbnail', 'medium', 'medium_large', 'large' ] ) ) {
                $sizes[$_size]['width']  = get_option( "{$_size}_size_w" );
                $sizes[$_size]['height'] = get_option( "{$_size}_size_h" );
                $sizes[$_size]['crop']   = (bool) get_option("{$_size}_crop");

            } elseif ( isset( $_wp_additional_image_sizes[ $_size ] ) ) {

                $sizes[$_size] = [
                    'width'  => $_wp_additional_image_sizes[ $_size ][ 'width' ],
                    'height' => $_wp_additional_image_sizes[ $_size ][ 'height' ],
                    'crop'   => $_wp_additional_image_sizes[ $_size ][ 'crop' ],
                ];
            }
        }

        return $sizes;
    }

    private function _choose_image_size_from_cdn( $size, $image_sizes = [] ) {

        if ( empty( $image_sizes ) ) {
            return false;
        }

        $min_size = $max_size = false;

        foreach ( $image_sizes as $props ) {

            if ( (int) $size[ 'width' ] == (int) $props[ 'width' ] ) {
                return $props;
            }

            if ( intval( $size[ 'width' ] ) < intval( $props[ 'width' ] ) && ( !$min_size || intval( $min_size[ 'width' ] ) > intval( $props[ 'width' ] ) ) ) {
                $min_size = $props;
            }

            if ( !$max_size || ( intval( $max_size[ 'width' ] ) < intval( $props[ 'width' ] ) ) ) {
                $max_size = $props;
            }
        }

        return !$min_size ? $max_size : $min_size;
    }

    private function getFullUrlImg( $url ) {
        $url = preg_replace('/(_\d+x\d+\.(jpg|jpeg|gif|png|webp))/','',$url);

        return $url;
    }

    public function is_url_img( $url ){
        return preg_match( '/[^\?]+\.(jpg|jpeg|gif|png|webp)\b/i', $url, $matches );
    }

    private function getContentData( $data ) {

        if( substr( $data, 0, 2 ) == '//' ) {
            $data = 'http:' . $data;
        }

        if( substr( $data, 0, 4 ) == 'http' ) {
            return $this->file_get_contents( $data );
        }

        return base64_decode( $data );
    }

    public function getImageById( $id, $size = 'thumbnail' ) {

        $img = wp_get_attachment_image_src( $id, $size, false );

        if ( $img ) {
            return $img[ 0 ];
        }

        return false;
    }

    private function getAttachImageId( $image_url ) {

        global $wpdb;

        $attachment = $wpdb->get_col(
            $wpdb->prepare(
                "SELECT ID FROM {$wpdb->posts} WHERE post_title LIKE '%s';", '%' . $wpdb->esc_like( $image_url ) . '%'
            )
        );

        return isset( $attachment[ 0 ] ) ? $attachment[ 0 ] : false;
    }

    private function file_get_contents( $file ) {

        $response = wp_remote_get(
            $file,
            array(
                'timeout'   => 15,
                'sslverify' => false
            )
        );

        if ( ! is_wp_error( $response ) || wp_remote_retrieve_response_code( $response ) === 200 ){
            return $response['body'];
        }

        return false;
    }

    public function attachmentImage( $post_id, $url, $size = 'full', $name = '' ) {

        $url = $this->getFullUrlImg($url);

        preg_match( '/[^\?]+\.(jpg|jpeg|gif|png|webp)\b/i', $url, $matches );
        if ( ! $matches ) {
            return false;
        }

        $p = strrpos( $url, "." );
        $r = substr( $url, $p );

        if ( $name != '' ) {
            $filename = $post_id . '-' . sanitize_title( $name ) . $r;
        } else {
            $filename = $post_id . '-' . sanitize_title( $this->hashName( $url ) ) . $r;
            $name     = basename( $filename );
        }

        $attach_id = $this->getAttachImageId( basename( $filename ) );

        if ( empty( $attach_id ) ) {
            $uploaddir = wp_upload_dir();
            $uploadfile = $uploaddir['path'] . '/' . $filename;

            if (!file_exists($uploadfile)) {
                $contents = $this->file_get_contents($url);
                if ($contents == false)
                    return false;

                $fp = @fopen($uploadfile, 'x');
                if ($fp) {

                    fwrite($fp, $contents);
                    fclose($fp);
                }
            }


            if (!file_exists($uploadfile)) {
                return false;
            }

            if (filesize($uploadfile) == 0) {
                unlink($uploadfile);
                return false;
            }

            $wp_filetype = wp_check_filetype(basename($filename), null);

            $attachment_data = array(
                'post_mime_type' => $wp_filetype['type'],
                'post_title' => $name,
                'post_content' => '',
                'post_status' => 'inherit'
            );

            $attach_id = wp_insert_attachment($attachment_data, $uploadfile, $post_id);
            $imagenew = get_post($attach_id);
            $fullsizepath = get_attached_file($imagenew->ID);

            if (!function_exists('wp_generate_attachment_metadata')) {
                include(ABSPATH . 'wp-admin/includes/image.php');
            }

            $attach_data = \wp_generate_attachment_metadata($attach_id, $fullsizepath);
            \wp_update_attachment_metadata($attach_id, $attach_data);
        }else{
            $this->localize_attachment( $attach_id );
        }


        return array(
            'url' => $this->getImageById( $attach_id, $size ),
            'id'  => $attach_id
        );
    }

    public function attachmentImageBlob( $post_id, $blob, $name = false, $ext ) {

        $name = str_replace( $ext, '', $name );

        $filename = !empty($post_id) ? $post_id . '-' : '';

        if ($name) {
            $filename .= sanitize_title( $name ) . '.' . $ext;
        } else {
            $filename .= $this->hashName($name) . '.' . $ext;
        }

        $attach_id = $this->getAttachImageId( basename( $filename ) );

        if ( empty( $attach_id ) ) {

            $wp_filetype = wp_check_filetype($filename);

            if (!$wp_filetype) {
                return [
                    'error' => 'error: file_type',
                    'url' => false,
                    'id' => false
                ];
            }

            $attach_id = $this->loadImages($post_id, $blob, $filename, basename($filename), true);

            if (!$attach_id) {
                return [
                    'error' => 'error: not attach_id',
                    'url' => false,
                    'id' => false
                ];
            }
        }

        return [
            'url' => $this->getImageById($attach_id, 'full'),
            'id' => $attach_id
        ];
    }

    private function loadImages( $post_id, $data , $filename, $name, $replace = false) {

        $uploaddir  = wp_upload_dir();
        $uploadfile = $uploaddir[ 'path' ] . '/' . $filename;

        if ( ! file_exists( $uploadfile ) || $replace ) {

            if( isset( $data[ 'tmp_name' ] ) ) {
                if( !move_uploaded_file( $data[ 'tmp_name' ], $uploadfile ) ) {
                    return false;
                }
            }else{
                $contents = $this->getContentData( $data );

                if( $contents == false )
                    return false;

                $savefile = @fopen( $uploadfile, 'w' );

                if( $savefile ) {

                    fwrite( $savefile, $contents );
                    fclose( $savefile );
                } else {
                    return false;
                }
            }

        }


        if ( !file_exists( $uploadfile ) ) {
            return false;
        }

        if ( filesize( $uploadfile ) == 0 ) {
            unlink( $uploadfile );
            return false;
        }

        $wp_filetype = wp_check_filetype( basename( $filename ), null );

        $attachment_data = [
            'post_mime_type' => $wp_filetype[ 'type' ],
            'post_title'     => $name,
            'post_content'   => '',
            'post_status'    => 'inherit'
        ];

        $attach_id    = wp_insert_attachment( $attachment_data, $uploadfile, $post_id );
        $imagenew     = get_post( $attach_id );
        $fullsizepath = get_attached_file( $imagenew->ID );

        if( !function_exists( 'wp_generate_attachment_metadata' ) ){
            include( ABSPATH . 'wp-admin/includes/image.php' );
        }


        $attach_data = \wp_generate_attachment_metadata( $attach_id, $fullsizepath );

        $attach_data[ 'image_meta' ][ 'title' ] = $name;

        \wp_update_attachment_metadata( $attach_id, $attach_data );

        update_post_meta( $attach_id, '_wp_attachment_image_alt', $name );

        return $attach_id;
    }

    public function searchImgInContent( $content ) {
        return preg_match_all( '/<img.*src=["\']([^""\'\']*)["\'].*\/>/Uuim', $content, $match ) ? $match[ 1 ] : false;
    }

    public function replaceImgContent( $post_id, $image ){

        $post    = get_post( $post_id );
        $content = $post->post_content;

        $content = str_replace( $image['url'], wp_get_attachment_url( $image['post_id'] ), $content );

        wp_update_post( [
            'ID'           => $post_id,
            'post_content' => $content
        ] );

        return true;
    }

    public function replaceImgShort( $post_id, $image ){

        $post    = get_post( $post_id );
        $excerpt = $post->post_excerpt;

        $excerpt = str_replace( $image['url'], wp_get_attachment_url( $image['post_id'] ), $excerpt );

        wp_update_post( [
            'ID'           => $post_id,
            'post_excerpt' => $excerpt
        ] );

        return true;
    }

    public function isExternal( $url ) {
        return strripos( $url, $_SERVER[ "SERVER_NAME" ] ) === false;
    }
}