<?php

namespace sellvia\Media\Upload;

use sellvia\Media\MediaAds;

class uploadExternalMediaAds extends uploadExternalMedia {

    public function __construct(){

        parent::__construct();
    }

    public function getProduct( $post_id ){

        global $wpdb;

        return $wpdb->get_row(
            $wpdb->prepare(
                "SELECT pr.* , pm.* , pr.post_id

                  FROM {$wpdb->ads_products} pr
                    INNER JOIN {$wpdb->posts} posts ON pr.post_id = posts.ID
                    LEFT JOIN {$wpdb->ads_products_meta} pm ON pm.post_id = pr.post_id
                  WHERE pr.post_id <> 0 AND pr.post_id = %d", $post_id
            ),
            ARRAY_A
        );
    }

    protected function get_next_product( $current ) {

        global $wpdb;

        $row = $wpdb->get_row(
            $wpdb->prepare(
                "SELECT
					posts.ID as post_id, posts.post_title, m.meta_value as sellvia_post_id
                FROM {$wpdb->posts} posts
                    INNER JOIN {$wpdb->postmeta} m ON m.post_id = posts.ID
                    LEFT JOIN {$wpdb->ads_products} pr ON pr.post_id = posts.ID
                WHERE m.meta_key = 'sellvia_post_id'
                    AND posts.post_type = 'product' 
                    AND posts.post_status IN('publish','draft','importlist') 
                ORDER BY post_id ASC LIMIT %d, 1", $current
            ),
            ARRAY_A
        );

        if( ! empty( $row ) ) {
            $row[ 'thumbnail' ]       = get_the_post_thumbnail_url( $row['post_id'], 'thumbnail' );
            $row[ 'view_on_sellvia' ] = esc_url( SELLVIA_CATALOG_PATH . 'product/' . $row['sellvia_post_id'] );
            $row[ 'permalink' ]       = get_the_permalink($row['post_id']);
        }

        return empty( $row ) ? false : $row;
    }

    protected function link_feature( $post_id ) {

        $image_id = get_post_meta( $post_id, '_thumbnail_id', true );

        $media = new MediaAds();

        if( empty( $image_id ) ) {

            $product = $this->getProduct( $post_id );
            $url     = $product[ 'imageUrl' ];

            if( ! $media->is_url_img( $url ) || ! $media->isExternal( $url ) ){
                return [];
            }

            $image_id = $media->create_external_attachment( $post_id, $url );

            if( !$image_id ) {
                return [];
            }

            set_post_thumbnail( $post_id, $image_id );

        }else {
            $url = wp_get_attachment_url( $image_id );

            if( ! $media->is_url_img( $url ) || ! $media->isExternal( $url ) ){
                return [];
            }
        }

        return [
            [
                'place'   => 'thumb',
                'url'     => $url,
                'post_id' => intval( $image_id ),
                'local'   => 0
            ]
        ];
    }

    protected function link_gallery( $post_id ){

        $product = $this->getProduct( $post_id );
        $gallery = $product[ 'gallery' ];

        $gallery = maybe_unserialize( $gallery );

        if( ! $gallery && ! is_array( $gallery ) ) {
            return [];
        }

        $foo = [];

        $media = new MediaAds();

        foreach( $gallery as $k => $v ) {

            if( $media->is_url_img( $v ) && $media->isExternal( $v ) ) {

                $image_id = $media->create_external_attachment( $post_id, $v );

                if( !$image_id ) {
                    continue;
                }

                $foo[] = [
                    'place'   => 'gallery',
                    'url'     => $v,
                    'post_id' => intval( $image_id ),
                    'local'   => 0
                ];

                $gallery[$k] = $image_id;

            } elseif ( is_numeric( $v ) ){

                if( !$media->is_external_attachment( $v ) ){
                    continue;
                }

                $url = get_post_meta( $v, '_wp_attached_file', true);

                if( ! $media->is_url_img( $url ) || ! $media->isExternal( $url ) ){
                    continue;
                }

                $foo[] = [
                    'place'   => 'gallery',
                    'url'     => $url,
                    'post_id' => intval( $v ),
                    'local'   => 0
                ];
            }
        }

        global $wpdb;

        $wpdb->update(
            $wpdb->prefix . 'ads_products_meta',
            [
                'gallery' => maybe_serialize($gallery)
            ],
            [
                'post_id' => $post_id
            ]
        );

        return $foo;
    }

    protected function link_variation( $post_id ){

        $product = $this->getProduct( $post_id );

        $sku     = $product['sku'];
        $sku     = maybe_unserialize( $sku );

        if( ! $sku && ! is_array($sku) ) {
            return [];
        }

        $foo = [];

        $media = new MediaAds();

        foreach( $sku as $k => $v ) {

            if( $media->is_url_img( $v['img'] ) && $media->isExternal( $v['img'] ) ){

                $image_id = $media->create_external_attachment( $post_id, $v['img'] );

                if( !$image_id ) {
                    continue;
                }

                $foo[] = [
                    'place'   => 'sku',
                    'url'     => $v['img'],
                    'post_id' => intval( $image_id ),
                    'local'   => 0
                ];

                $sku[$k]['img'] = $image_id;

            } elseif ( is_numeric( $v['img'] ) ){

                if( !$media->is_external_attachment( $v['img'] ) ){
                    continue;
                }

                $url = get_post_meta( $v['img'], '_wp_attached_file', true);

                if( ! $media->is_url_img( $url ) || ! $media->isExternal( $url ) ){
                    continue;
                }

                $foo[] = [
                    'place'   => 'sku',
                    'url'     => $url,
                    'post_id' => intval( $v['img'] ),
                    'local'   => 0
                ];
            }
        }

        global $wpdb;

        $wpdb->update(
            $wpdb->prefix . 'ads_products_meta',
            [
                'sku' => maybe_serialize($sku)
            ],
            [
                'post_id' => $post_id
            ]
        );

        return $foo;
    }
}
