<?php

namespace sellvia\Media\Upload;

use sellvia\Media\MediaWoo;

class uploadExternalMediaWoo extends uploadExternalMedia {

    public function __construct(){

        parent::__construct();
    }

    protected function get_next_product( $current ) {

        global $wpdb;

        $row = $wpdb->get_row(
            $wpdb->prepare(
                "SELECT
					posts.ID as post_id, posts.post_title, m.meta_value as sellvia_post_id
                FROM {$wpdb->posts} posts
                    INNER JOIN {$wpdb->postmeta} m ON m.post_id = posts.ID
                WHERE m.meta_key = 'sellvia_post_id'
                    AND posts.post_type = 'product' 
                    AND posts.post_status IN('publish','draft','importlist') 
                ORDER BY post_id ASC LIMIT %d, 1", $current
            ),
            ARRAY_A
        );

        if( ! empty( $row ) ) {
            $row[ 'thumbnail' ]       = get_the_post_thumbnail_url( $row['post_id'], 'thumbnail' );
            $row[ 'view_on_sellvia' ] = esc_url( SELLVIA_CATALOG_PATH . 'product/' . $row['sellvia_post_id'] );
            $row[ 'permalink' ]       = get_the_permalink($row['post_id']);
        }

        return empty( $row ) ? false : $row;
    }

    protected function link_feature( $post_id ) {

        $image_id = get_post_meta( $post_id, '_thumbnail_id', true );

        $media = new MediaWoo();

        if( empty( $image_id ) ) {
            return [];
        }else {
            $url = wp_get_attachment_url( $image_id );

            if( ! $media->is_url_img( $url ) || ! $media->isExternal( $url ) ){
                return [];
            }
        }

        return [
            [
                'place'   => 'thumb',
                'url'     => $url,
                'post_id' => intval( $image_id ),
                'local'   => 0
            ]
        ];
    }

    protected function link_gallery( $post_id ) {

        $gallery = get_post_meta( $post_id, '_product_image_gallery', true );
        $gallery = ! empty( $gallery ) ? explode(',', $gallery) : [];

        if( empty( $gallery ) ){
            return [];
        }

        $media = new MediaWoo();

        $foo = [];

        foreach ( $gallery as $v ){

            if( !$media->is_external_attachment( $v ) ){
                continue;
            }

            $url = get_post_meta( $v, '_wp_attached_file', true);

            if( ! $media->is_url_img( $url ) || ! $media->isExternal( $url ) ){
                continue;
            }

            $foo[] = [
                'place'   => 'gallery',
                'url'     => $url,
                'post_id' => intval( $v ),
                'local'   => 0
            ];
        }

        return $foo;
    }

    protected function link_variation( $post_id ){

        global $wpdb;

        $var_ids = implode( ',', $this->getVariablesID( $post_id ) );

        $variation_images_id = $wpdb->get_col(
            "SELECT meta_value FROM {$wpdb->postmeta} 
				 WHERE post_id IN ({$var_ids}) 
				    AND meta_key = '_thumbnail_id'"
        );

        if( empty( $variation_images_id ) ){
            return [];
        }

        $media = new MediaWoo();

        $foo = [];

        foreach ( $variation_images_id as $v ){

            if( !$media->is_external_attachment( $v ) ){
                continue;
            }

            $url = get_post_meta( $v, '_wp_attached_file', true);

            if( ! $media->is_url_img( $url ) || ! $media->isExternal( $url ) ){
                continue;
            }

            $foo[] = [
                'place'   => 'sku',
                'url'     => $url,
                'post_id' => intval( $v ),
                'local'   => 0
            ];
        }

        return $foo;
    }

    private function getVariablesID( $post_id ) {

        global $wpdb;

        $result = $wpdb->get_results(
            $wpdb->prepare( "SELECT ID FROM {$wpdb->posts} WHERE post_parent = %d AND post_type = 'product_variation'", $post_id )
        );

        $foo = [];
        if( $result ) foreach ( $result as $res )
            $foo[] = $res->ID;

        return $foo;
    }
}
