<?php

namespace sellvia\Media;

class cdnMedia {

    public function __construct(){

        add_filter( 'wp_get_attachment_url',                 [ $this, 'get_attachment_url' ], 1000, 2 );
        add_filter( 'wp_calculate_image_srcset',             [ $this, 'calculate_image_srcset' ], 1000, 5 );
        add_filter( 'ads_get_list_images_url',               [ $this, 'ads_get_list_images_url' ], 1000, 2 );

        add_filter( 'manage_media_columns',                  [ $this, 'media_extra_column' ] );
        add_action( 'manage_media_custom_column',            [ $this, 'media_extra_column_value' ], 10, 2 );
        add_filter( 'wp_prepare_attachment_for_js',          [ $this, 'wp_prepare_attachment_for_js' ], 10, 3 );
        add_action( 'admin_print_footer_scripts-upload.php', [ $this, 'add_class_to_media_library_grid_elements' ] );
        add_action( 'admin_print_styles-upload.php',         [ $this, 'add_styles_to_media_library_grid_elements' ] );
    }

    function get_attachment_url( $url, $id ) {

        // if not sellvia attached return to default function

        if ( !get_post_meta( $id, '_wp_sellvia_attached_file', true ) ) {
            return $url;
        }

        if ( !$post = get_post( (int) $id ) ) {
            return false;
        }

        if ( 'attachment' != $post->post_type ) {
            return false;
        }

        $new_url = '';
        if ( $file = get_post_meta( $post->ID, '_wp_attached_file', true ) ) {
            if ( substr( $file, 0, 7 ) === "http://" || substr( $file, 0, 8 ) === "https://" ) {
                $new_url = $file;
            }
        }

        if ( empty( $new_url ) ) {
            return false;
        }

        return $new_url;
    }

    function calculate_image_srcset( $sources, $size_array, $image_src, $image_meta, $attachment_id ) {

        // if not sellvia attached return to default function
        if ( !$sources || !get_post_meta( $attachment_id, '_wp_sellvia_attached_file', true ) ) {
            return $sources;
        }

        $upload_dir = wp_get_upload_dir();
        $image_baseurl = trailingslashit( $upload_dir['baseurl'] );

        if ( is_ssl() && 'https' !== substr($image_baseurl, 0, 5) && parse_url($image_baseurl, PHP_URL_HOST) === $_SERVER['HTTP_HOST'] ) {

            $image_baseurl = set_url_scheme($image_baseurl, 'https');
        }

        foreach ( $sources as &$src ) {

            $src['url'] = str_replace( $image_baseurl, '', $src['url'] );
        }

        return $sources;
    }

    function ads_get_list_images_url( $url, $attachment_id ){

        // if not sellvia attached return to default function
        if ( !$url || !get_post_meta( $attachment_id, '_wp_sellvia_attached_file', true ) ) {
            return $url;
        }

        $upload_dir = wp_get_upload_dir();
        $image_baseurl = trailingslashit( $upload_dir['baseurl'] );

        if ( is_ssl() && 'https' !== substr($image_baseurl, 0, 5) && parse_url($image_baseurl, PHP_URL_HOST) === $_SERVER['HTTP_HOST'] ) {

            $image_baseurl = set_url_scheme($image_baseurl, 'https');
        }

        return str_replace( $image_baseurl, '', $url );
    }

    function media_extra_column( $columns ) {

        return sellvia_array_insert( $columns, 2, [
            'sellvia_storage' => __( 'Storage type', 'sel' )
        ] );
    }

    function media_extra_column_value( $column_name, $id ) {

        if( $column_name == 'sellvia_storage' ){

            $tmp = get_post_meta( $id, '_wp_sellvia_attached_file', true );

            $source =  ($tmp && intval($tmp) === 1) ? __( 'Sellvia CDN', 'sel' ) : __( 'Local', 'sel' );

            echo $source;
        }
    }

    function wp_prepare_attachment_for_js( $response, $attachment, $meta ) {

        $tmp = get_post_meta( $response[ 'id' ], '_wp_sellvia_attached_file', true );

        $response['customClass'] = ($tmp && intval($tmp) === 1) ? "attachment-sellvia-cdn" : '';

        return $response;
    }

    function add_class_to_media_library_grid_elements(){
        global $mode;
        if( 'grid' === $mode ) {
            echo '<script>
            (function($) {
                $(document).ready( function() {
                    if ( undefined !== wp.media ) {
                        wp.media.view.Attachment.Library = wp.media.view.Attachment.Library.extend({
                            className: function () { return \'attachment \' + this.model.get( \'customClass\' ); }
                        });
                    }
                });
            })(jQuery);
        </script>';
        }
    }

    function add_styles_to_media_library_grid_elements() {
        global $mode;
        if( 'grid' === $mode ) {
            $background_url = SELLVIA_URL . '/assets/images/sellvia_icon_white.svg';
            echo '<style>.attachment.attachment-sellvia-cdn .thumbnail:before{display: block; z-index: 100; content: ""; position: absolute; left: 4px; bottom: 4px; width: 24px; height: 24px; background: url(' . $background_url . ') center/contain no-repeat }</style>';
        }else{
            echo '<style>.sellvia_storage { width: 10%; }</style>';
        }

    }

}