<?php
namespace sellvia\Order;

abstract class Order {

    protected $fulfillment = [];

    public function __construct(){

        $this->setup();
    }

    private function setup(){
        $this->fulfillment = sellvia_get_fulfillment_settings();
    }

    public function actions( $post ) {

        if( isset( $post['sellvia_action'] ) && current_user_can( 'activate_plugins' ) ) {

            $action = 'action_' . $post['sellvia_action'];
            $args        = isset( $post['args'] ) ? $post['args'] : $post;
            $data        = [];

            if( is_array( $args ) )
                $data = $args;
            else
                parse_str( $args, $data );


            if( method_exists( $this, $action ) ) {

                return $this->$action( $data );
            }
        }

        return [ 'error' => __( 'Undefined action', 'sel' ) ];
    }

    public function income_method( $post ){

        if( isset( $post['action'] ) && isset( $post['method'] ) ) {

            $method = 'method_' . $post['method'];
            $args        = $post['data'] ?? $post;
            $data        = [];

            if( is_array( $args ) )
                $data = $args;
            else
                parse_str( $args, $data );


            if( method_exists( $this, $method ) ) {

                return $this->$method( $data );
            }
        }

        return [ 'error' => __( 'Undefined action', 'sel' ) ];
    }

    /**
     * Поиск по базе товаров со склада
     *
     * @param $data
     *
     * @return array
     */
    private function action_order_search( $data ) {

        $orders = [
            'quantity', 'sale_price', 'id', 'title'
        ];

        $foo = [
            'pageNo'   => isset( $data['pageNo'] ) ? (int) $data['pageNo'] : 1,
            'pageSize' => isset( $data['pageSize'] ) ? (int) $data['pageSize'] : 50,
            'order'    => isset( $data['order'] ) && $data['order'] == 'asc' ? 'ASC' : 'DESC',
            'orderBy'  => isset( $data['orderBy'] ) && in_array( $data['orderBy'], $orders ) ? $data['orderBy'] : 'id',
            'search'   => isset( $data['search'] ) ? sanitize_text_field( $data['search'] ) : ''
        ];

        return $this->call( 'search', $foo );
    }


    /**
     * Получение данных заказа магазина и со склада
     *
     * @param $data
     *
     * @return array
     */

    protected function action_order_data( $data ){

        return $this->order_data( $data['order_id'] );
    }

    abstract protected function order_data( $order_id ) : array;

    /**
     * Размещение заказа на складе
     *
     * @param $data
     * @return array
     */
    protected function action_place_order( $data ) {

        return $this->place_order( $data );
    }

    abstract protected function place_order( $data, $auto = false ) : array;

    abstract public function place_order_automatically( $data );

    abstract public function method_set_order_tracking( $data );

    static function call( $path, $args = [] ) {

        $keys = sellvia_get_api_keys();

        $public_key  = $keys['public_key'];
        $secret_key  = $keys['secret_key'];

        $auth_args = [
            'public_key'  => $public_key,
            'signature'   => md5( $public_key . $secret_key ),
            'domain'      => get_home_url(),
            'plugin'      => 'sellvia',
        ];

        $args = array_merge( $args, $auth_args );

//        var_dump($args);
//        var_dump(json_encode( $args ));

        $response = wp_remote_post( SELLVIA_REQUEST_PATH . '/sellvia_customer_order/' . $path, [
            'method'    => 'POST',
            'timeout'   => 120,
            'sslverify' => false,
            'headers'   => [],
            'body'      => json_encode( $args )
        ] );

//        var_dump($response);

        if( is_wp_error( $response ) )
            return [ 'error' => $response->get_error_message() ];

        if( ! wp_remote_retrieve_response_code( $response ) == '200' )
            return [ 'error' => __( 'The response from the server has not been received', 'sel' ) ];

        return json_decode( wp_remote_retrieve_body( $response ), true );
    }

}