<?php

namespace sellvia\Order;

use ads\adsOrders;

class OrderAds extends Order {

    public function __construct() {

        parent::__construct();
    }

    protected function order_data( $order_id ) : array {

        $order = new adsOrders( $order_id, true );

        if( !empty( $order->get_hash() ) ) {

            $ff_status           = ads_prepare_options( ads_fulfillment_status() );
            $abandoned_ff_status = ads_prepare_options( [ 'not_processed' => ads_fulfillment_status()['not_processed'] ] );

            $foo = [
                'already'         => false,
                'canEdit'         => true,
                'hash'            => $order->get_hash(),
                'status'          => $order->get_fulfillment(),
                'status_values'   => $order->get_status() === 'abandoned' ? $abandoned_ff_status : $ff_status,
                'status_colors'   => [
                    'not_processed' => 'info',
                    'processing'    => 'warning',
                    'completed'     => 'success',
                    'shipped'       => '',
                    'refund'        => 'danger'
                ],
                'order_number'    => $order->get_trans_id(),
                'email'           => $order->get_email(),
                'full_name'       => $this->entity_decode( $order->get_full_name() ),
                'country'         => $this->entity_decode( $order->get_country() ), //две буквы
                'state'           => $this->entity_decode( $order->get_state() ),
                'city'            => $this->entity_decode( $order->get_city() ),
                'cpf'             => $this->entity_decode( $order->get_cpf() ),
                'cpf2'            => $this->entity_decode( $order->get_cpf2() ),
                'street'          => trim( $this->entity_decode( $order->get_address() ) ),
                'zip'             => trim( $order->get_postal_code() ),
                'phone_number'    => trim( $order->get_phone_number() ),
                'description'     => trim( $order->get_description() ),
                'items'           => $order->get_orders(),
                'country_options' => ads_prepare_options( ads_shipping_countries() ),
                'ordered'         => [], //данные с базы склада,
                'plugin'          => in_array( SELLVIA_PLUGIN, ['ADSW', 'WOO'] ) ? 'WOO' : SELLVIA_PLUGIN,
                'redirect'        => null,
            ];

            $wh_order = $this->findOrder( $foo );

            if( $wh_order && isset( $wh_order['hash'] ) ) {

                if( isset( $wh_order['error'] ) ) {
                    return $wh_order;
                }

                $foo['already']      = true;
                $foo['canEdit']      = false;
                $foo['full_name']    = $wh_order['full_name'];
                $foo['country']      = $wh_order['country'];
                $foo['state']        = $wh_order['state'];
                $foo['city']         = $wh_order['city'];
                $foo['cpf']          = $wh_order['cpf'] ?? NULL;
                $foo['cpf2']         = $wh_order['cpf2'] ?? NULL;
                $foo['street']       = $wh_order['address'];
                $foo['zip']          = $wh_order['zip'];
                $foo['phone_number'] = $wh_order['phone_number'];
                $foo['ordered']      = $wh_order['items']; //данные с базы склада
                $foo['redirect']     = $wh_order['redirect'];

            } elseif( isset( $wh_order['ordered'] ) ) {

                $foo['ordered'] = $wh_order['ordered'];
            }

            $items = [];

            if( ! empty( $foo['items'] ) ) foreach( $foo['items'] as $k => $item ) {

                $items[ $item['id'] ] = $item;
                $items[ $item['id'] ]['has_sync'] = false;
            }

            if( ! empty( $foo['ordered'] ) ) foreach( $foo['ordered'] as $item ) {

                $items[ $item['order_id'] ]['has_sync'] = true;
            }


            $foo['items'] = array_values( $items );

            $foo['order_number'] = false;

            return $foo;
        }

        $order = new adsOrders( 0, false );

        return $order->actions( [
            'ads_action' => 'order_details',
            'args'       => [
                'id' => (int) $order_id
            ]
        ] );
    }


    protected function place_order( $data, $auto = false ) : array {

        $order = new adsOrders( (int) $data['order_id'], true );

        $countries = ads_shipping_countries();

        $full_name = isset( $data['full_name'] ) && ! empty( $data['full_name'] ) ?
            trim( strip_tags( sanitize_text_field( $data['full_name'] ) ) ) : false;

        $country = isset( $data['country'] ) && isset( $countries[ $data['country'] ] ) ?
            $data['country'] : false;

        $state = isset( $data['state'] ) ?
            trim( strip_tags( sanitize_text_field( $data['state'] ) ) ) : '';

        $city = isset( $data['city'] ) && ! empty( $data['city'] ) ?
            trim( strip_tags( sanitize_text_field( $data['city'] ) ) ) : false;

        $cpf = isset( $data['cpf'] ) && ! empty( $data['cpf'] ) ?
            trim( strip_tags( sanitize_text_field( $data['cpf'] ) ) ) : false;+

        $cpf2 = isset( $data['cpf2'] ) && ! empty( $data['cpf2'] ) ?
            trim( strip_tags( sanitize_text_field( $data['cpf2'] ) ) ) : false;

        $street = isset( $data['street'] ) && ! empty( $data['street'] ) ?
            trim( strip_tags( sanitize_text_field( $data['street'] ) ) ) : false;

        $zip = isset( $data['zip'] ) && ! empty( $data['zip'] ) ?
            trim( strip_tags( sanitize_text_field( $data['zip'] ) ) ) : false;

        $phone_number = isset( $data['phone_number'] ) && ! empty( $data['phone_number'] ) ?
            trim( strip_tags( sanitize_text_field( $data['phone_number'] ) ) ) : '';

        $description = isset( $data['description'] ) && ! empty( $data['description'] ) ?
            trim( strip_tags( sanitize_text_field( $data['description'] ) ) ) : '';

        if(
            ! $full_name ||
            ! $country ||
            ! $city ||
            ! $street ||
            ! $zip
        ) {
            return [
                'error' => __( 'Some fields are not filled', 'sel' )
            ];
        }

        if( ! isset( $data['ordered'] ) || ! is_array( $data['ordered'] ) || empty( $data['ordered'] ) ) {
            return [
                'error' => __( 'No products selected', 'sel' )
            ];
        }

        if( !empty( $order->get_hash() ) ) {

            $items = [];

            foreach( $data['ordered'] as $item ) {

                $order_id   = isset( $item['order_id'] ) && ! empty( $item['order_id'] ) ? (int) $item['order_id'] : false;
                $product_id = isset( $item['id'] ) && ! empty( $item['id'] ) ? (int) $item['id'] : false;
                $quantity   = isset( $item['quantity'] ) && ! empty( $item['quantity'] ) ? (int) $item['quantity'] : false;
                $sku        = isset( $item['sku'] ) && ! empty( $item['sku'] ) && is_array( $item['sku'] ) ? $item['sku'] : false;

                if( ! $order_id || ! $product_id || ! $quantity ) {
                    return [
                        'error' => __( 'One of product has bad data, check it', 'sel' )
                    ];
                }

                $items[] = [
                    'order_id'   => $order_id,
                    'product_id' => $product_id,
                    'quantity'   => $quantity,
                    'sku'        => $sku,
                ];
            }

            $foo = [
                'hash'         => $order->get_hash(),
                'order_number' => $order->get_trans_id(),
                'email'        => $order->get_email(),
                'full_name'    => $full_name,
                'country'      => $country, //две буквы
                'state'        => $state,
                'city'         => $city,
                'cpf'          => $cpf,
                'cpf2'         => $cpf2,
                'street'       => $street,
                'zip'          => $zip,
                'phone_number' => $phone_number,
                'description'  => $description,
                'items'        => $items,
                'order_amount' => ads_floatvalue( $order->get_amount_clean() ),
            ];

            $data = $this->call( 'place_order', $foo );

            if( isset( $data['error'] ) ) {
                return $data;
            } elseif( isset( $data['status'] ) && $data['status'] != 'success' ) {
                return [ 'error' => $data['message'] ];
            }

            $fulfillment = $order->get_fulfillment();

            if( ! in_array( $order->get_fulfillment(), ['processing', 'shipped', 'completed', 'refund'] ) ) {

                $order->change_fulfillment( $order->get_trans_id(), 'processing' );

                $fulfillment = 'processing';
            }

            if( isset( $data['data']['hash'] ) ) {
                foreach ($items as $item) {

                    $this->setOrderIdSellvia( $item['order_id'], $data['data']['hash'] );
                }
            }

            do_action(
                'add_log',
                $data,
                $auto ? __( 'Order was placed on Sellvia automatically.', 'sel' ) : __( 'Order was placed on Sellvia.', 'sel' ),
                'order',
                'success',
                0,
                $order->get_hash()
            );

            return [
                'fulfillment' => $fulfillment,
                'message'     => $data['message'],
                'redirect'    => $data['data']['redirect']
            ];
        }

        return [
            'error' => __( 'Unable to Place an Order', 'sel' )
        ];
    }

    /**
     * Получаем заказ со склада при его наличии возвращается массив с данными по заказу
     * и переключатель на редактирование должен быть поставлен в режим false
     *
     * @param $order
     * @return array|false
     */
    private function findOrder( $order ) {

        $foo = [];
        if( ! empty( $order['items'] ) ) foreach( $order['items'] as $item ) {

            $post_id = get_post_meta( $item['post_id'], 'sellvia_post_id', true );
            if( ! $post_id )
                continue;

            $sku = ! empty( $item['details'] ) ? array_map( function( $s ) {
                return $s['prop_id'] . ':' . $s['sku_id'];
            }, $item['details'] ) : [];

            $foo[] = [
                'order_id'   => $item['id'],
                'quantity'   => $item['quantity'],
                'product_id' => $post_id,
                'sku'        => $sku
            ];
        }

        $data = $this->call( 'find_order', [
            'hash'       => $order['hash'],
            'items'      => $foo,
        ] );

        if( is_array( $data ) ) {

            if( isset( $data['status'] ) ) {

                if( $data['status'] != 'success' )
                    return [ 'error' => $data['message'] ];

                return $data['data'];

            } elseif( isset( $data['error'] ) ) {

                return $data;
            }
        }

        return [ 'error' => __( 'Request to Sellvia is wrong', 'sel' ) ];
    }

    public function place_order_automatically( $pay ){

        if( empty( $this->fulfillment ) || !isset( $this->fulfillment['auto'] ) || intval( $this->fulfillment['auto'] ) !== 1 ){
            return false;
        }

        if( !isset( $pay->id ) || empty( $pay->id ) ){
            return false;
        }

        $order_id = $pay->id;

        $order_data = $this->order_data( $order_id );

        if( empty( $order_data['ordered'] ) || ( isset( $order_data['already'] ) && $order_data['already'] === true ) ){
            return false;
        }

        $data = [
            'order_id'     => $order_id,
            'full_name'    => $order_data['full_name'],
            'country'      => $order_data['country'],
            'state'        => $order_data['state'],
            'city'         => $order_data['city'],
            'cpf'          => $order_data['cpf'],
            'cpf2'         => $order_data['cpf2'],
            'street'       => $order_data['street'],
            'zip'          => $order_data['zip'],
            'phone_number' => $order_data['phone_number'],
            'description'  => $order_data['description'],
            'ordered'      => $order_data['ordered'],
        ];

        $this->place_order( $data, true );
    }

    public function method_set_order_tracking( $data ) {

        if( empty( $data['items'] ) || empty( $data['order_hash'] ) ){
            return [ 'error' => __( 'Request to Sellvia is wrong', 'sel' ) ];
        }

        $count = 0;
        foreach ( $data['items'] as $item ){
            $response = $this->setOrderTip( $item['tracking_number'], $item['orders_item_id'], $data['customer_hash'], $data['order_hash'] );

            if( $response ){
                $count++;
            }
        }

        return [
            'status' => 'success',
            'message' => __( 'Tracking number was set for ', 'sel' ) . $count . ' ' . _n( 'item', 'items', $count, 'sel' )
        ];
    }

    private function setOrderIdSellvia( $order_item_id, $sellvia_order_hash ){

        global $wpdb;

        return $wpdb->update(
            $wpdb->ads_orders_item,
            [ 'order_number' => $sellvia_order_hash ],
            [ 'id' => $order_item_id ],
            [ '%s' ],
            [ '%d' ]
        );
    }

    private function setOrderTip( $tip, $orders_item_id, $order_hash, $income_order_number ){

        if( empty( $tip ) || empty( $orders_item_id ) || empty( $order_hash ) || empty( $income_order_number ) ) return false;

        global $wpdb;

        $trans_id = $this->findTransaction( $order_hash );

        if( empty( $trans_id ) ) return false;

        $response = $wpdb->update(
            $wpdb->ads_orders_item,
            [ 'tip' => $tip, 'order_number' => $income_order_number ],
            [ 'id' => $orders_item_id, 'trans_id' => $trans_id ],
            [ '%s', '%s' ],
            [ '%s', '%d' ]
        );

        if( $response ){
            do_action('ads_set_order_tip', $tip, $income_order_number );
        }

        return $response;
    }

    private function findTransaction( $hash ) {

        global $wpdb;

        return $wpdb->get_var( $wpdb->prepare(
            "SELECT id FROM {$wpdb->payment_transaction} t WHERE t.hash = %s", $hash
        ) );
    }

    private function entity_decode( $str ){
        return stripslashes( html_entity_decode( $str, ENT_QUOTES ));
    }
}