<?php

namespace sellvia\Order;

class OrderWoo extends Order {

    public function __construct() {

        parent::__construct();
    }

    protected function order_data( $order_id ) : array {

        $order = wc_get_order( $order_id );
        $order_data  = $order->get_data();

        $shipping_key =
            ( empty( $order_data['shipping']['first_name'] ) && empty( $order_data['shipping']['last_name'] ) ) ||
            empty( $order_data['shipping']['address_1'] ) ||
            empty( $order_data['shipping']['city'] ) ||
            empty( $order_data['shipping']['postcode'] ) ||
            empty( $order_data['shipping']['country'] ) ? 'billing' : 'shipping';

        $shipping = $order_data[ $shipping_key ];

        $countries_obj = new \WC_Countries();
        $countries     = $this->prepare_options( $countries_obj->__get('countries') );

        $states = $countries_obj->get_states( $shipping['country'] );

        if( $order_data ) {

            $foo = [
                'already'         => false,
                'canEdit'         => true,
                'hash'            => $order_data['number'],
                'status'          => 'wc-' . $order_data['status'],
                'status_values'   => $this->prepare_options( wc_get_order_statuses() ),
                'status_colors'   => [
                    'wc-pending'    => 'info',
                    'wc-processing' => 'success',
                    'wc-on-hold'    => 'warning',
                    'wc-completed'  => 'info',
                    'wc-cancelled'  => 'danger',
                    'wc-refunded'   => 'danger',
                    'wc-failed'     => 'danger',
                ],
                'order_number'    => $order_data['number'],
                'email'           => $order_data['billing']['email'],
                'full_name'       => $shipping['first_name'] . ' ' . $shipping['last_name'],
                'country'         => $shipping['country'], //две буквы
                'state'           => isset( $states[ $shipping['state'] ] ) ? $states[ $shipping['state'] ] : $shipping['state'],
                'city'            => $shipping['city'],
                'cpf'             => '',
                'cpf2'            => '',
                'street'          => trim( $shipping['address_1'] . ' ' . $shipping['address_2'] ),
                'zip'             => trim( $shipping['postcode'] ),
                'phone_number'    => trim( $order_data['billing']['phone'] ),
                'description'     => $order->get_customer_note(),
                'items'           => [],
                'country_options' => $countries,
                'ordered'         => [], //данные с базы склада,
                'plugin'          => in_array( SELLVIA_PLUGIN, ['ADSW', 'WOO'] ) ? 'WOO' : SELLVIA_PLUGIN,
                'redirect'        => null,
            ];

            $order_items = $order->get_items();
            foreach( $order_items as $item_id => $item ) {

                $product = $item->get_product();

                if( ! $product )
                    continue;

                $type = $item->get_type();
                if( $type != 'line_item' )
                    continue;

                $imageUrl = '';

                $vars = [];
                // Only for product variation
                if( $product->is_type('variation') ) {

                    $var_id = $item->get_variation_id();

                    $imageUrl = get_the_post_thumbnail_url( $var_id );

                    $meta_sku = get_post_meta( $var_id, 'adswSKU', true );
                    if( $meta_sku )
                        $meta_sku = explode( ';', $meta_sku );

                    $i = 0;

                    $variation_attributes = $product->get_variation_attributes();
                    foreach( $variation_attributes as $attribute_taxonomy => $term_slug ) {

                        $aoo = [
                            'img' => ''
                        ];

                        $taxonomy = str_replace('attribute_', '', $attribute_taxonomy );

                        $aoo['sku_id'] = $taxonomy;

                        $attribute_name = wc_attribute_label( $taxonomy, $product );

                        if( taxonomy_exists( $taxonomy ) ) {

                            $attribute_tax = get_term_by( 'slug', $term_slug, $taxonomy );
                            if( $attribute_tax ) {

                                $attribute_value = $attribute_tax->name;
                                $aoo['prop_id'] = $attribute_tax->term_id;
                            } else {
                                $attribute_value = $term_slug;
                                $aoo['prop_id'] = $term_slug;
                            }

                        } else {
                            $attribute_value = $term_slug;
                            $aoo['prop_id'] = $term_slug;
                        }

                        $aoo['prop_title'] = $attribute_name;
                        $aoo['title'] = empty( $attribute_value ) ? 'None' : $attribute_value;

                        if( $meta_sku && isset( $meta_sku[ $i ] ) ) {
                            $m = explode( ':', $meta_sku[ $i ] );
                            $aoo['prop_id'] = $m[0];
                            $aoo['sku_id']  = $m[1];
                        }

                        $i++;

                        $vars[] = $aoo;

                    }

                    if( empty( $imageUrl ) )
                        $imageUrl = get_the_post_thumbnail_url( $item->get_product_id() );

                }

                if( empty( $imageUrl ) )
                    $imageUrl = get_the_post_thumbnail_url( $item->get_product_id() );

                $foo['items'][ $item_id ] = [
                    'id'            => $item_id,
                    'trans_id'      => $order_id,
                    'post_id'       => $item->get_product_id(),
                    'title'         => $item->get_name(),
                    'imageUrl'      => $imageUrl,
                    'quantity'      => $item->get_quantity(),
                    'currency_code' => $order->get_currency(),
                    'total_sum'     => $item->get_total(),
                    'details'       => $vars,
                    'has_sync'      => false
                ];
            }

            $foo['items'] = array_values( $foo['items'] );

            $wh_order = $this->findOrder( $foo );

            if( $wh_order && isset( $wh_order['hash'] ) ) {

                if( isset( $wh_order['error'] ) ) {
                    return $wh_order;
                }

                $foo['already']      = true;
                $foo['canEdit']      = false;
                $foo['full_name']    = $wh_order['full_name'];
                $foo['country']      = $wh_order['country'];
                $foo['state']        = $wh_order['state'];
                $foo['city']         = $wh_order['city'];
                $foo['cpf']          = $wh_order['cpf'] ?? NULL;
                $foo['cpf2']         = $wh_order['cpf2'] ?? NULL;
                $foo['street']       = $wh_order['address'];
                $foo['zip']          = $wh_order['zip'];
                $foo['phone_number'] = $wh_order['phone_number'];
                $foo['ordered']      = $wh_order['items']; //данные с базы склада
                $foo['redirect']     = $wh_order['redirect'];

            } elseif( isset( $wh_order['ordered'] ) ) {

                $foo['ordered'] = $wh_order['ordered'];
            }

            $items = [];

            if( ! empty( $foo['items'] ) ) foreach( $foo['items'] as $k => $item ) {

                $items[ $item['id'] ] = $item;
                $items[ $item['id'] ]['has_sync'] = false;
            }

            if( ! empty( $foo['ordered'] ) ) foreach( $foo['ordered'] as $item ) {

                $items[ $item['order_id'] ]['has_sync'] = true;
            }


            $foo['items'] = array_values( $items );

            $foo['order_number'] = false;

            return $foo;
        }

        return $order_data;
    }

    protected function place_order( $data, $auto = false ) : array {

        $order_id = (int) $data['order_id'];

        $countries_obj = new \WC_Countries();
        $countries     = $countries_obj->__get('countries');

        $full_name = isset( $data['full_name'] ) && ! empty( $data['full_name'] ) ?
            trim( strip_tags( sanitize_text_field( $data['full_name'] ) ) ) : false;

        $country = isset( $data['country'] ) && isset( $countries[ $data['country'] ] ) ?
            $data['country'] : false;

        $state = isset( $data['state'] ) ?
            trim( strip_tags( sanitize_text_field( $data['state'] ) ) ) : '';

        $city = isset( $data['city'] ) && ! empty( $data['city'] ) ?
            trim( strip_tags( sanitize_text_field( $data['city'] ) ) ) : false;

        $cpf = isset( $data['cpf'] ) && ! empty( $data['cpf'] ) ?
            trim( strip_tags( sanitize_text_field( $data['cpf'] ) ) ) : false;

        $cpf2 = isset( $data['cpf2'] ) && ! empty( $data['cpf2'] ) ?
            trim( strip_tags( sanitize_text_field( $data['cpf2'] ) ) ) : false;

        $street = isset( $data['street'] ) && ! empty( $data['street'] ) ?
            trim( strip_tags( sanitize_text_field( $data['street'] ) ) ) : false;

        $zip = isset( $data['zip'] ) && ! empty( $data['zip'] ) ?
            trim( strip_tags( sanitize_text_field( $data['zip'] ) ) ) : false;

        $phone_number = isset( $data['phone_number'] ) && ! empty( $data['phone_number'] ) ?
            trim( strip_tags( sanitize_text_field( $data['phone_number'] ) ) ) : '';

        $description = isset( $data['description'] ) && ! empty( $data['description'] ) ?
            trim( strip_tags( sanitize_text_field( $data['description'] ) ) ) : '';

        if(
            ! $full_name ||
            ! $country ||
            ! $city ||
            ! $street ||
            ! $zip
        )
            return [ 'error' => __( 'Some fields are not filled', 'sel' ) ];

        if( ! isset( $data['ordered'] ) || ! is_array( $data['ordered'] ) || empty( $data['ordered'] ) )
            return [ 'error' => __( 'No products selected', 'sel' ) ];

        $order = wc_get_order( $order_id );
        $order_data  = $order->get_data();

        if( $order_data ) {

            $items = [];

            foreach( $data['ordered'] as $item ) {

                $order_id   = isset( $item['order_id'] ) && ! empty( $item['order_id'] ) ? (int) $item['order_id'] : false;
                $product_id = isset( $item['id'] ) && ! empty( $item['id'] ) ? (int) $item['id'] : false;
                $quantity   = isset( $item['quantity'] ) && ! empty( $item['quantity'] ) ? (int) $item['quantity'] : false;
                $sku        = isset( $item['sku'] ) && ! empty( $item['sku'] ) && is_array( $item['sku'] ) ? $item['sku'] : [];

                if( ! $order_id || ! $product_id || ! $quantity ) {
                    return [ 'error' => __( 'One of product has bad data, check it', 'sel' ) ];
                }

                $items[] = [
                    'order_id'   => $order_id,
                    'product_id' => $product_id,
                    'quantity'   => $quantity,
                    'sku'        => $sku,
                ];
            }

            $foo = [
                'hash'         => $order_data['number'],
                'order_number' => $order_data['number'],
                'email'        => $order_data['billing']['email'],
                'full_name'    => $full_name,
                'country'      => $country, //две буквы
                'state'        => $state,
                'city'         => $city,
                'cpf'          => $cpf,
                'cpf2'         => $cpf2,
                'street'       => $street,
                'zip'          => $zip,
                'phone_number' => $phone_number,
                'description'  => $description,
                'items'        => $items,
                'order_amount' => $order->get_total(),
            ];

            $data = $this->call( 'place_order', $foo );

            if( isset( $data['error'] ) ) {
                return $data;
            } elseif( isset( $data['status'] ) && $data['status'] != 'success' ) {
                return [ 'error' => $data['message'] ];
            }

            $order->set_status( 'wc-sellvia-placed' );
            $order->add_order_note( $auto ? __( 'Order was placed on Sellvia automatically.', 'sel' ) : __( 'Order was placed on Sellvia.', 'sel' ) );

            $order->save();

            return [
                'message'  => $data['message'],
                'redirect' => $data['data']['redirect']
            ];
        }

        return [ 'error' => __( 'Order not found', 'sel' ) ];
    }

    /**
     * Получаем заказ со склада при его наличии возвращается массив с данными по заказу
     * и переключатель на редактирование должен быть поставлен в режим false
     *
     * @param $order
     * @return array|false
     */
    private function findOrder( $order ) {

        $foo = [];
        if( ! empty( $order['items'] ) ) foreach( $order['items'] as $item ) {

            $post_id = get_post_meta( $item['post_id'], 'sellvia_post_id', true );
            if( ! $post_id )
                continue;

            $sku = ! empty( $item['details'] ) ? array_map( function( $s ) {
                return $s['prop_id'] . ':' . $s['sku_id'];
            }, $item['details'] ) : [];

            $foo[] = [
                'order_id'   => $item['id'],
                'quantity'   => $item['quantity'],
                'product_id' => $post_id,
                'sku'        => $sku
            ];
        }

        $data = $this->call( 'find_order', [
            'hash'       => $order['hash'],
            'items'      => $foo,
        ] );

        if( is_array( $data ) ) {

            if( isset( $data['status'] ) ) {

                if( $data['status'] != 'success' )
                    return [ 'error' => $data['message'] ];

                return $data['data'];

            } elseif( isset( $data['error'] ) ) {

                return $data;
            }
        }

        return [ 'error' => __( 'Request to Sellvia is wrong', 'sel' ) ];
    }

    public function place_order_automatically( $order_id ){

        if( empty($order_id) || empty( $this->fulfillment ) || !isset( $this->fulfillment['auto'] ) || intval( $this->fulfillment['auto'] ) !== 1 ){
            return false;
        }

        $order_data = $this->order_data( $order_id );

        if( empty( $order_data['ordered'] ) || ( isset( $order_data['already'] ) && $order_data['already'] === true ) ){
            return false;
        }

        $data = [
            'order_id'     => $order_id,
            'full_name'    => $order_data['full_name'],
            'country'      => $order_data['country'],
            'state'        => $order_data['state'],
            'city'         => $order_data['city'],
            'cpf'          => $order_data['cpf'],
            'cpf2'         => $order_data['cpf2'],
            'street'       => $order_data['street'],
            'zip'          => $order_data['zip'],
            'phone_number' => $order_data['phone_number'],
            'description'  => $order_data['description'],
            'ordered'      => $order_data['ordered'],
        ];

        $this->place_order( $data );
    }

    public function method_set_order_tracking( $data ) {
        return [
            'status' => 'success',
            'message' => __( 'WooCommerce does not have tracking support', 'sel' )
        ];
    }

    protected function prepare_options( $data = [] ) {

        $foo = [];

        foreach( $data as $key => $val )
            $foo[] = [
                'value' => $key,
                'title' => $val
            ];

        return $foo;
    }
}