<?php


namespace sellvia;


class Preset {

    public function get_preset_info( $preset_id ){

        $info = self::call( '/sellvia_customer_plugin/preset_info', [
            'preset_id'  => $preset_id,
        ]);

        $product_list = isset( $info['data']['current_preset']['product_list'] ) ? $info['data']['current_preset']['product_list'] : [];

        $category_list = isset( $info['data']['current_preset']['category_list'] ) ? $info['data']['current_preset']['category_list'] : [];

        if( !empty( $category_list ) ){
            foreach( $category_list as &$category ){
                $category['selected'] = false;
            }
        }

        if( !empty($product_list) ){

            $in = array_map( function ($it){ return $it['catalog_product_id']; }, $product_list);

            $in = implode( ',', $in );

            global $wpdb;

            $already_list = $wpdb->get_results(
                "SELECT post_id, meta_value as sellvia_post_id
                    FROM {$wpdb->prefix}postmeta
                    WHERE meta_key = 'sellvia_post_id' AND meta_value IN({$in})",
                ARRAY_A
            );

            $already_catalog_product_ids = [];

            if( count( $already_list ) > 0 ){
                foreach ( $already_list as $item ){
                    $already_catalog_product_ids[ $item['sellvia_post_id'] ] = $item['post_id'];
                }
            }

            $already_catalog_product_id = $already_list;

            foreach ( $product_list as &$pr ){

                if( array_key_exists( $pr['catalog_product_id'], $already_catalog_product_ids) ){
                    $pr['already'] = true;
                    $pr['failed'] = false;
                    $pr['post_id'] = $already_catalog_product_ids[ $pr['catalog_product_id'] ];
                    $pr['need_import'] = true;

                }else{
                    $pr['already'] = false;
                    $pr['failed'] = false;
                    $pr['post_id'] = null;
                    $pr['need_import'] = true;
                }

                if( !empty( $pr['category_ids'] ) ){
                    foreach( $pr['category_ids'] as $category_id ){
                        $cat_key = array_search( $category_id, array_column( $category_list, 'id' ) );

                        if( $cat_key !== false ){
                            $category_list[$cat_key]['selected'] = true;
                        }
                    }
                }
            }
        }

        return [
            'current_preset' => [
                'id'            => isset( $info['data']['current_preset']['id'] ) ? $info['data']['current_preset']['id'] : null,
                'description'   => isset( $info['data']['current_preset']['description'] ) ? $info['data']['current_preset']['description'] : null,
                'category_list' => $category_list,
                'product_list'  => $product_list,
            ],
            'preset_list' => isset( $info['data']['preset_list'] ) ? $info['data']['preset_list'] : [],
            'super_key'   => $info['data']['super_key'],
        ];

    }

    static function call( $path, $args = [] ) {

        $keys = sellvia_get_api_keys();

        $public_key  = $keys['public_key'];
        $secret_key  = $keys['secret_key'];

        $auth_args = [
            'public_key'  => $public_key,
            'signature'   => md5( $public_key . $secret_key ),
            'domain'      => get_home_url(),
            'plugin'      => 'sellvia',
        ];

        $args = array_merge( $args, $auth_args );

//        var_dump( json_encode($args) );

        $response = wp_remote_post( SELLVIA_REQUEST_PATH . $path, [
            'method'    => 'POST',
            'timeout'   => 120,
            'sslverify' => false,
            'headers'   => [],
            'body'      => json_encode( $args )
        ] );

        if( is_wp_error( $response ) )
            return [ 'error' => $response->get_error_message() ];

        if( ! wp_remote_retrieve_response_code( $response ) == '200' )
            return [ 'error' => __( 'The response from the server has not been received', 'sel' ) ];

        return json_decode( wp_remote_retrieve_body( $response ), true );
    }

}