<?php


namespace sellvia;


class ProductActivities {

    private $sql_select = '';

    public function actions( $post ) {

        if( isset( $post['sellvia_action'] ) && current_user_can( 'activate_plugins' ) ) {

            $action = 'action_' . $post['sellvia_action'];
            $args        = isset( $post['args'] ) ? $post['args'] : $post;
            $data        = [];

            if( is_array( $args ) )
                $data = $args;
            else
                parse_str( $args, $data );


            if( method_exists( $this, $action ) ) {

                return $this->$action( $data );
            }
        }

        return [ 'error' => __( 'Undefined action', 'sel' ) ];
    }

    private function action_list( $args ){

        if( ! isset( $args['page'] ) ){
            $args['page'] = 1;
        }

        if( ! isset( $args['pageSize'] ) ){
            $args['pageSize'] = 20;
        }

        $defaults = [
            'page'  => 1,
            'pageSize' => 20,
            'type'      => '',
            'from'      => false,
            'to'        => false,
            's'         => '',
            'hash'      => false,
            'post_id'   => false,
        ];

        $args  = sellvia_parse_args( $defaults, $args );

        $items = $this->get_list( $args[ 'page' ], $args[ 'pageSize' ], $args );

        $total = $this->get_count_list();

        if( ! $items ) {
            return [
                'result'  => true,
                'items' => [],
                'total' => $total,
                'total_shown' => 0,
                'total_pages' => ceil( (int) ($total) / $args[ 'pageSize' ] ),
            ];
        }



        return [
            'result'  => true,
            'items' => $this->prepareList( $items ),
            'total' => $total,
            'total_shown' => count($items),
            'total_pages' => ceil( (int) ($total) / $args[ 'pageSize' ] ),
        ];


    }

    private function action_delete( $args ){

        if( ! isset( $args['ids'] ) || !is_array( $args['ids'] ) || empty( $args['ids'] ) ) {
            return[
                'result' => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' =>  __( 'Nothing to remove', 'sel' )
            ];
        }

        $in = implode( ',', array_map( 'absint', $args['ids'] ) );

        global $wpdb;

        $counter = $wpdb->query( "DELETE FROM {$wpdb->sellvia_activities} WHERE ID IN($in)" );

        $counter = $counter !== false ? $counter : 0;

        return[
            'result' => true,
            'title' => __( 'Success', 'sel' ),
            'message' => $counter . ' ' . _n( 'activity has', 'activities have', $counter, 'sel' ) . ' '. __( 'been removed', 'sel' )
        ];
    }

    private function action_delete_all( $args ){

        global $wpdb;

        $where = '1 = 1';
        if( isset( $args['post_id'] ) ) {
            $post_id = $args['post_id'];
            $where .= " AND post_id = {$post_id}";
        }

        $counter = $wpdb->query( "DELETE FROM {$wpdb->sellvia_activities} WHERE {$where}" );

        $counter = $counter !== false ? $counter : 0;

        return[
            'result' => true,
            'title' => __( 'Success', 'sel' ),
            'message' => $counter . ' ' . _n( 'activity has', 'activities have', $counter, 'sel' ) . ' '. __( 'been removed', 'sel' )
        ];
    }

    protected function get_list( $page = 1, $pageSize = 20, $args = [] ) {

        global $wpdb;

        $where    = '1 = 1';

        $defaults = [
            'type'    => '',
            'from'    => '',
            'to'      => '',
            's'       => '',
            'hash'    => false,
            'post_id' => false
        ];

        $types = $this->notify_types();
        $args  = sellvia_parse_args( $defaults, $args );

        if( ! isset( $types[ $args[ 'type' ] ] ) )
            unset( $args[ 'type' ] );

        if( ! $args[ 'hash' ] )
            unset( $args[ 'hash' ] );

        if( ! $args[ 'post_id' ] )
            unset( $args[ 'post_id' ] );

        $between = empty( $args[ 'from' ] ) || empty( $args[ 'to' ] ) ? '' : sprintf(
            "AND sel_act.date BETWEEN '%s' AND '%s'",
            date( 'Y-m-d 00:00:00', strtotime( $args[ 'from' ] ) ),
            date( 'Y-m-d 23:59:59', strtotime( $args[ 'to' ] ) )
        );

        $s = trim( esc_attr( $args[ 's' ] ) );
        $search = '';
        $join   = '';
        $select = '';

        //$join   .= " LEFT JOIN {$wpdb->payment_transaction} tran ON sel_act.hash = tran.hash ";
        //$select .= ',tran.id AS trans_id';

        if( $s != '' ) {
            $search = " AND ( sel_act.anonce LIKE '%{$s}%' OR sel_act.hash LIKE '%{$s}%' OR posts.post_title LIKE '%{$s}%' OR tran.email LIKE '%{$s}%') ";
            $join .= " LEFT JOIN {$wpdb->posts} posts ON sel_act.post_id = posts.ID ";
            $select .= ' ,posts.post_title';
        }

        unset( $args[ 'from' ], $args[ 'to' ], $args[ 's' ] );

        foreach( $args as $key => $val ) {
            $where .= " AND sel_act.{$key} = '{$val}'";
        }

        $types = "'" . implode("','", array_keys($types)) . "'";
        $where .= " AND sel_act.type in ({$types}) ";

        $limit = $page * $pageSize - $pageSize;
        if( $limit < 0 )
            $limit = 0;

        $this->sql_select = "SELECT COUNT(sel_act.id) FROM {$wpdb->sellvia_activities} sel_act {$join} WHERE {$where} {$between} {$search}";

        return $wpdb->get_results(
            "SELECT sel_act.*{$select} FROM {$wpdb->sellvia_activities} sel_act {$join}
			 WHERE {$where} {$between} {$search} ORDER BY sel_act.date DESC LIMIT {$limit}, {$pageSize}"
        );
    }

    protected function get_count_list() {

        global $wpdb;

        return $wpdb->get_var( $this->sql_select );
    }

    protected function prepareList( $items ) {

        $list_types = $this->notify_types();

        if( ! $items ) {
            return false;
        }

        foreach( $items as &$i ) {

            $content = maybe_unserialize( $i->product_data );

            if( $content ) {
                $i->product_data = $content;
            }

            $i->params = $this->params( $i );
            $i->date   = sellvia_format_date( $i->date );
            $i->e_type = isset( $list_types[ $i->type ] ) ? $list_types[ $i->type ] : $i->type;
        }

        return $items;
    }

    protected function params( $i ) {

//        $too = [
//            'isProduct' => false,
//            'isOrder'   => false
//        ];

        if( $i->type == 'product' ) {

            $post = get_post( $i->post_id );

            if( $post ) {
                $too[ 'product_link' ]  = get_permalink( $post );
                $too[ 'product_title' ] = $post->post_title;
            }

//            $too[ 'isProduct' ] = true;

//        } elseif( $i->type == 'order' ) {
//
//            $too[ 'trans_id' ] = $i->trans_id;
//            $too[ 'isOrder' ]  = true;
//            $too[ 'isNotify' ] = false;
//
//            $i->product_data = (array) $i->product_data;
//
//            if( isset( $i->product_data[ 'type' ] ) && $i->product_data[ 'type' ] == 'notify' ) {
//                $too[ 'isNotify' ] = true;
//            }
        }

        return $too;
    }

    public function notify_types() {

        $foo = [
            'product' => __( 'Product', 'sel'),
        ];

        return apply_filters( 'sellvia_notify_types', $foo );

    }
}