<?php


namespace sellvia\ProductInstaller;


class ProductInstaller{

    public static $default_setting_reset = [
        'title'       => 0,
        'featured'    => 0,
        'gallery'     => 0,
        'description' => 0,
        'short'       => 0,
        'variant'     => 1,
        'reviews'     => 0,
    ];

    protected $reset_settings = [
        'title'       => 0,
        'featured'    => 0,
        'gallery'     => 0,
        'description' => 0,
        'short'       => 0,
        'variant'     => 0,
        'reviews'     => 0,
    ];

    protected function get_products_list( $page = 1, $limit = 20, $search = [], $imported_ids = [] ){

        return self::call( '/sellvia_customer_plugin/productsList', [
            'page'         => $page,
            'limit'        => $limit,
            'search'       => $search,
            'imported_ids' => $imported_ids
        ]);
    }

    public function connect_product( $post_id, $sellvia_product_url ){

        $sellvia_post_id = get_post_meta( $post_id, 'sellvia_post_id', true );

        if( ! empty( $sellvia_post_id ) ) {
            return[
                'success' => false,
                'message' => __('This product is already connected', 'sel'),
            ];
        }

        $request = self::call( '/sellvia_customer_plugin/connectProduct', [
            'sellvia_product_url' => $sellvia_product_url,
        ]);

        if( isset( $request['status'] ) && $request['status'] === 'success' && isset($request['data']) && isset( $request['data']['sellvia_post_id'] ) ){

            update_post_meta( $post_id, 'sellvia_post_id', $request['data']['sellvia_post_id'] );
            update_post_meta( $post_id, 'sellvia_need_update', 1 );

            return[
                'success' => true,
                'message' => __('Product has been connected to Sellvia', 'sel'),
            ];
        }

        return[
            'success' => false,
            'message' => isset($request['message']) ? $request['message'] : __('Something went wrong', 'sel'),
        ];
    }

    public function get_product_cat_list() {

        $categories = get_terms( 'product_cat', [ 'hide_empty' => false ] );

        if( ! $categories || is_wp_error( $categories ) ) {
            return [];
        }

        $terms = [];
        $this->sort_terms_hierarchically( $categories, $terms );

        if( count( $terms ) == 0 )
            return [];

        $foo = $this->prepare_terms( $terms );
        $foo = sellvia_prepare_options( $foo );

        return $foo;
    }

    public function get_premium_all(){

        return self::call( '/sellvia_customer_plugin/get_premium_all', [] );
    }

    protected function sort_terms_hierarchically( array &$cats, array &$into, $parentId = 0, $depth = '' ) {

        $str = '–';
        $str = $depth == '' ? '' : $depth . $str . ' ';

        foreach( $cats as $i => $cat ) {

            if( $cat->parent == $parentId ) {

                $cat->name = $str . htmlspecialchars_decode( $cat->name );

                $into[ $cat->term_id ] = $cat;

                unset( $cats[ $i ] );
            }
        }

        $depth .= '–';

        foreach( $into as $topCat ) {

            $topCat->children = [];

            $this->sort_terms_hierarchically( $cats, $topCat->children, $topCat->term_id, $depth );
        }
    }

    protected function prepare_terms( $terms ) {

        $foo = [];

        foreach( $terms as $term ) {

            $foo[ $term->term_id ] = $term->name;

            if( isset( $term->children ) && count( $term->children ) ) {
                $too = $this->prepare_terms( $term->children );

                if( count( $too ) )
                    $foo += $too;
            }
        }

        return $foo;
    }

	protected function existsReview( $cd ) {

		global $wpdb;

		$sql = $wpdb->prepare(
			"SELECT count(comment_ID) as countComment FROM `{$wpdb->comments}`
             WHERE `comment_post_ID` = '%d' AND `comment_date` = '%s' AND `comment_author` = '%s'",
			$cd['comment_post_ID'],
			$cd['comment_date'],
			$cd['comment_author']
		);

		$count = $wpdb->get_var( $sql );
		return $count > 0;
	}

	protected function getImageById( $id, $size = 'thumbnail' ) {

		$img = wp_get_attachment_image_src( $id, $size, false );

		if ( $img ) {
			return $img[ 0 ];
		}

		return false;
	}

    static function call( $path, $args = [] ) {

        $keys = sellvia_get_api_keys();

        $public_key  = $keys['public_key'];
        $secret_key  = $keys['secret_key'];

        $auth_args = [
            'public_key'  => $public_key,
            'signature'   => md5( $public_key . $secret_key ),
            'domain'      => get_home_url(),
            'plugin'      => 'sellvia',
        ];

        $args = array_merge( $args, $auth_args );

//        var_dump( SELLVIA_REQUEST_PATH . $path );
//        var_dump( json_encode($args) );

        $response = wp_remote_post( SELLVIA_REQUEST_PATH . $path, [
            'method'    => 'POST',
            'timeout'   => 120,
            'sslverify' => false,
            'headers'   => [],
            'body'      => json_encode( $args )
        ] );

//        var_dump( $response );

        if( is_wp_error( $response ) )
            return [ 'error' => $response->get_error_message() ];

        if( ! wp_remote_retrieve_response_code( $response ) == '200' )
            return [ 'error' => __( 'The response from the server has not been received', 'sel' ) ];

        return json_decode( wp_remote_retrieve_body( $response ), true );
    }

    public function delete_product_images( $post_id ){

        if( !in_array( get_post_type( $post_id ), [ 'product' ] ) ) {
            return;
        }

        $attachments = get_children( [ 'post_type' => 'attachment', 'post_parent' => $post_id ] );
        if( $attachments ) {
            foreach( $attachments as $attachment ) wp_delete_attachment( $attachment->ID, true );
        }

    }

    public function remove_product( $post_ids ){

        add_action( 'before_delete_post', [ $this, 'delete_product_images' ], 10 );

        return wp_delete_post( $post_ids, true);
    }

    public function get_reset_settings(){

        return sellvia_parse_args( self::$default_setting_reset,  [] );
    }

    public function get_reset_labels(){

        return [
            'title'       => __( 'Product Title', 'sel' ),
            'featured'    => in_array( SELLVIA_PLUGIN, ['ADSW', 'WOO'] ) ? __( 'Product image', 'sel' ) : __( 'Featured image', 'sel' ),
            'gallery'     => __( 'Gallery images', 'sel' ),
            'description' => __( 'Product description', 'sel' ),
            'short'       => __( 'Product short description', 'sel' ),
            'variant'     => in_array( SELLVIA_PLUGIN, ['ADSW', 'WOO'] ) ? __( 'Product variations', 'sel' ) : __( 'Product variants', 'sel' ),
            'reviews'     => __( 'Product reviews', 'sel' ),
        ];
    }

    public function get_reset_help(){

        return [
            'title'       => '',//__( 'Title image lorem ipsum', 'sel' ),
            'featured'    => '',//__( 'Featured image lorem ipsum', 'sel' ),
            'gallery'     => '',//__( 'Gallery lorem ipsum', 'sel' ),
            'description' => '',//__( 'Description lorem ipsum', 'sel' ),
            'short'       => '',//__( 'Short description lorem ipsum', 'sel' ),
            'variant'     => '',//__( 'Attributes & Variations lorem ipsum', 'sel' ),
            'reviews'     => '',//__( 'Reviews lorem ipsum', 'sel' ),
        ];
    }
}
