<?php

namespace sellvia\ProductInstaller;

use sellvia\ImportProduct\ImportProductWoo as ImportProductWoo;
use sellvia\Media\MediaWoo;

class ProductInstallerWoo extends ProductInstaller{

    public function products_list( $page = 1, $limit = 20, $search = [] ){

        $imported_ids = [];

        if( in_array( $search['sorting'], [ 'imported', 'not_imported' ] ) ){

            global $wpdb;

            $imported_ids = $wpdb->get_col(
                "SELECT m.meta_value FROM {$wpdb->prefix}postmeta m
                            WHERE m.meta_key = 'sellvia_post_id'");
        }

        $result = $this->get_products_list( $page, $limit, $search, $imported_ids );

        if( !is_array($result) || $result['status'] !== 'success' ){
            return[
                'success' => false,
                'data' => $result,
            ];
        }

        $products = isset( $result['data']['products'] ) ? $result['data']['products'] : [];

        if(!empty($products)){

            global $wpdb;

            foreach ( $products as &$product ){

                $post_id  = $wpdb->get_var(
                    "SELECT m.post_id FROM {$wpdb->prefix}postmeta m
                            WHERE m.meta_key = 'sellvia_post_id' AND m.meta_value = '{$product['sellvia_post_id']}'" );

                $product['already'] = $post_id !== NULL;
                $product['need_import'] = false;

                $product['already_link'] = false;
                $product['loading'] = false;
                $product['message'] = '';
                $product['product_cat'] = [];

                if( $product['already'] ){
                    $product['already_link'] = $post_id ? get_permalink( $post_id ) : false;
                    $product['post_id'] = $post_id ? $post_id : false;

                    $categories = wp_get_post_terms( $product['post_id'], 'product_cat' );

                    if( !empty($categories) ){
                        $product['product_cat'] = array_map( function ($item){ return $item->term_id; }, $categories);
                    }
                }
            }
        }

        $page = isset( $result['data']['page'] ) ? $result['data']['page'] : [];
        $total = isset( $result['data']['total'] ) ? $result['data']['total'] : [];

        return [
            'success' => true,
            'data' => [
                'page'               => intval($page),
                'total'              => intval($total),
                'total_shown'        => count($products),
                'products'           => $products,
                'niches'             => $result['data']['niches'],
                'niches_placeholder' => __( 'All', 'sel' ) . ' (' . intval($result['data']['total']) . ')',
                'has_private'        => isset( $result['data']['has_private'] ) ? boolval ( $result['data']['has_private'] ) : false,
                'has_premium'        => isset( $result['data']['has_premium'] ) ? boolval ( $result['data']['has_premium'] ) : false,
            ]
        ];
    }

    public function import_product( $sellvia_post_id, $product_cat = [], $preset_id = null ){

        $import = $this->import( $sellvia_post_id, $product_cat, $preset_id );

        if( isset( $import['error'] ) ){
            return[
                'success' => false,
                'message' =>  $import['error'],
            ];
        }

        return[
            'success' => true,
            'data' => [
                'product' => $import
            ]
        ];
    }

    private function import( $sellvia_post_id, $product_cat = [], $preset_id = null ){

        $currency    = \get_woocommerce_currency();

        $images_settings = sellvia_get_images_settings();

        $request = self::call( '/sellvia_customer_plugin/importProduct', [
            'sellvia_post_id' => $sellvia_post_id,
            'currency'        => $currency,
            'preset_id'       => $preset_id,
            'action'          => 'import',
            'webp'            => intval( $images_settings['webp'] )
        ]);

        if( isset( $request['status'] ) && $request['status'] !== 'success' ) {
            return [
                'error' => $request['message'] ?? __('Something went wrong', 'sel'),
            ];
        }

        if( !isset ( $request['data'] ) || !isset ( $request['data']['data'] ) || !isset ($request['data']['post_id'] ) || empty($request['data']['post_id']) ){
            return[
                'error' =>  __('Product not found', 'sel'),
            ];
        }

        $importProduct = new ImportProductWoo();

        $data = $request['data']['data'];
        $data['sellvia_post_id'] = $request['data']['post_id'];

        $result = $importProduct->importProduct( $data, $product_cat );

        if( isset( $result['error'] ) ){
            return[
                'error' =>  __('This product already exist', 'sel'),
            ];
        }

        $images = $importProduct->process_images();

        return [
            'sellvia_post_id' => $sellvia_post_id,
            'post_id'         => $importProduct->post_id,
            'title'           => $importProduct->data['product_title'],
            'img'             => $importProduct->data['imageUrl'],
            'images'          => $images,
            'comments'        => $importProduct->data['comments'],
            'message'         => __('Importing...', 'sel'),
            'already'         => true,
            'already_link'    => get_permalink( $importProduct->post_id ),
            'currency'        => $currency
        ];
    }

    public function upload_images( $post_id, $images ){

        $uploaded = false;
        $foo = [];

        foreach ( $images as &$image ) {

            if ($uploaded || $image['post_id'] > 0 ) {
                $foo[] = $image;
                continue;
            }

            if ($image['post_id'] == 0) {

                $media = new MediaWoo();

                $img_id = sellvia_upload_images(
                    $media,
                    $images,
                    $post_id,
                    sellvia_prepare_image_url($image['url'])
                );

                if($img_id == false){
                    continue;
                }
                $image['post_id'] = $img_id;

                $foo[] = $image;

                global $wpdb;

                switch( $image[ 'place' ] ) {

                    case 'thumb':

                        \set_post_thumbnail( $post_id, $image['post_id'] );

                        break;

                    case 'gallery':

                        $gallery = \get_post_meta( $image[ 'id' ], '_product_image_gallery', true );

                        $gallery = ! empty( $gallery ) ? explode(',', $gallery) : [];

                        $gallery[]     = $image['post_id'];

                        \update_post_meta( $post_id, '_product_image_gallery', implode(',', $gallery) );

                        break;

                    case 'sku':

                        $var_ids = implode( ',', $this->getVariablesID( $post_id ) );

                        $variation_post_id = $wpdb->get_var(
                            $wpdb->prepare(
                                "SELECT post_id FROM {$wpdb->prefix}postmeta 
								 WHERE post_id IN ({$var_ids}) 
								 	AND meta_key = 'adswSKU' 
								 	AND meta_value = %s",
                                $image['id']
                            )
                        );

                        if( !empty($variation_post_id) ){

                            \update_post_meta( $variation_post_id, '_thumbnail_id', $image['post_id'] );
                        }

                        break;

                    case 'content':
                        $media->replaceImgContent( $post_id, $image );
                        break;

                    case 'short':
                        $media->replaceImgShort( $post_id, $image );
                        break;

                    case 'ai_gallery':

                        $gallery = $wpdb->get_var(
                            $wpdb->prepare("SELECT gallery FROM {$wpdb->prefix}mnsb_product_options WHERE post_id = %s", $post_id)
                        );

                        if( $gallery ){

                            $gallery = maybe_unserialize( $gallery );

                            if( is_array($gallery) ){

                                foreach ($gallery as $k => $v) {
                                    if ($v == $image['url']) {

                                        $gallery[$k] = $image['post_id'];
                                    }
                                }

                                $wpdb->update(
                                    $wpdb->prefix . 'mnsb_product_options',
                                    ['gallery' => maybe_serialize($gallery)],
                                    ['post_id' => $post_id]
                                );
                            }
                        }
                }

                $uploaded = true;
            }
        }

        return [
            'success' => true,
            'data' => [
                'post_id' => $post_id,
                'images' =>$foo
            ]
        ];
    }

    private function getVariablesID( $post_id ) {

        global $wpdb;

        $result = $wpdb->get_results(
            $wpdb->prepare( "SELECT ID FROM {$wpdb->posts} WHERE post_parent = %d AND post_type = 'product_variation'", $post_id )
        );

        $foo = [];
        if( $result ) foreach ( $result as $res )
            $foo[] = $res->ID;

        return $foo;
    }

    public function upload_comments( $post_id, $comments ){

        $imported = false;
        $foo = [];

        $media = new MediaWoo();

        $media_options = $media->getOptions();

        if( !empty( $comments ) ){

            foreach ( $comments as $key => &$comment ){
                $comment = (array) $comment;

                if ( $imported || $comment['done'] > 0) {
                    $foo[] = $comment;
                    continue;
                }

                $commentdata = [
                    'comment_post_ID'      => $post_id,
                    'comment_author'       => $comment['comment_author'],
                    'comment_author_email' => $comment['comment_author_email'],
                    'comment_content'      => base64_decode($comment['comment_content']),
                    'comment_type'         => 'review',
                    'user_ID'              => 0,
                    'comment_date'         => $comment['comment_date'],
                    'comment_approved'     => 1,
                    'comment_author_url'   => ''
                ];

                if( $this->existsReview( $commentdata ) ){
                    $comment['done'] = 1;
                    continue;
                }

                add_filter('comment_flood_filter', '__return_false');

                $comment_id = wp_new_comment( $commentdata, true );

                if( $comment_id && !is_wp_error( $comment_id ) && intval( $comment_id ) > 0 ) {
                    $comment['done'] = $comment_id;
                    $foo[] = $comment;
                    add_comment_meta( $comment_id, 'flag', $comment['flag'], true );
                    add_comment_meta( $comment_id, 'rating', intval($comment['star']), true );

                    if( isset( $comment['images'] ) && !empty( $comment['images'] ) ) {

                        if( $media_options['reviews_images'] === 0 ){

                            $coo = [];
                            foreach ( $comment['images'] as $image_url ){
                                $img_id = $media->create_external_attachment( $post_id, $image_url );

                                if( $img_id == false )
                                    continue 1;

                                $coo[] = $img_id;
                            }

                            add_comment_meta( $comment_id, 'images', $coo, true);
                        }else{
                            add_comment_meta( $comment_id, 'images', $comment['images'], true);

                            $media->downloadReviewImage( $comment_id, $post_id );
                            $imported = true;
                        }
                    }

                    wp_update_comment( [ 'comment_ID' => $comment_id, 'comment_approved' => 1, 'user_ID' => 0 ] );
                } else {
                    unset($comments[$key]);
                    $imported = true;
                }
            }
        }
        return [
            'success' => true,
            'data' => [
                'post_id' => $post_id,
                'comments' =>$foo
            ]
        ];
    }

    public function reset_product( $post_id, $settings = [] ){

        $this->reset_settings = sellvia_parse_args( self::$default_setting_reset, array_map( function ($i){ return intval($i); } , $settings ) );

        $import = $this->reset( $post_id );

        if( empty( $settings['reviews'] ) ){
            unset( $import['comments'] );
        }

        if( isset( $import['error'] ) ){
            return[
                'success' => false,
                'message' =>  $import['error'],
            ];
        }

        return[
            'success' => true,
            'data' => [
                'product' => $import
            ]
        ];

    }

    private function reset( $post_id ){

        $currency = \get_woocommerce_currency();

        $sellvia_post_id = get_post_meta( $post_id, 'sellvia_post_id', true );

        if( empty( $sellvia_post_id ) ) {
            return[
                'error' =>  __('This is not Sellvia product', 'sel'),
            ];
        }

        $images_settings = sellvia_get_images_settings();

        $request = self::call( '/sellvia_customer_plugin/importProduct', [
            'sellvia_post_id' => $sellvia_post_id,
            'currency'        => $currency,
            'action'          => 'reset',
            'webp'            => intval( $images_settings['webp'] )
        ]);

        if( !isset ( $request['data'] ) || !isset ( $request['data']['data'] ) || !isset ($request['data']['post_id'] ) || empty($request['data']['post_id']) ){
            return[
                'error' =>  __('Product not found', 'sel'),
            ];
        }

        $importProduct = new ImportProductWoo();

        $data = $request['data']['data'];
        $data['sellvia_post_id'] = $request['data']['post_id'];

        $result = $importProduct->resetProduct( $post_id, $data, $this->reset_settings );

        if( isset( $result['error'] ) ){
            return[
                'error' =>  __('Something went wrong', 'sel'),
            ];
        }

        $images = $importProduct->process_images();

        return [
            'post_id'         => $importProduct->data['post_id'],
            'message'         => $importProduct->message,
            'images'          => $images,
            'comments'        => !empty( $importProduct->data['comments'] ) ? $importProduct->data['comments'] : [],
        ];
    }

    public function premium_all(){

        $result = $this->get_premium_all();

        if( !is_array($result) || $result['status'] !== 'success' ){
            return[
                'success' => false,
                'data' => $result,
            ];
        }

        $products = isset( $result['data']['products'] ) ? $result['data']['products'] : [];

        if(!empty($products)){

            global $wpdb;

            foreach ( $products as &$product ) {

                $post_id = $wpdb->get_var(
                    "SELECT m.post_id FROM {$wpdb->prefix}postmeta m
                            WHERE m.meta_key = 'sellvia_post_id' AND m.meta_value = '{$product['sellvia_post_id']}'");

                $product['already'] = $post_id !== NULL;
                $product['post_id'] = $post_id ?: false;
            }
        }

        return [
            'success' => true,
            'data' => [
                'product_list' => $products,
            ]
        ];
    }
}
