<?php


namespace sellvia\ProductUpdate;

use \ads\adsPrice;
use sellvia\ImportProduct\ImportProductAds;
use sellvia\Shipping\ShippingAds;

class ProductUpdateAds extends ProductUpdate {

    private $data = [];

    private $row_to_update = NULL;

    private $post_id = NULL;

    private $product = [];

    private $changePrice = false;

    private $changeQuantity = false;

    private $disappeared    = false;

    public $message = '';

    public $messageStatus = '';

    private $action_variant = false;

    public function __construct(){

        parent::__construct();
    }

    protected function get_next_product( $current ) {

        global $wpdb;

        $row = $wpdb->get_row(
            $wpdb->prepare(
                "SELECT
					posts.ID as post_id, pr.quantity, posts.post_title, m.meta_value as sellvia_post_id
                FROM {$wpdb->posts} posts
                    INNER JOIN {$wpdb->postmeta} m ON m.post_id = posts.ID
                    INNER JOIN {$wpdb->postmeta} m2 ON m2.post_id = posts.ID
                    LEFT JOIN {$wpdb->ads_products} pr ON pr.post_id = posts.ID
                WHERE m.meta_key = 'sellvia_post_id'
                    AND m2.meta_key = 'sellvia_need_update' 
                    AND m2.meta_value = '1'
                    AND posts.post_type = 'product' 
                    AND posts.post_status IN('publish','draft','importlist') 
                ORDER BY post_id ASC LIMIT %d, 1", $current
            ),
            ARRAY_A
        );

        if( ! empty( $row ) ) {
            $row[ 'thumbnail' ]       = get_the_post_thumbnail_url( $row['post_id'], 'thumbnail' );
            $row[ 'view_on_sellvia' ] = esc_url( SELLVIA_CATALOG_PATH . 'product/' . $row['sellvia_post_id'] );
            $row[ 'permalink' ]       = get_the_permalink($row['post_id']);
        }

        return empty( $row ) ? false : $row;
    }

    protected function update( $post_id, $sellvia_post_id ){

        $availability = true;

        $trouble = '';

        $this->post_id = $post_id;

        $response = self::call( '/sellvia_customer_plugin/product_update', [
            'sellvia_post_id' => $sellvia_post_id,
            'currency' => ADS_CUR_DEF,
        ]);

        if( empty($response) || !isset( $response['status'] )  ){

            return[
                'result' => false,
                'error' => __( 'Service unavailable', 'sel' )
            ];
        }

        if( $response['status'] != 'success' ){
            $availability = false;
            $this->message = $response['message'];
        }

        if( empty( $response['data'] ) ){
            $availability = false;
            $this->message = __( 'Product data not found', 'sel' );
        }

        if( !$availability ){

            $trouble = 'no_longer_available';

            $this->row_to_update = $this->getRowToUpdate();

            $this->updateStatus( $availability, $trouble );

            return[
                'result' => false,
                'message' => $this->message
            ];
        }

//        if( (int)$this->row->needUpdate !== 1 ){
//            $this->message = __( 'Product is disabled from update!', 'sel' );
//            return false;
//        }

        $this->data = $response['data']['data'];

        $this->prepare();

        $this->row_to_update = $this->getRowToUpdate();

        if( $this->checkOption( 'update', 'stock' ) ) {

            $this->mergeVariations( 'stock' );

            if( intval( $this->row_to_update['quantity'] ) != intval( $this->product['quantity'] ) ){

                $this->setChangeQuantity( $this->row_to_update['quantity'], $this->product['quantity'] );

                do_action( 'sellvia_log',
                    [
                        'type' => 'stock',
                        'details' => sprintf(__('Stock was changed from %s to %s', 'sel'), $this->row_to_update['quantity'] , $this->product['quantity'])
                    ],
                    sprintf( __( 'Stock for Product was changed on Sellvia. Stock was updated.', 'sel' ), $this->post_id ),
                    'product',
                    'success',
                    $this->post_id,
                    null,
                    ''
                );

                $this->message = __( 'Stock changed', 'sel' );
            }

            $this->doUpdate( $this->post_id, 'stock' );
        }

        if( apply_filters( 'sellvia_update_price_product', $this->checkOption( 'update', 'cost' ), $this->post_id ) ){

            $this->mergeVariations( 'prices' );

            if( floatval( $this->row_to_update['salePrice'] ) != floatval( $this->product['salePrice'] ) ){

                do_action( 'sellvia_log',
                    [
                        'type' => 'cost',
                        'details' => sprintf( __('Price was changed from %s to %s', 'sel'), $this->row_to_update['salePrice'] , $this->product['salePrice'] )
                    ],
                    sprintf( __( 'Price for Product was changed on Sellvia. Price was updated.', 'sel' ), $this->post_id ),
                    'product',
                    'success',
                    $this->post_id,
                    null,
                    ''
                );

                $this->setChangePrice( floatval( $this->row_to_update['salePrice'] ), floatval( $this->product['salePrice'] ) );

                if( $this->message == '' ){
                    $this->message = __( 'Price changed', 'sel' );
                }else{
                    $this->message = __( 'Stock and price changed', 'sel' );
                }
            }

            $this->doUpdate( $this->post_id, 'prices' );

            do_action( 'ads_update_price', $this->post_id );

        }

        $wh_shipping = isset( $this->data['wh_shipping'] ) && !empty( $this->data['wh_shipping'] ) ? $this->data['wh_shipping'] : [];
        ShippingAds::save_product_shipping( $post_id, maybe_unserialize( $this->data['skuAttr'] ), $wh_shipping );

        if( isset( $this->data['size_chart'] ) && !empty( $this->data['size_chart'] ) ){

            $sizes = ImportProductAds::prepareSizes( $this->data['size_chart'] );

            if( !empty( $sizes ) ) {
                global $wpdb;
                $wpdb->update(
                    $wpdb->ads_products_meta,
                    array( 'sizeAttr' => maybe_serialize($sizes) ),
                    array( 'post_id' => $post_id )
                );
                update_post_meta( $this->post_id, 'sellvia_sizeAttr', $this->data['size_chart'] );
            }
        }else{
            global $wpdb;
            $wpdb->update(
                $wpdb->ads_products_meta,
                array( 'sizeAttr' => maybe_serialize([]) ),
                array( 'post_id' => $post_id )
            );
            delete_post_meta( $this->post_id, 'sellvia_sizeAttr' );
        }

        if( isset( $this->data['marketing_url'] ) && !empty( $this->data['marketing_url'] ) && sellvia_is_url( $this->data['marketing_url'] ) ){
            update_post_meta( $this->post_id, 'sellvia_marketing_url', $this->data['marketing_url'] );
        }else{
            delete_post_meta( $this->post_id, 'sellvia_marketing_url' );
        }

        $this->row_to_update = $this->getRowToUpdate();

        if( $this->getQunatity() === 0 ){

            $trouble = 'stock_is_zero';

            $availability = false;
        }

        $this->updateStatus( $availability, $trouble );


        $this->message = $this->message == '' ? __( 'Up to date', 'sel' ) : $this->message;

        return[
            'result' => true,
            'message' => $this->message
        ];
    }

    private function prepare(){

        $this->data['prices'] = self::preparePrices( $this->data['prices'] );
        $this->data['skuAttr'] = self::prepareSkuAttr( $this->data['skuAttr'] );

        $this->product[ 'price' ]           = $this->data['prices'][ 'price' ];
        $this->product[ 'priceMax' ]        = $this->data['prices'][ 'priceMax' ];
        $this->product[ 'salePrice' ]       = $this->data['prices'][ 'salePrice' ];
        $this->product[ 'salePriceMax' ]    = $this->data['prices'][ 'salePriceMax' ];
        $this->product[ 'currency' ]        = 'USD';
        $this->product[ 'quantity' ]        = intval( $this->data[ 'quantity' ] );
        $this->product[ 'skuAttr' ]         = maybe_unserialize( $this->data['skuAttr'] );
    }

    private function getRowToUpdate() {

        global $wpdb;

        $row = $wpdb->get_row(
            $wpdb->prepare(
            "SELECT p.price,  p.priceMax,  p.salePrice,  p.salePriceMax, p.quantity, p.currency, p.discount, p.packageType, po.post_status, m.skuAttr 
                FROM {$wpdb->ads_products_meta} m
                    INNER JOIN {$wpdb->ads_products} p ON p.post_id = m.post_id
                    INNER JOIN {$wpdb->posts} po ON po.ID = p.post_id
                WHERE m.post_id = %d",
                $this->post_id),
        ARRAY_A );

        if( !empty($row) ){
            $row['skuAttr'] = maybe_unserialize( $row['skuAttr'] );
        }

        return $row;
    }

    private function mergeVariations( $action = 'prices' ){

        $current = $this->row_to_update['skuAttr'];
        $import  = $this->product[ 'skuAttr' ];

        $too = array();

        if( $current ) {

            foreach ( $current as $key => $val ) {

                if( isset( $import[ $key ] ) ){

                    $value = $import[ $key ];

                    $val['isActivity'] = isset( $val['isActivity'] ) ? $val['isActivity'] : 1;

                    $isActivity = $this->checkOption( 'disable', 'variant' ) ? intval( $value['quantity'] ) > 0 : $val['isActivity'];

                    if( $action == 'prices' ) {
                        $moo = array(
                            'quantity' => $val['quantity'],
                            'price' => $value['price'],
                            'isActivity' => $isActivity ? 1 : 0,
                            'salePrice' => $value['salePrice']
                        );
                    } else {
                        $moo = array(
                            'quantity' => $value['quantity'],
                            'price' => $val['price'],
                            'isActivity' => $isActivity ? 1 : 0,
                            'salePrice' => $val['salePrice']
                        );

                    }

                    $too[ $key ] = $moo;
                } else {

                    if( !$this->action_variant ) {

                        if ($this->checkOption('disable', 'variant')) {
                            $val['isActivity'] = 0;

                            do_action('sellvia_log',
                                [
                                    'type' => 'variant',
                                    'details' => $val
                                ],
                                sprintf(__('Variation for Product disappeared on Sellvia. Variation was disabled.', 'sel'), $this->post_id),
                                'product',
                                'info',
                                $this->post_id,
                                null,
                                'variation_no_longer_available'
                            );

                        }

                        if ($this->checkOption('zero', 'variant')) {
                            $val['quantity'] = 0;

                            do_action('sellvia_log',
                                [
                                    'type' => 'variant',
                                    'details' => $val
                                ],
                                sprintf(__('Variation for Product disappeared on Sellvia. Variation stock was set to zero.', 'sel'), $this->post_id),
                                'product',
                                'info',
                                $this->post_id,
                                null,
                                'variation_no_longer_available'
                            );

                        }

                        if ($this->checkOption('doNothing', 'variant')) {

                            do_action('sellvia_log',
                                [
                                    'type' => 'variant',
                                    'details' => $val
                                ],
                                sprintf(__('Variation for Product disappeared on Sellvia.', 'sel'), $this->post_id),
                                'product',
                                'info',
                                $this->post_id,
                                null,
                                'variation_no_longer_available'
                            );

                        }
                        $this->action_variant = true;
                    }
                    $too[ $key ] = $val;
                }
            }
        }

        $this->row_to_update['skuAttr'] = $too;
    }

    private function doUpdate( $post_id, $action = 'prices' ) {

        global $wpdb;

        if( 'prices' == $action ) {
            $wpdb->update(
                $wpdb->ads_products,
                array(
                    'price'        => $this->product[ 'price' ],
                    'priceMax'     => $this->product[ 'priceMax' ],
                    'salePrice'    => $this->product[ 'salePrice' ],
                    'salePriceMax' => $this->product[ 'salePriceMax' ],
                ),
                array( 'post_id' => $post_id )
            );

            $wpdb->update(
                $wpdb->ads_products_meta,
                array( 'skuAttr' => maybe_serialize( $this->row_to_update[ 'skuAttr' ] ) ),
                array( 'post_id' => $post_id )
            );
        } else {

            $wpdb->update(
                $wpdb->ads_products,
                array( 'quantity' => $this->product[ 'quantity' ] ),
                array( 'post_id'  => $post_id )
            );

            $wpdb->update(
                $wpdb->ads_products_meta,
                array( 'skuAttr' => maybe_serialize( $this->row_to_update[ 'skuAttr' ] ) ),
                array( 'post_id' => $post_id )
            );
        }
    }

    private function updateStatus( $availability, $trouble = '' ){

        $post_status = $this->row_to_update['post_status'];

        if( ! $availability ) {

            $this->messageStatus = 'no_longer_available';

            if( $this->checkOption( 'doNothing', 'status' ) ) {

                $this->message = __( 'No longer available', 'sel' );

                $this->setDisappeared( true );

                do_action( 'sellvia_log',
                    [
                        'type' => 'status',
                        'details' => ''
                    ],
                    sprintf( __( 'Product is no longer available on Sellvia.', 'sel' ), $this->post_id ),
                    'product',
                    'danger',
                    $this->post_id,
                    null,
                    $trouble
                );

            } elseif( $this->checkOption( 'draft', 'status' ) ) {

                $this->message = __( 'No longer available', 'sel' );

                $this->setDisappeared( true );

                if( $post_status !== 'draft' ){

                    wp_update_post( array( 'ID' => $this->post_id, 'post_status' => 'draft' ), true );

//                    $this->message = __( 'Product is no longer available. Moved to Draft.', 'sel' );
                    $this->message = __( 'No longer available', 'sel' );
                    $this->messageStatus = 'to_draft';

                    do_action( 'sellvia_log',
                        [
                            'type' => 'status',
                            'details' => ''
                        ],
                        sprintf( __('Product is no longer available on Sellvia. It was sent to Draft.', 'sel'), $this->post_id ),
                        'product',
                        'info',
                        $this->post_id,
                        null,
                        $trouble
                    );

                }


            } elseif( $this->checkOption( 'zero', 'status' ) ) {

                $this->setQuantityZero();

//                $this->message = __( 'Product is no longer available. Quantity set Zero.', 'sel' );
                $this->message = __( 'No longer available', 'sel' );

                $this->setDisappeared( true );

                do_action( 'sellvia_log',
                    [
                        'type' => 'status',
                        'details' => ''
                    ],
                    sprintf( __('Product is no longer available on Sellvia. Stock was set to zero.', 'sel'), $this->post_id ),
                    'product',
                    'info',
                    $this->post_id,
                    null,
                    $trouble
                );

            }

            return true;

        } elseif( $this->checkOption( 'draft', 'status' ) ){

            if( $post_status == 'draft'){

//                $this->message = __( 'Product is available again. Moved to Published.', 'sel' );
                $this->message = __( 'Moved to Published', 'sel' );

                do_action( 'sellvia_log',
                    [
                        'type' => 'status',
                        'details' => ''
                    ],
                    sprintf( __('Product is available again. Moved to Published.', 'sel'), $this->post_id ),
                    'product',
                    'info',
                    $this->post_id,
                    null,
                    ''
                );

                wp_update_post( array( 'ID' => $this->post_id, 'post_status' => 'publish' ) );
            }
        }
    }

    public function checkOption( $str, $option ) {

        $option = $this->settings[ $option ];

        return $str == $option;
    }

    /**
     * @return mixed
     */
    public function getChangePrice() {
        return $this->changePrice;
    }

    public function setChangePrice( $old, $new ) {

        if( intval( $old ) === intval ( $new ) ) {
            $this->changePrice = false;
        } else {
            $this->changePrice = intval( $old ) < intval ( $new ) ? 'up' : 'down';
        }
    }

    /**
     * @return mixed
     */
    public function getChangeQuantity() {
        return $this->changeQuantity;
    }

    public function setChangeQuantity( $old, $new ) {

        if( intval( $old ) === intval ( $new ) ) {
            $this->changeQuantity = false;
        } else {
            $this->changeQuantity = intval( $old ) < intval ( $new ) ? 'up' : 'down';
        }
    }

    public function getDisappeared() {
        return $this->disappeared;
    }

    public function setDisappeared( $value = false ) {
        $this->disappeared = $value;
    }

    public function getQunatity() {

        $current  = $this->row_to_update;

        if( !empty( $current['skuAttr'] ) ) {

            $quantity = 0;

            foreach ( $current['skuAttr'] as $key => $val ) {
                if($val['isActivity']){
                    $quantity += intval( $val[ 'quantity' ] );
                }
            }

            return $quantity;

        } elseif ( isset( $current['quantity'] ) ){
            return intval( $current['quantity'] );
        }

        return true;
    }

    private function setQuantityZero() {

        global $wpdb;

        $orig     = !empty( $this->product['skuAttr'] ) ? $this->product['skuAttr'] : [];
        $current  = $this->row_to_update['skuAttr'];

        $quantity = 0;

        if( $current ) {

            foreach ( $current as $key => &$val ) {

                if ( isset( $orig[ $key ] ) ) {

                    $val[ 'isActivity' ] = 0;
                    $val[ 'quantity' ]   = 0;
                } else {
                    $quantity += $val[ 'quantity' ];
                }
            }

            $wpdb->update(
                $wpdb->ads_products_meta,
                array( 'skuAttr' => serialize( $current ) ),
                array( 'post_id' => $this->post_id )
            );
        }

        $wpdb->update(
            $wpdb->ads_products,
            array( 'quantity' => $quantity ),
            array( 'post_id'  => $this->post_id )
        );
    }

    protected function preparePricing() {

        $data = $this->getRowToUpdate();

        $layout = '';

        if ( $data[ 'price' ] != $data[ 'salePrice' ] && $data[ 'priceMax' ] == 0 ) {
            $layout .= sprintf(
                '%s: <span class="salePrice"><strong>%s</strong><i></i></span>',
                __( 'Price', 'sel' ),
                adsPrice::format( $data[ 'salePrice' ], $data[ 'currency' ] )
            );

            if( $data[ 'price' ] != 0 ) {
                $layout .= sprintf('<span class="price"><strike>%s</strike></span>',
                    adsPrice::format($data['price'], $data['currency'])
                );
            }
        } elseif ( $data[ 'priceMax' ] != 0 && $data[ 'priceMax' ] != $data[ 'price' ] && $data[ 'price' ] == $data[ 'salePrice' ] ) {
            $layout .= sprintf( '%s: <span class="salePrice"><strong>%s - %s</strong><i></i></span>',
                __( 'Price', 'sel' ),
                adsPrice::format( $data[ 'price' ], $data[ 'currency' ] ),
                adsPrice::format( $data[ 'priceMax' ], $data[ 'currency' ] )
            );
        } elseif ( $data[ 'priceMax' ] != 0 && $data[ 'priceMax' ] != $data[ 'price' ] ) {
            $layout .= sprintf( '%s: <span class="salePrice"><strong>%s - %s</strong><i></i></span>',
                __( 'Price', 'sel' ),
                adsPrice::format( $data[ 'salePrice' ], $data[ 'currency' ] ),
                adsPrice::format( $data[ 'salePriceMax' ], $data[ 'currency' ] )
            );
//            $layout .= sprintf( '<div class="price"><strike>%s - %s</strike> <span class="discount">%s&#37; %s</span></div>',
//                adsPrice::format( $data[ 'price' ], $data[ 'currency' ] ),
//                adsPrice::format( $data[ 'priceMax' ], $data[ 'currency' ] ),
//                $data[ 'discount' ], __( 'off', 'sel' ) );
        } else {
            $layout .= sprintf( '%s: <span class="salePrice"><strong>%s</strong><i></i></span>',
                __( 'Price', 'sel' ),
                adsPrice::format( $data[ 'price' ], $data[ 'currency' ] )
            );
        }
        return $layout;
    }

    protected function prepareQuantity(){

        $data = $this->getRowToUpdate();

        $color = ( $data[ 'quantity' ] <= 20 ) ? '#e74c3c' : '#2cc36b';

        return sprintf( '<div class="quantity">%s: <span style="color:%s">%d</span><i></i></div>', __( 'Stock', 'sel' ), $color, $data[ 'quantity' ] );
    }
}