<?php


namespace sellvia\ProductUpdate;


use sellvia\ImportProduct\ImportProductAds;
use sellvia\Shipping\ShippingWoo;

class ProductUpdateWoo extends ProductUpdate {

    private $data = [];

    private $row_to_update = NULL;

    private $post_id = NULL;

    private $product = [];

    private $meta = [];

    private $changePrice = false;

    private $changeQuantity = false;

    private $disappeared    = false;

    public $message = '';

    public $messageStatus = '';

    private $action_variant = false;

    public function __construct(){

        parent::__construct();
    }

    protected function get_next_product( $current ) {

        global $wpdb;

        $row = $wpdb->get_row(
            $wpdb->prepare(
                "SELECT
					posts.ID as post_id, posts.post_title, m.meta_value as sellvia_post_id
                FROM {$wpdb->posts} posts
                    INNER JOIN {$wpdb->postmeta} m ON m.post_id = posts.ID
                    INNER JOIN {$wpdb->postmeta} m2 ON m2.post_id = posts.ID
                WHERE m.meta_key = 'sellvia_post_id'
                    AND m2.meta_key = 'sellvia_need_update' 
                    AND m2.meta_value = '1'
                    AND posts.post_type = 'product' 
                    AND posts.post_status IN('publish','draft','importlist') 
                ORDER BY post_id ASC LIMIT %d, 1", $current
            ),
            ARRAY_A
        );

        if( ! empty( $row ) ) {
            $row[ 'quantity' ]     = get_post_meta( $row[ 'post_id' ], '_stock', true );
            $row[ 'thumbnail' ]       = get_the_post_thumbnail_url( $row['post_id'], 'thumbnail' );
            $row[ 'view_on_sellvia' ] = esc_url( SELLVIA_CATALOG_PATH . 'product/' . $row['sellvia_post_id'] );
            $row[ 'permalink' ]       = get_the_permalink($row['post_id']);
        }

        return empty( $row ) ? false : $row;
    }

    protected function update( $post_id, $sellvia_post_id ){

        $availability = true;

        $trouble = '';

        $this->post_id = $post_id;

        $response = self::call( '/sellvia_customer_plugin/product_update', [
            'sellvia_post_id' => $sellvia_post_id,
            'currency' => \get_woocommerce_currency(),
        ]);

        if( empty($response) || !isset( $response['status'] )  ){

            return[
                'result' => false,
                'error' => __( 'Service unavailable', 'sel' )
            ];
        }

        if( $response['status'] != 'success' ){
            $availability = false;
            $this->message = $response['message'];
        }

        if( empty( $response['data'] ) ){
            $availability = false;
            $this->message = __( 'Product data not found', 'sel' );
        }

        if( !$availability ){

            $trouble = 'no_longer_available';
            $this->updateStatus( $availability, $trouble );

            return[
                'result' => false,
                'message' => $this->message
            ];
        }

        $this->row_to_update = $this->getRowToUpdate();

//        if( (int)$this->row->needUpdate !== 1 ){
//            $this->message = __( 'Product is disabled from update!', 'sel' );
//            return false;
//        }

        $this->data = $response['data']['data'];

        $this->prepare();

        $this->preparePostMeta();

        if( $this->checkOption( 'update', 'stock' ) ) {

            $stock = get_post_meta( $this->post_id, '_stock', true );

            if( $stock != $this->product['quantity'] ){

                $this->setChangeQuantity( $stock, $this->product['quantity'] );

                do_action( 'sellvia_log',
                    [
                        'type' => 'stock',
                        'details' => sprintf(__('Stock was changed from %s to %s', 'sel'), $this->row_to_update['quantity'] , $this->product['quantity'])
                    ],
                    sprintf( __( 'Stock for Product was changed on Sellvia. Stock was updated.', 'sel' ), $this->post_id ),
                    'product',
                    'success',
                    $this->post_id,
                    null,
                    ''
                );

                $this->message = __( 'Stock changed', 'sel' );
            }

            $this->doUpdate( $this->post_id, 'stock' );
        }

        if( apply_filters( 'sellvia_update_price_product', $this->checkOption( 'update', 'cost' ), $this->post_id ) ){

            if( floatval( $this->row_to_update['salePrice'] ) != floatval( $this->product['salePrice'] ) ){

                do_action( 'sellvia_log',
                    [
                        'type' => 'cost',
                        'details' => sprintf( __('Price was changed from %s to %s', 'sel'), $this->row_to_update['salePrice'] , $this->product['salePrice'] )
                    ],
                    sprintf( __( 'Price for Product was changed on Sellvia. Price was updated.', 'sel' ), $this->post_id ),
                    'product',
                    'success',
                    $this->post_id,
                    null,
                    ''
                );

                $this->setChangePrice( floatval( $this->row_to_update['salePrice'] ), floatval( $this->product['salePrice'] ) );

                if( $this->message == '' ){
                    $this->message = __( 'Price changed', 'sel' );
                }else{
                    $this->message = __( 'Stock and price changed', 'sel' );
                }
            }

            $this->doUpdate( $this->post_id, 'prices' );
        }

        if( $this->product[ 'quantity' ] === 0 ){
            if($trouble === 'no_longer_available'){
                $trouble = 'stock_is_zero';
            }
            $availability = false;
        }

        $this->updateStatus( $availability, $trouble);

        $this->message = $this->message == '' ? __( 'Up to date', 'sel' ) : $this->message;

        $this->setProductMetaLookup( $this->meta, $this->post_id );
        \WC_Product_Variable::sync( $this->post_id );
        wc_delete_product_transients( $this->post_id );

        //Shipping
        $wh_shipping = isset( $this->data['wh_shipping'] ) && !empty( $this->data['wh_shipping'] ) ? $this->data['wh_shipping'] : [];
        ShippingWoo::save_product_shipping( $this->post_id, maybe_unserialize( $this->data['skuAttr'] ), $wh_shipping );

        return[
            'result' => true,
            'message' => $this->message
        ];

    }

    private function prepare(){

        $this->data['prices'] = self::preparePrices( $this->data['prices'] );
        $this->data['skuAttr'] = self::prepareSkuAttr( $this->data['skuAttr'] );

        $this->product[ 'price' ]           = $this->data['prices'][ 'price' ];
        $this->product[ 'priceMax' ]        = $this->data['prices'][ 'priceMax' ];
        $this->product[ 'salePrice' ]       = $this->data['prices'][ 'salePrice' ];
        $this->product[ 'salePriceMax' ]    = $this->data['prices'][ 'salePriceMax' ];
        $this->product[ 'currency' ]        = 'USD';

        $this->product[ 'quantity' ]        = intval( $this->data[ 'quantity' ] );

        $skuAttr = maybe_unserialize( $this->data[ 'skuAttr' ] );

        if( !empty( $skuAttr ) && is_array( $skuAttr ) && count( $skuAttr ) > 0 ){

            foreach ( $skuAttr as $key => $val ){

                $this->product[ 'skuAttr' ][ $key ] = $this->prepareVariationMeta( $val, $val[ 'quantity' ] , $key );
            }
        } else {
            $this->product[ 'skuAttr' ] = [];
        }
    }

    private function preparePostMeta() {

        $regular_price = $this->product['price'] == 0 ? $this->product['salePrice'] : $this->product['price'];
        $sale_price = ! empty( $this->product[ 'salePrice' ] ) && $this->product[ 'price' ] > $this->product[ 'salePrice' ] ? $this->product[ 'salePrice' ] : '';

        $this->meta = [
            '_regular_price' => $regular_price,
            '_sale_price'    => $sale_price,
            '_price'         => $sale_price != '' ? $sale_price : $regular_price,
            '_stock'         => intval( sellvia_floatvalue( $this->product['quantity'] ) ), //общее количество товара с учётом всех вариаций
            '_stock_status'  => intval( sellvia_floatvalue( $this->product['quantity'] ) ) > 0 ? 'instock' : 'outofstock',
        ];

        if( isset( $this->data['size_chart'] ) && !empty( $this->data['size_chart'] ) ){

            $sizes = ImportProductAds::prepareSizes( $this->data['size_chart'] );

            if( $sizes ) {
                update_post_meta( $this->post_id, 'adswSizeAttr', $sizes );
                update_post_meta( $this->post_id, 'sellvia_sizeAttr', $this->data['size_chart'] );
            }
        }else{
            delete_post_meta( $this->post_id, 'adswSizeAttr' );
            delete_post_meta( $this->post_id, 'sellvia_sizeAttr' );
        }

        if( isset( $this->data[ 'marketing_url'] ) && !empty( $this->data['marketing_url'] ) && sellvia_is_url( $this->data['marketing_url'] ) ){
            update_post_meta( $this->post_id, 'sellvia_marketing_url', $this->data['marketing_url'] );
        }else{
            delete_post_meta( $this->post_id, 'sellvia_marketing_url' );
        }
    }

    private function prepareVariationMeta( $prices, $quantity, $key_skuAttr ){

        $regular_price = $prices['price'] == 0 ? $prices['salePrice'] : $prices['price'];
        $sale_price = ! empty( $prices[ 'salePrice' ] ) && $prices[ 'price' ] > $prices[ 'salePrice' ] ? $prices[ 'salePrice' ] : '';

        $meta = [
            '_regular_price' => $regular_price,
            '_sale_price'    => $sale_price,
            '_price'         => $sale_price != '' ? $sale_price : $regular_price,
            '_stock'         => $quantity,
            '_stock_status'  => $quantity > 0 ? 'instock' : 'outofstock',
        ];

        return $meta;
    }

    private function getRowToUpdate() {

        global $wpdb;

        $row = $wpdb->get_row(
            $wpdb->prepare(
                "SELECT p.post_status
                FROM {$wpdb->posts} p 
                WHERE p.ID= %d",
                $this->post_id),
            ARRAY_A );

        if( !empty($row) ){

            $product = wc_get_product( $this->post_id );
            $row['post_status']  = $product->get_status();
            $row['packageType']  = 'piece';
            $row['price'] = $product->get_regular_price();
            $row['salePrice'] = $product->get_price();
            $row['quantity'] = $product->get_stock_quantity();
            $row['price_html'] = $product->get_price_html();
        }
        return $row;
    }

    private function doUpdate( $post_id, $action = 'prices' ) {

        if( 'prices' == $action ) {
            $args = [
                '_regular_price' => $this->meta[ '_regular_price' ],
                '_price'         => $this->meta[ '_price' ],
                '_sale_price'    => $this->meta[ '_sale_price' ],
            ];

            $this->updatePostMeta( $args );

            //update variations price
            $this->updateVariations( $post_id, $action );

        } elseif ( 'stock' == $action ) {

            wc_update_product_stock( $post_id , $this->meta[ '_stock' ] );
            wc_update_product_stock_status( $post_id, $this->meta[ '_stock_status' ] );

            //update variations stock
            $this->updateVariations( $post_id, $action );

        }

        return true;
    }

    private function updateStatus( $availability, $trouble = '' ){

        $post_status = $this->row_to_update['post_status'];

        if( ! $availability ) {

            $this->messageStatus = 'no_longer_available';

            if( $this->checkOption( 'doNothing', 'status' ) ) {

//                $this->message = __( 'Product is no longer available.', 'sel' );
                $this->message = __( 'No longer available', 'sel' );

                $this->setDisappeared( true );

                do_action( 'sellvia_log',
                    [
                        'type' => 'status',
                        'details' => ''
                    ],
                    sprintf( __( 'Product is no longer available on Sellvia.', 'sel' ), $this->post_id ),
                    'product',
                    'danger',
                    $this->post_id,
                    null,
                    $trouble
                );

            } elseif( $this->checkOption( 'draft', 'status' ) ) {

//                $this->message = __( 'Product is no longer available.', 'sel' );
                $this->message = __( 'No longer available', 'sel' );

                $this->setDisappeared( true );

                if( $post_status !== 'draft' ){

                    wp_update_post( array( 'ID' => $this->post_id, 'post_status' => 'draft' ), true );

//                    $this->message = __( 'Product is no longer available. Moved to Draft.', 'sel' );
                    $this->message = __( 'No longer available', 'sel' );

                    $this->messageStatus = 'to_draft';

                    do_action( 'sellvia_log',
                        [
                            'type' => 'status',
                            'details' => ''
                        ],
                        sprintf( __('Product is no longer available on Sellvia. It was sent to Draft.', 'sel'), $this->post_id ),
                        'product',
                        'info',
                        $this->post_id,
                        null,
                        $trouble
                    );

                }


            } elseif( $this->checkOption( 'zero', 'status' ) ) {

                $this->setQuantityZero();

//                $this->message = __( 'Product is no longer available. Quantity set Zero.', 'sel' );
                $this->message = __( 'No longer available', 'sel' );

                $this->setDisappeared( true );

                do_action( 'sellvia_log',
                    [
                        'type' => 'status',
                        'details' => ''
                    ],
                    sprintf( __('Product is no longer available on Sellvia. Stock was set to zero.', 'sel'), $this->post_id ),
                    'product',
                    'info',
                    $this->post_id,
                    null,
                    $trouble
                );

            }

            return true;

        }elseif( $this->checkOption( 'draft', 'status' ) ){

            if( $post_status == 'draft'){

//                $this->message = __( 'Product is available again. Moved to Published.', 'sel' );
                $this->message = __( 'Moved to Published', 'sel' );

                do_action( 'sellvia_log',
                    [
                        'type' => 'status',
                        'details' => ''
                    ],
                    sprintf( __('Product is available again. Moved to Published.', 'sel'), $this->post_id ),
                    'product',
                    'info',
                    $this->post_id,
                    null,
                    ''
                );

                wp_update_post( array( 'ID' => $this->post_id, 'post_status' => 'publish' ) );
            }

        }
    }

    protected function updatePostMeta( $args, $post_id = false ) {

        if( ! $post_id )
            $post_id = $this->post_id;

        if ( ! $args || empty( $args ) ) {
            return false;
        }

        foreach( $args as $key => $val ) {
            update_post_meta( $post_id, $key, $val );
        }

        return true;
    }

    private function updateVariations( $post_id, $action = 'prices'  ){

        if( empty( $this->product[ 'skuAttr' ] ) ) {
            return true;
        }

        global $wpdb;

        $results = $wpdb->get_results(
            $wpdb->prepare(
                "SELECT ID, meta_value as SKU FROM {$wpdb->posts} p
				INNER JOIN {$wpdb->postmeta} m ON p.ID = m.post_id  
				WHERE post_parent = %d AND post_type = 'product_variation' AND meta_key = 'adswSKU'", $post_id
            )
        );

        if( ! $results )
            return true;

        foreach( $results as $result ) {

            if( isset( $this->product[ 'skuAttr' ][ $result->SKU ] ) ){

                if( 'prices' == $action ) {

                    update_post_meta( $result->ID, '_regular_price', $this->product[ 'skuAttr' ][ $result->SKU ][ '_regular_price' ] );
                    update_post_meta( $result->ID, '_sale_price', $this->product[ 'skuAttr' ][ $result->SKU ][ '_sale_price' ] );
                    update_post_meta( $result->ID, '_price', $this->product[ 'skuAttr' ][ $result->SKU ][ '_price' ] );

                } elseif ( 'stock' == $action ){

                    if ( $this->product[ 'skuAttr' ][ $result->SKU ][ '_stock' ] > 0 ) {
                        wp_update_post( [
                            'ID' => $result->ID,
                            'post_status' => 'publish'
                        ] );
                    }else{

                        if( $this->checkOption( 'disable', 'variant' ) ){

                            do_action('sellvia_log',
                                [
                                    'type' => 'variant',
                                    'details' => $result,
                                ],
                                sprintf(__('Variation for Product disappeared on Sellvia. Variation was disabled.', 'sel'), $this->post_id),
                                'product',
                                'info',
                                $this->post_id,
                                'variation_no_longer_available'
                            );

                            wp_update_post( [
                                'ID' => $result->ID,
                                'post_status' => 'private'
                            ] );
                        }
                    }

                    wc_update_product_stock( $result->ID , $this->product[ 'skuAttr' ][ $result->SKU ][ '_stock' ] );
                    wc_update_product_stock_status( $result->ID, $this->product[ 'skuAttr' ][ $result->SKU ][ '_stock_status' ] );
                }
            } else {

                if( !$this->action_variant ) {

                    if ($this->checkOption('disable', 'variant')) {

                        wp_update_post([
                            'ID' => $result->ID,
                            'post_status' => 'private'
                        ]);

                        \wc_update_product_stock($result->ID, 0);
                        \wc_update_product_stock_status($result->ID, 'outofstock');

                        do_action('sellvia_log',
                            [
                                'type' => 'variant',
                                'details' => $result,
                            ],
                            sprintf(__('Variation for Product disappeared on Sellvia. Variation was disabled.', 'sel'), $this->post_id),
                            'product',
                            'info',
                            $this->post_id,
                            'variation_no_longer_available'
                        );
                    }

                    if ($this->checkOption('zero', 'variant')) {

                        \wc_update_product_stock($result->ID, 0);
                        \wc_update_product_stock_status($result->ID, 'outofstock');

                        do_action('sellvia_log',
                            [
                                'type' => 'variant',
                                'details' => $result,
                            ],
                            sprintf(__('Variation for Product disappeared on Sellvia. Variation stock was set to zero', 'sel'), $this->post_id),
                            'product',
                            'info',
                            $this->post_id,
                            'variation_no_longer_available'
                        );
                    }

                    if ($this->checkOption('doNothing', 'variant')) {

                        do_action('sellvia_log',
                            [
                                'type' => 'variant',
                                'details' => $result,
                            ],
                            sprintf(__('Variation for Product disappeared on Sellvia.', 'sel'), $this->post_id),
                            'product',
                            'info',
                            $this->post_id,
                            'variation_no_longer_available'
                        );
                    }
                    $this->action_variant = true;
                }
            }
        }
    }

    private function setProductMetaLookup( $args, $post_id = false ){

        global $woocommerce;

        if( version_compare( $woocommerce->version, '3.6.0', "<" ) )
            return false;

        if( ! $post_id )
            return false;


        if ( ! $args || empty( $args ) ) {
            return false;
        }

        global $wpdb;

        $table = $wpdb->prefix . 'wc_product_meta_lookup';

        $update_data = [
            'min_price'      => $args[ '_sale_price' ] != '' ? $args[ '_sale_price' ] : $args[ '_regular_price' ],
            'max_price'      => $args[ '_sale_price' ] != '' ? $args[ '_sale_price' ] : $args[ '_regular_price' ],
            'onsale'         => $args[ '_sale_price' ] != '' ? 1 : 0,
            'stock_quantity' => wc_stock_amount( $args[ '_stock' ] ),
            'stock_status'   => $args[ '_stock_status' ],
        ];

        $wpdb->update(
            $table,
            $update_data,
            [ 'product_id' => $post_id ]
        );

        wp_cache_set( 'lookup_table', $update_data, 'object_' . $post_id );

        return true;
    }

    public function checkOption( $str, $option ) {

        $option = $this->settings[ $option ];

        return $str == $option;
    }

    /**
     * @return mixed
     */
    public function getChangePrice() {
        return $this->changePrice;
    }

    public function setChangePrice( $old, $new ) {

        if( intval( $old ) === intval ( $new ) ) {
            $this->changePrice = false;
        } else {
            $this->changePrice = intval( $old ) < intval ( $new ) ? 'up' : 'down';
        }
    }

    /**
     * @return mixed
     */
    public function getChangeQuantity() {
        return $this->changeQuantity;
    }

    public function setChangeQuantity( $old, $new ) {

        if( intval( $old ) === intval ( $new ) ) {
            $this->changeQuantity = false;
        } else {
            $this->changeQuantity = intval( $old ) < intval ( $new ) ? 'up' : 'down';
        }
    }

    public function getDisappeared() {
        return $this->disappeared;
    }

    public function setDisappeared( $value = false ) {
        $this->disappeared = $value;
    }

    private function setQuantityZero() {

        global $wpdb;

        wc_update_product_stock( $this->post_id , 0 );
        wc_update_product_stock_status( $this->post_id, 'outofstock' );

        $results = $wpdb->get_results(
            $wpdb->prepare(
                "SELECT ID, meta_value as SKU FROM {$wpdb->posts} p
				    INNER JOIN {$wpdb->postmeta} m ON p.ID = m.post_id  
				WHERE post_parent = %d AND post_type = 'product_variation' AND meta_key = 'sellvia_post_id'", $this->post_id
            )
        );

        if( ! $results )
            return true;

        foreach( $results as $result ) {
            wc_update_product_stock( $result->ID , 0 );
            wc_update_product_stock_status( $result->ID, 'outofstock' );
        }

        return true;
    }

    protected function preparePricing() {

        $data = $this->getRowToUpdate();

        return __( 'Price', 'sel' ) . ': ' . $data['price_html'];
    }

    protected function prepareQuantity(){

        $data = $this->getRowToUpdate();

        $color = ( $data[ 'quantity' ] <= 20 ) ? '#e74c3c' : '#2cc36b';

        return sprintf( '<div class="quantity">%s: <span style="color:%s">%d</span><i></i></div>', __( 'Quantity', 'sel' ), $color, $data[ 'quantity' ] );
    }

    public static function calc_discount( $price, $salePrice){

        $price        = sellvia_floatvalue( $price );
        $salePrice    = sellvia_floatvalue( $salePrice );

        if( $price == 0 || $salePrice == 0 )
            return 0;

        return round( ( ( $price - $salePrice ) / $price ) * 100, 0 );
    }

}