<?php

namespace sellvia;

use sellvia\Media\Media;
use sellvia\Plugins;
use sellvia\ShippingBadge\BusinessDaysCalculator;
use sellvia\Preset;

class Request {

    private $productClass = NULL;

    public function __construct(){

        if( SELLVIA_PLUGIN == 'ADS'){
            $this->productClass = 'sellvia\ProductInstaller\ProductInstallerAds';
        }
        if( SELLVIA_PLUGIN == 'WOO' || SELLVIA_PLUGIN == 'ADSW' ){
            $this->productClass = 'sellvia\ProductInstaller\ProductInstallerWoo';
        }

    }

    public function actions( $post ) {

        if( isset( $post['sellvia_action'] ) ) {

            $action = 'action_' . $post['sellvia_action'];
            $args        = isset( $post['args'] ) ? $post['args'] : $post;
            $data        = [];

            if( is_array( $args ) )
                $data = $args;
            else
                parse_str( $args, $data );


            if( method_exists( $this, $action ) ) {

                return $this->$action( $data );
            }
        }

        return [ 'error' => __( 'Undefined action', 'sel' ) ];
    }

    private function action_apikeys( $params ) {

        $keys = sellvia_get_api_keys();

        return [
            'result'      => true,
            'public_key'  => $keys['public_key'],
            'secret_key'  => $keys['secret_key'],
            'lan' => [
                'Public key' => __( 'Public key', 'sel' ),
                'Secret key' => __( 'Secret key', 'sel' ),
                'API Keys'   => __( 'API Keys', 'sel' ),
                'Activate'   => __( 'Activate', 'sel' ),
                'Clear' => __( 'Clear', 'sel'),
                "I don't have the keys" => __( "I don't have the keys", 'sel'),
                "Get your keys to unlock Product List, Fast Shipping Badge and Sellvia Orders" => __( "Get your keys to unlock Product List, Fast Shipping Badge and Sellvia Orders", 'sel'),
                "Get Sellvia Keys" => __( "Get Sellvia Keys", 'sel'),
            ]
        ];
    }

    private function action_save_apikeys( $args ) {

        $result = true;
        $message = '';

        if( ! isset( $args['public_key'] ) || ! isset( $args['secret_key'] ) ) {
            return [
                'result'  => false,
                'message' => __('Wrong Data', 'sel')
            ];
        }

        $public_key = trim( $args['public_key'] );
        $secret_key = trim( $args['secret_key'] );

        if( empty($public_key) && empty($secret_key) ){

            delete_option( 'sellvia-api-keys' );

            return [
                'result'  => true,
                'redirect'    => admin_url( 'admin.php?page=sellvia' ),
                'message' => __('API Keys have been removed', 'sel'),
            ];
        }

        if( $public_key == '' ) {
            return [
                'result'  => false,
                'message' => __( 'Enter public key', 'sel' )
            ];
        }
        if( $secret_key == '' ) {
            return [
                'result'  => false,
                'message' => __( 'Enter secret key', 'sel' )
            ];
        }

        update_option('sellvia-api-keys', sellvia_prepare_api_keys([
            'public_key' => $public_key,
            'secret_key' => $secret_key
        ]));

        $result = self::call( '/sellvia_customer_plugin/apikeys' );

        if( !isset( $result['status'] ) || !isset( $result['message'] ) ){
            return [
                'result'  => false,
                'message' => __( 'Something went wrong!', 'sel' ),
                'r' => $result
            ];
        }
        if( $result['status'] != 'success' ){
            return [
                'result'  => false,
                'message' => $result['message'],
            ];
        }

        return [
            'result'  => true,
            'message' => $result['message'],
            'redirect'    => admin_url( 'admin.php?page=sellvia' ),
        ];

    }

    private function action_check_all(){

        //проверяем ключи
        $keys = sellvia_get_api_keys();

        if( empty( $keys['public_key'] ) || empty( $keys['secret_key'] ) ){
            return[
                'result' => false,
                'type' => 'warning',
                'title' => __('Error', 'sel'),
                'message' => sprintf(
                    __( 'API-keys are empty<br /><a href="%1$s">Enter API-keys</a>', 'sel' ),
                    admin_url('admin.php?page=sellvia_license')
                ),
                'plugin' => in_array( SELLVIA_PLUGIN, ['ADSW', 'WOO'] ) ? 'WOO' : SELLVIA_PLUGIN,
            ];
        }


        $result = self::call( '/sellvia_customer_plugin/checkKeys' );

        if( !isset( $result['status'] ) || $result['status'] !== 'success' ){

            return [
                'result' => false,
                'type'  => 'error',
                'title'   => __('Error', 'sel'),
                'message' => isset( $result['message'] ) ?  $result['message'] : __('Something went wrong', 'sel'),
                'plugin' => in_array( SELLVIA_PLUGIN, ['ADSW', 'WOO'] ) ? 'WOO' : SELLVIA_PLUGIN,
            ];

        }

        //проверяем, установлен и активирован ли alidropship Plugin

        $plugin_installer = new Plugins();

        $status_alids = $plugin_installer->get_plugin_status('alids');
        $status_woo = $plugin_installer->get_plugin_status('woocommerce');

        if( in_array( $status_alids, [ 'active' ] ) && in_array( $status_woo, [ 'active' ] ) ) {
            return [
                'result' => false,
                'type'  => 'error',
                'title'   => __('Error', 'sel'),
                'message' => __("You can't use Sellvia plugin with AliDropship original and Woocommerce plugin at the same time.", 'sel'),
                'plugin' => in_array( SELLVIA_PLUGIN, ['ADSW', 'WOO'] ) ? 'WOO' : SELLVIA_PLUGIN,
            ];
        }

        if( in_array( $status_alids, [ 'active' ] ) || in_array( $status_woo, [ 'active' ] ) ) {
            return [
                'result' => true,
                'plugin' => in_array( SELLVIA_PLUGIN, ['ADSW', 'WOO'] ) ? 'WOO' : SELLVIA_PLUGIN,
                'super_key' => isset($result['data']['super_key']) && intval($result['data']['super_key']) === 1
            ];
        }

        if( in_array( $status_alids, [ 'not_installed' ] ) || in_array( $status_woo, [ 'not_installed' ] ) ) {
            return [
                'result' => false,
                'type'  => 'error',
                'title'   => __('Error', 'sel'),
                'message' => __('Please install and activate AliDropship original or WooCommerce to make Sellvia plugin work properly.1', 'sel'),
                'plugin' => in_array( SELLVIA_PLUGIN, ['ADSW', 'WOO'] ) ? 'WOO' : SELLVIA_PLUGIN,
            ];
        }

        if( in_array( $status_alids, [ 'not_active' ] ) || in_array( $status_woo, [ 'not_active' ] ) ) {
            return [
                'result' => false,
                'type'  => 'error',
                'title'   => __('Error', 'sel'),
                'message' => __('Please install and activate AliDropship original or WooCommerce to make Sellvia plugin work properly.2', 'sel'),
                'plugin' => in_array( SELLVIA_PLUGIN, ['ADSW', 'WOO'] ) ? 'WOO' : SELLVIA_PLUGIN,
            ];
        }
    }

    private function action_subscription_info(){

        //проверяем ключи
        $keys = sellvia_get_api_keys();

        if( empty( $keys['public_key'] ) || empty( $keys['secret_key'] ) ){
            return[
                'result' => false,
                'type' => 'warning',
                'title' => __('Error', 'sel'),
                'message' => sprintf(
                    __( 'API-keys are empty<br /><a href="%1$s">Enter API-keys</a>', 'sel' ),
                    admin_url('admin.php?page=sellvia_license')
                )
            ];
        }

        $result = self::call( '/sellvia_customer_plugin/subscriptionInfo' );

        if( !isset( $result['status'] ) || $result['status'] !== 'success' ){

            return [
                'result' => false,
                'type'  => 'error',
                'title'   => __('Error', 'sel'),
                'message' => isset( $result['message'] ) ?  $result['message'] : __('Something went wrong', 'sel'),
            ];

        }

        $today = date_i18n( 'Y-m-d H:i:s', strtotime('tomorrow -1 second'), true );
        $date_expired = date_i18n( 'Y-m-d H:i:s', strtotime($result['data']['date_expired'] . ' tomorrow -1 second'), true );
        $date_expired_formatted = date_i18n( 'j F Y', strtotime( $date_expired ), true );

        $data = [
            'result'                 => $result['status'],
            'today'                  => $today,
            'date_expired'           => $date_expired,
            'date_expired_formatted' => $date_expired_formatted,
            'active'                 => strtotime($date_expired) >=  strtotime($today),
        ];

        if( $data['active'] ){
            $data['status_until'] = 'success';
//            $text_until   = __('Your Sellvia subscription expires on ', 'sel') . $date_expired_formatted;
            $data['text_until']   = __('Your Sellvia subscription is active', 'sel');
        }else{
            $data['status_until'] = 'warning';
            $data['text_until']   = __('Your Sellvia subscription has expired on ', 'sel') . $date_expired_formatted;
            $data['update_subscription_url'] = 'https://sellvia.com/checkout/?email=' . $result['data']['email'];
        }

        return $data;

    }

    private function action_products_list( $args ){

        $page = isset( $args['page'] ) && !empty( $args['page'] ) ? intval( $args['page'] ) : 1;
        $limit = isset( $args['limit'] ) && !empty( $args['limit'] ) ? intval( $args['limit'] ) : 20;
        $search = isset( $args['search'] ) && !empty( $args['search'] ) ? $args['search'] : [ 'niches' => null ];

        $product_installer = new $this->productClass();

        $result = $product_installer->products_list( $page, $limit, $search );

        if( !$result['success']) {
            return [
                'result' => $result['success'],
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' => isset($result['data']) && isset($result['data']['message']) ? $result['data']['message'] : __('Something went wrong', 'sel'),
            ];
        }

        return[
            'result' => $result['success'],
            'data' => $result['data'],
            'product_cat' => $product_installer->get_product_cat_list(),
        ];

    }

    private function action_import_product( $args ){

        if( ! isset( $args['sellvia_post_id'] ) ) {
            return[
                'result' => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' =>  __( 'Wrong "sellvia_post_id" parameter', 'sel' )
            ];
        }

        $product_installer = new $this->productClass();

        if( empty($args['product_cat']) || !is_array($args['product_cat'])){
            $args['product_cat'] = [];
        }

        if( empty($args['preset_id']) ){
            $args['preset_id'] = null;
        }

        $result = $product_installer->import_product( $args['sellvia_post_id'], $args['product_cat'], $args['preset_id']  );

        if( !$result['success']) {
            return [
                'result' => $result['success'],
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' => isset($result['message']) ? $result['message'] : __('Something went wrong', 'sel'),
            ];
        }

        return[
            'result' => $result['success'],
            'data' => $result['data'],
        ];
    }

    private function action_get_reset_options( $args ){

        $product_installer = new $this->productClass();

        return[
            'result' => true,
            'settings' => $product_installer->get_reset_settings(),
            'labels' => $product_installer->get_reset_labels(),
            'help' => $product_installer->get_reset_help(),
        ];
    }

    private function action_reset_product( $args ){

        if( ! isset( $args['post_id'] ) ) {
            return[
                'result' => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' =>  __( 'Wrong "post_id" parameter', 'sel' )
            ];
        };

        $args['settings'] = $args['settings'] ?? [];

        $product_installer = new $this->productClass();

        $result = $product_installer->reset_product( $args['post_id'], $args['settings'] );

        if( !$result['success']) {
            return [
                'result' => $result['success'],
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' => isset($result['message']) ? $result['message'] : __('Something went wrong', 'sel'),
            ];
        }

        return[
            'result' => $result['success'],
            'data' => $result['data'],
        ];

    }

    private function action_connect_product( $args ){

        if( ! isset( $args['post_id'] ) ) {
            return[
                'result' => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' =>  __( 'Wrong "post_id" parameter', 'sel' )
            ];
        };

        if( ! isset( $args['url'] ) || ! sellvia_is_url($args['url']) || strripos( $args['url'], SELLVIA_CATALOG_PATH ) === false ) {
            return[
                'result' => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' =>  __( 'Wrong "Sellvia product URL"', 'sel' )
            ];
        };

        $product_installer = new $this->productClass();

        $result = $product_installer->connect_product( $args['post_id'], $args['url'] );

        if( !$result['success'] ) {
            return [
                'result' => $result['success'],
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' => isset($result['message']) ? $result['message'] : __('Something went wrong', 'sel'),
            ];
        }

        return[
            'result' => $result['success'],
            'message' => $result['message'],
        ];
    }

    private function action_upload_images( $args ){

        if( ! isset( $args['post_id'] ) ) {
            return[
                'result' => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' =>  __( 'Wrong "post_id" parameter', 'sel' )
            ];
        };

        $product_installer = new $this->productClass();

        $result = $product_installer->upload_images( $args['post_id'], $args['images'] ?? [] );

        if( !$result['success']) {
            return [
                'result' => $result['success'],
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' => isset($result['message']) ? $result['message'] : __('Something went wrong', 'sel'),
            ];
        }

        return[
            'result' => $result['success'],
            'data' => $result['data'],
        ];
    }

    private function action_upload_comments( $args ){

        if( ! isset( $args['post_id'] ) ) {
            return[
                'result' => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' =>  __( 'Wrong "post_id" parameter', 'sel' )
            ];
        };

        $product_installer = new $this->productClass();

        $result = $product_installer->upload_comments( $args['post_id'], json_decode( base64_decode($args['comments']) ) );

        if( !$result['success']) {
            return [
                'result' => $result['success'],
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' => isset($result['message']) ? $result['message'] : __('Something went wrong', 'sel'),
            ];
        }

        return[
            'result' => $result['success'],
            'data' => $result['data'],
        ];
    }

    private function action_report_product( $args ){

        if( ! isset( $args['sellvia_post_id'] ) || empty($args['sellvia_post_id']) || !is_numeric( $args['sellvia_post_id'] ) ) {
            return[
                'result' => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' =>  __( 'Wrong product!', 'sel' )
            ];
        };

        if( ! isset( $args['message'] ) || empty($args['message']) ) {
            return[
                'result' => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' =>  __( 'Please describe the issue with the product', 'sel' )
            ];
        }

        $uploaded = [];

        if( !empty( $_FILES ) ){

            $obj = new Media();

            $file_names = $_FILES["file"]["name"];

            for ($i = 0; $i < count($file_names); $i++) {

                $fileName = $file_names[$i];
                $ext       = pathinfo($fileName, PATHINFO_EXTENSION);
                $imageBlob = [
                    'name' => $fileName,
                    'type' => $_FILES["file"]['type'][$i],
                    'tmp_name' => $_FILES['file']['tmp_name'][$i],
                    'error' => $_FILES['file']['error'][$i],
                    'size' => $_FILES['file']['size'][$i],
                ];

                $request = $obj->attachmentImageBlob( $args['post_id'] ?? NULL, $imageBlob, $fileName , $ext);

                if( $request && isset( $request[ 'error' ] ) ){
                    return[
                        'result' => false,
                        'type' => 'error',
                        'title' => __('Error', 'sel'),
                        'message' =>  $request[ 'error' ]
                    ];
                }

                if ( ! $request ) {
                    return[
                        'result' => false,
                        'type' => 'error',
                        'title' => __('Error', 'sel'),
                        'message' =>  'error attachmentImage'
                    ];

                }else if( !$request[ 'url' ] ){
                    return[
                        'result' => false,
                        'type' => 'error',
                        'title' => __('Error', 'sel'),
                        'message' =>  'error: images not uploaded'
                    ];
                }

                $uploaded[] = $request[ 'url' ];
            }
        }

        $result = self::call( '/sellvia_customer_plugin/report_product', [
            'sellvia_post_id' => $args['sellvia_post_id'],
            'message'         => $args['message'],
            'images'          => $uploaded
        ] );

        if( !isset( $result['status'] ) || !isset( $result['message'] ) ){
            return [
                'result'  => false,
                'message' => __( 'Something went wrong!', 'sel' ),
            ];
        }
        if( $result['status'] != 'success' ){
            return [
                'result'  => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' => $result['message']
            ];
        }

        return [
            'result'  => true,
            'type' => 'success',
            'title' => __('Product Reported', 'sel'),
            'message' => __('We will look into the issue and provide a fix if possible.', 'sel'),
        ];
    }

    private function action_get_shipping_badge_settings() {

        $settings = sellvia_get_shipping_badge_settings();

        return [
            'result'      => true,
            'settings'    => $settings,
            'lan'         => [
                'Fast Shipping Badge' => __('Fast Shipping Badge', 'sel')
            ],
            'plugin' => in_array( SELLVIA_PLUGIN, ['ADSW', 'WOO'] ) ? 'WOO' : SELLVIA_PLUGIN,
        ];
    }

    private function action_save_shipping_badge_settings( $args ) {

        $result = true;
        $message = __( 'Settings have been saved', 'sel' );

        if( ! isset( $args['settings'] ) ) {
            return [
                'result'  => false,
                'message' => __('Wrong Data', 'sel')
            ];
        }

        $default = sellvia_prepare_shipping_badge_settings();
        $settings = sellvia_get_shipping_badge_settings();
        $income = [];

        if( isset( $args['settings']['enable'] ) ){

            $income['enable'] = intval( $args['settings']['enable'] );

            if( $settings['enable'] != $income['enable'] ){
                $message =  $income['enable'] === 1 ? __( 'FAST SHIPPING BADGE has been activated', 'sel' ) : __( 'FAST SHIPPING BADGE has been deactivated', 'sel' );
            }
        }

        if( isset( $args['settings']['add_processing_time'] ) ){

            $income['add_processing_time'] = intval( $args['settings']['add_processing_time'] );

            if( $settings['add_processing_time'] != $income['add_processing_time'] ){
                $message =  $income['add_processing_time'] === 0 ? __( 'Processing time has been removed', 'sel' ) : __( 'Processing time has been added', 'sel' );
            }
        }
        if( isset( $args['settings']['processing_time'] ) ){

            $income['processing_time'] = intval( $args['settings']['processing_time'] );

            if( $settings['processing_time'] != $income['processing_time'] ){
                $message = __('Processing time has been set to', 'sel') . ' ' . sprintf ( _n("%d day", "%d days", $income['processing_time'], 'sel' ), number_format_i18n( $income['processing_time'] ) );
            }
        }
        if( isset( $args['settings']['add_arrives_by'] ) ){

            $income['add_arrives_by'] = intval( $args['settings']['add_arrives_by'] );

            if( $settings['add_arrives_by'] != $income['add_arrives_by'] ){
                $message =  $income['add_arrives_by'] === 0 ? __( "'Arrives by' has been removed", 'sel' ) : __( "'Arrives by' has been added", 'sel' );
            }
        }

        if( isset( $args['settings']['free'] ) ){

            $income['free'] = intval( $args['settings']['free'] );

            if( $settings['free'] != $income['free'] ){
                $message =  $income['free'] === 0 ? __( "'FREE' has been removed", 'sel' ) : __( "'FREE' has been added", 'sel' );
            }
        }

        if( isset( $args['settings']['main_color'] ) ){

            $income['main_color'] = trim( $args['settings']['main_color'] );

            if( empty( $income['main_color'] ) ){
                $income['main_color'] = $default['main_color'];
                $message = __( 'Main color has been set to default', 'sel' );
            }
            elseif( $settings['main_color'] != $income['main_color'] ){
                $message = __( 'Main color has been saved', 'sel' );
            }
        }
        if( isset( $args['settings']['primary_text_color'] ) ){

            $income['primary_text_color'] = trim( $args['settings']['primary_text_color'] );

            if( empty( $income['primary_text_color'] ) ){
                $income['primary_text_color'] = $default['primary_text_color'];
                $message = __( 'Primary text color has been set to default', 'sel' );
            }
            elseif( $settings['primary_text_color'] != $income['primary_text_color'] ){
                $message = __( 'Primary text color has been saved', 'sel' );
            }
        }
        if( isset( $args['settings']['secondary_text_color'] ) ){

            $income['secondary_text_color'] = trim( $args['settings']['secondary_text_color'] );

            if( empty( $income['secondary_text_color'] ) ){
                $income['secondary_text_color'] = $default['secondary_text_color'];
                $message = __( 'Secondary text color has been set to default', 'sel' );
            }
            elseif( $settings['secondary_text_color'] != $income['secondary_text_color'] ){
                $message = __( 'Secondary text color has been saved', 'sel' );
            }
        }

        $income = sellvia_prepare_shipping_badge_settings( $income );

        update_option('sellvia-shipping-badge-settings', $income );

        return [
            'result'  => true,
            'message' => $message,
            'settings' => $income
        ];

    }

    private function action_get_shipping_badge_getArrivesByDate( $args ){


        if( ! isset( $args['processingTime'] ) ) {
            return [
                'result'  => false,
                'message' => __('Wrong Data', 'sel')
            ];
        }

        $bdc = new BusinessDaysCalculator();

        $date_gtm = strtotime('now');

        $date_east = $date_gtm - 3600 * 5;

        $date_east_formatted = date_i18n( 'Y-m-d', $date_east, true);

        $arrivesByDate = $bdc->addBusinessDays( $date_east_formatted, 3 + intval( $args['processingTime'] ) );

        return [
            'result'  => true,
            'data' => [
                'arrivesByDate' => $arrivesByDate
            ],
        ];
    }

    private function action_get_product_shipping_badge_settings( $args ){

        if( ! isset( $args['post_id'] ) ) {
            return [
                'result'  => false,
                'message' => __('Wrong "post_id"', 'sel')
            ];
        }

        $sellvia_post_id = get_post_meta( $args['post_id'], 'sellvia_post_id', true  );
        if( empty( $sellvia_post_id ) )
            return [
                'result'  => false,
                'message' => __('Wrong "post_id"', 'sel')
            ];

        $product_show_badge = get_post_meta( $args['post_id'], 'sellvia_shipping_badge_show', true  ) === "0" ? 0 : 1;

        return [
            'result'  => true,
            'settings' => [
                'show' => $product_show_badge
            ],
        ];

    }

    private function action_save_product_shipping_badge_settings( $args ){

        if( ! isset( $args['post_id'] ) ) {
            return [
                'result'  => false,
                'message' => __('Wrong "post_id"', 'sel')
            ];
        }
        if( ! isset( $args['settings'] ) || !isset( $args['settings']['show'] ) ) {
            return [
                'result'  => false,
                'message' => __('Wrong data', 'sel')
            ];
        }

        update_post_meta( $args['post_id'], 'sellvia_shipping_badge_show', intval( $args['settings']['show'] ) );

        $product_show_badge = get_post_meta( $args['post_id'], 'sellvia_shipping_badge_show', true  ) === "0" ? 0 : 1;

        $message = __( 'Fast Shipping Badge has been', 'sel' ) . ' ' . ( $product_show_badge === 0 ? __( 'disabled', 'sel' ) : __( 'enabled', 'sel' ) ) . ' ' . __( 'for this product', 'sel' );

        return [
            'result'  => true,
            'message' => $message,
            'settings' => [
                'show' => $product_show_badge
            ],
        ];

    }

    private function action_get_preset_info( $args ){

        if( ! isset( $args['preset_id'] ) ) {
            return[
                'result' => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' =>  __( 'Wrong "preset_id" parameter', 'sel' )
            ];
        }

        $preset_obj = new Preset();

        $result = $preset_obj->get_preset_info( $args['preset_id'] );

        return [
            'result'            => true,
            'data' => [
                'current_preset_id' => $result['current_preset']['id'],
                'description'       => $result['current_preset']['description'],
                'category_list'     => $result['current_preset']['category_list'],
                'product_list'      => $result['current_preset']['product_list'],
                'preset_list'       => $result['preset_list'],
                'super_key'         => $result['super_key'],
            ]
        ];

    }

    private function action_remove_products( $args ){

        if( ! isset( $args['post_ids'] ) || !is_array( $args['post_ids'] ) || empty( $args['post_ids'] ) ) {
            return[
                'result' => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' =>  __( 'Nothing to remove', 'sel' )
            ];
        }

        $product_installer = new $this->productClass();

        $counter = 0;

        foreach ( $args['post_ids'] as $post_id ){

            if( $product_installer->remove_product( $post_id ) ){
                $counter++;
            }
        }

        return[
            'result' => true,
            'message' => $counter . ' ' . _n( 'product has', 'products have', $counter, 'sel' ) . ' '. __( 'been removed', 'sel' )
        ];

    }

    private function action_get_premium_all( $args ){

        $product_installer = new $this->productClass();

        $result = $product_installer->premium_all();

        if( !$result['success']) {
            return [
                'result' => $result['success'],
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' => isset($result['data']) && isset($result['data']['message']) ? $result['data']['message'] : __('Something went wrong', 'sel'),
            ];
        }

        return[
            'result' => $result['success'],
            'data' => $result['data'],
        ];

    }

    private function action_size_chart_settings(){

        $settings = sellvia_get_size_chart_settings();

        return[
            'result' => true,
            'settings' => $settings,
        ];
    }

    private function action_save_size_chart_settings( $args ){

        if( ! isset( $args['settings'] ) || !isset( $args['settings']['enable'] ) ) {
            return [
                'result'  => false,
                'message' => __('Wrong data', 'sel')
            ];
        }

        $income = sellvia_prepare_size_chart_settings( $args['settings'] );

        update_option('sellvia-size-chart-settings', $income );

        return [
            'result'  => true,
            'message' => __('Size guide settings have been updated', 'sel')
        ];
    }

    private function action_review_settings(){

        $settings = sellvia_get_review_settings();

        return[
            'result' => true,
            'settings' => $settings,
        ];
    }

    private function action_save_review_settings( $args ){

        if( ! isset( $args['settings'] ) || !isset( $args['settings']['enable'] ) || !isset( $args['settings']['enable_popup'] ) ) {
            return [
                'result'  => false,
                'message' => __('Wrong data', 'sel')
            ];
        }

        $income = sellvia_prepare_review_settings( $args['settings'] );

        update_option('sellvia-review-settings', $income );

        return [
            'result'  => true,
            'message' => __('Review settings have been updated', 'sel')
        ];
    }

    private function action_shipping_settings(){

        $settings = sellvia_get_shipping_settings();

        return[
            'result' => true,
            'settings' => $settings,
        ];
    }

    private function action_save_shipping_settings( $args ){

        if( ! isset( $args['settings'] ) || !isset( $args['settings']['enable'] ) ) {
            return [
                'result'  => false,
                'message' => __('Wrong data', 'sel')
            ];
        }

        $income = sellvia_prepare_shipping_settings( $args['settings'] );

        update_option('sellvia_shipping_settings', $income );

        return [
            'result'  => true,
            'message' => __('Shipping settings have been updated', 'sel')
        ];
    }

    private function action_price_model_settings(){

        $settings = sellvia_get_price_model_settings();

        return[
            'result' => true,
            'settings' => $settings,
        ];
    }

    private function action_save_price_model_settings( $args ){

        if( ! isset( $args['settings'] ) || !isset( $args['settings']['value'] )
        ) {
            return [
                'result'  => false,
                'message' => __('Wrong data', 'sel')
            ];
        }

        $income = sellvia_prepare_price_model_settings( $args['settings'] );

        update_option('sellvia_price_model_settings', $income );

        return [
            'result'  => true,
            'message' => __('Pricing strategy settings have been updated', 'sel')
        ];
    }

    private function action_fulfillment_settings(){

        $settings = sellvia_get_fulfillment_settings();

        return[
            'result' => true,
            'settings' => $settings,
        ];
    }

    private function action_save_fulfillment_settings( $args ){

        if( ! isset( $args['settings'] ) || !isset( $args['settings']['auto'] ) ) {
            return [
                'result'  => false,
                'message' => __('Wrong data', 'sel')
            ];
        }

        $income = sellvia_prepare_fulfillment_settings( $args['settings'] );

        update_option('sellvia_fulfillment_settings', $income );

        return [
            'result'  => true,
            'message' => __('Fulfillment settings have been updated', 'sel')
        ];
    }

    private function action_nice_attributes_settings(){

        $settings = sellvia_get_nice_attributes_settings();

        return[
            'result' => true,
            'settings' => $settings,
        ];
    }

    private function action_save_nice_attributes_settings( $args ){

        if( ! isset( $args['settings'] )
            || !isset( $args['settings']['enable'] )
            || !isset( $args['settings']['text_only'] )
            || !isset( $args['settings']['custom_css'] )
            || !isset( $args['settings']['custom_width'] )
            || !isset( $args['settings']['custom_height'] )
            || !isset( $args['settings']['round_variations'] )
            || !isset( $args['settings']['variations_color_apply'] )
        ) {
            return [
                'result'  => false,
                'message' => __('Wrong data', 'sel')
            ];
        }

        $income = sellvia_prepare_nice_attributes_settings( $args['settings'] );

        $income['enable'] = intval( $income['enable'] ) === 1 ? 1 : 0;
        $income['text_only'] = intval( $income['text_only'] ) === 1 ? 1 : 0;
        $income['custom_css'] = trim( $income['custom_css'] ) ;
        $income['custom_width'] = is_numeric( trim( $income['custom_width'] ) ) && intval( trim( $income['custom_width'] ) ) > 0 ? min( (int) trim( $income['custom_width'] ), 100 ) : '';
        $income['custom_height'] =  is_numeric( trim( $income['custom_height'] ) ) && intval( trim( $income['custom_height'] ) ) > 0 ? min( (int) trim( $income['custom_height'] ), 100 ) : '';
        $income['round_variations'] = intval( $income['round_variations'] ) === 1 ? 1 : 0;
        $income['variations_color_apply'] = intval( $income['variations_color_apply'] ) === 1 ? 1 : 0;

        update_option('sellvia-nice-attributes-settings', $income );

        return [
            'result'  => true,
            'message' => __('Attribute settings have been updated', 'sel')
        ];
    }

    protected function action_get_marketing_url( $args ){

        if( ! isset( $args['post_id'] ) ) {
            return[
                'result' => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' =>  __( 'Wrong "post_id"', 'sel' )
            ];
        }

        $sellvia_marketing_url = get_post_meta( $args['post_id'], 'sellvia_marketing_url', true );

        return [
            'result'                => true,
            'sellvia_marketing_url' => $sellvia_marketing_url,
        ];
    }

    static function call( $path, $args = [] ) {

        $keys = sellvia_get_api_keys();

        $public_key  = $keys['public_key'];
        $secret_key  = $keys['secret_key'];

        $auth_args = [
            'public_key'  => $public_key,
            'signature'   => md5( $public_key . $secret_key ),
            'domain'      => get_home_url(),
            'plugin'      => 'sellvia',
        ];

        $args = array_merge( $args, $auth_args );

        $response = wp_remote_post( SELLVIA_REQUEST_PATH . $path, [
            'method'    => 'POST',
            'timeout'   => 120,
            'sslverify' => false,
            'headers'   => [],
            'body'      => json_encode( $args )
        ] );

//        var_dump($response);

        if( is_wp_error( $response ) )
            return [ 'error' => $response->get_error_message() ];

        if( ! wp_remote_retrieve_response_code( $response ) == '200' )
            return [ 'error' => __( 'The response from the server has not been received', 'sel' ) ];

        return json_decode( wp_remote_retrieve_body( $response ), true );
    }

}