<?php

namespace sellvia\Shipping;

class Shipping {

    protected $settings = [];

    public function __construct(){

        $this->setup();

        add_action( 'delete_post',  [ $this, 'delete_shipping' ], 5 );
    }

    private function setup(){

        $this->settings = sellvia_get_shipping_settings();
    }

    public function getSettings(){

        return $this->settings;
    }

    public function hasWhShipping( $post_id ){

        global $wpdb;

        $count = $wpdb->get_var( $wpdb->prepare(
            "SELECT COUNT(post_id) as con FROM {$wpdb->sellvia_shipping_relation} WHERE post_id = %d",
            $post_id
        ) );

        return $count > 0;
    }

    public function getWHShipping( $post_id, $skuAttrId ){

        global $wpdb;

        return $wpdb->get_results( $wpdb->prepare(
            "SELECT shipping_group_id, count FROM {$wpdb->sellvia_shipping_relation} WHERE post_id = %d AND skuAttrId = %s",
            $post_id,
            $skuAttrId
        ), ARRAY_A );
    }

    public function getWhShippingGroups( $shipping_group_ids = [] ){

        global $wpdb;

        $ids = implode( ',' , $shipping_group_ids );

        $result = $wpdb->get_results(
            "SELECT * FROM {$wpdb->sellvia_shipping_group} WHERE id IN({$ids})",
            ARRAY_A
        );

        return array_map( function ($i){
            $i['details']        = maybe_unserialize( $i['details'] );
            $i['retail_details'] = maybe_unserialize( $i['retail_details'] );
            return $i;
        }, $result );

    }

    public function countShippingGroup ( $group, $count ){

        switch ( intval( $this->settings['enable'] ) ){
            case 0:
            default:
                return 0;

            case 1:
                return self::countRetailShippingGroup( $group, $count );

            case 2:
                return self::countWhShippingGroup( $group, $count );
        }
    }

    public static function countRetailShippingGroup( $group = [], $count = 0 ){

        if( empty( $group )
            || empty( $group['retail'] )
            || empty( $group['retail_details'] )
            || !is_array( $group['retail_details'] )
            || count( $group['retail_details'] ) !== 5 ) return 0;

        $result = 0;

        $profit = floatval( $group['retail'] );

        for( $i = 1; $i <= $count; $i++ ) {
            $result += $profit + floatval( $group['retail_details'][ min($i, 5) ] );
        }

        if( $result == 0 ) {
            $result = self::countWhShippingGroup( $group, $count );
        }
        return $result;
    }

    public static function countWhShippingGroup( $group = [], $count = 0 ){

        if( empty( $group )
            || empty( $group['profit'] )
            || empty( $group['details'] )
            || !is_array( $group['details'] )
            || count( $group['details'] ) !== 5 ) return 0;

        $result = 0;

        $profit = floatval( $group['profit'] );

        $shipping_price = floatval( $group['details'][ $count < 5 ? $count : 5 ] + ( $count > 5 ? ( $group['details'][5] / 5 ) * ( $count - 5 ) : 0 ) );

        for( $i = 1; $i <= $count; $i++ ) {
            $result += ( $i >= 4 ? $profit * (1/3) : $profit * (1/$i) );
        }

        $result += $shipping_price;

        return $result;

    }

    public function delete_shipping( $pid ){

        $this->deleteShippingRelations( $pid );
    }

    private function deleteShippingRelations( $post_id ){

        global $wpdb;

        $wpdb->delete( $wpdb->sellvia_shipping_relation, [
            'post_id' => $post_id
        ], ['%d'] );
    }

    public static function save_product_shipping( $post_id, $skuAttr = [], $wh_shipping = [] ){

        global $wpdb;

        if( !empty( $wh_shipping ) ) {

            $updated_shipping_relation_ids = [];

            if( empty( $skuAttr ) ) {

                if( !empty( $wh_shipping['default'] ) ){

                    foreach ( $wh_shipping['default'] as $shipping ){

                        if( !empty( $shipping['shipping_group'] ) ){

                            $shipping_group_id = self::shipping_group_update( $shipping['shipping_group'] );

                            $shipping_relation_id = self::shipping_relation_update( $post_id, 'default', $shipping['wh_pr_id'], $shipping_group_id, $shipping['count'] );

                            $updated_shipping_relation_ids[] = $shipping_relation_id;
                        }
                    }
                }
            } else {

                foreach ( $skuAttr as $skuAttrId => $val ) {

                    if( array_key_exists( $skuAttrId, $wh_shipping ) ){

                        foreach ( $wh_shipping[$skuAttrId] as $shipping ){

                            if( !empty( $shipping['shipping_group'] ) ){

                                $shipping_group_id = self::shipping_group_update( $shipping['shipping_group'] );

                                $shipping_relation_id = self::shipping_relation_update( $post_id, $skuAttrId, $shipping['wh_pr_id'], $shipping_group_id, $shipping['count'] );

                                $updated_shipping_relation_ids[] = $shipping_relation_id;
                            }
                        }
                    }
                }
            }

            if( !empty( $updated_shipping_relation_ids ) ){

                $updated_shipping_relation_ids = array_map( 'intval', $updated_shipping_relation_ids );
                $updated_ids = implode( ',', $updated_shipping_relation_ids );

                $wpdb->query(
                    "DELETE FROM {$wpdb->sellvia_shipping_relation} WHERE post_id = {$post_id} AND id NOT IN ({$updated_ids})"
                );
            }
        }else{
            $wpdb->query( $wpdb->prepare( "DELETE FROM {$wpdb->sellvia_shipping_relation} WHERE post_id = %d", $post_id ) );
        }

    }

    public static function shipping_group_update( $shipping_group ){

        global $wpdb;

        $shipping_group_id = $wpdb->get_var(
            $wpdb->prepare(
                "SELECT id FROM {$wpdb->sellvia_shipping_group}
                                                WHERE wh_id = %d AND wh_shipping_id = %d",
                intval( $shipping_group['wh_id'] ),
                intval( $shipping_group['shipping_id'] )
            )
        );

        if ( !empty( $shipping_group_id ) ) {

            $wpdb->update(
                $wpdb->sellvia_shipping_group,
                [
                    'title'          => $shipping_group['title'],
                    'profit'         => sellvia_floatvalue( $shipping_group['profit'] ),
                    'details'        => maybe_serialize( array_map( 'sellvia_floatvalue', $shipping_group['details'] ) ),
                    'retail'         => sellvia_floatvalue( $shipping_group['retail']),
                    'retail_details' => maybe_serialize( array_map( 'sellvia_floatvalue', $shipping_group['retail_details'] ) ),
                ],
                [
                    'id' => $shipping_group_id
                ],
                [ '%s', '%f', '%s', '%f', '%s' ],
                [ '%d' ]
            );
        } else {

            $wpdb->insert(
                $wpdb->sellvia_shipping_group,
                [
                    'wh_id'          => intval( $shipping_group['wh_id'] ),
                    'wh_shipping_id' => intval( $shipping_group['shipping_id'] ),
                    'title'          => $shipping_group['title'],
                    'profit'         => sellvia_floatvalue( $shipping_group['profit'] ),
                    'details'        => maybe_serialize( array_map( 'sellvia_floatvalue', $shipping_group['details'] ) ),
                    'retail'         => sellvia_floatvalue( $shipping_group['retail']),
                    'retail_details' => maybe_serialize( array_map( 'sellvia_floatvalue', $shipping_group['retail_details'] ) ),
                ],
                [ '%d', '%d', '%s', '%f', '%s', '%f', '%s' ]
            );

            $shipping_group_id = $wpdb->insert_id;
        }

        return $shipping_group_id;
    }

    public static function shipping_relation_update( $post_id, $skuAttrId, $wh_pr_id, $shipping_group_id, $count ){

        global $wpdb;

        $shipping_relation_id = $wpdb->get_var(
            $wpdb->prepare(
                "SELECT id FROM {$wpdb->sellvia_shipping_relation}
                    WHERE post_id = %d AND skuAttrId = %s AND wh_pr_id = %d AND shipping_group_id = %d",
                $post_id,
                trim( $skuAttrId ),
                intval( $wh_pr_id ),
                $shipping_group_id
            )
        );

        if ( !empty( $shipping_relation_id ) ) {

            $wpdb->update(
                $wpdb->sellvia_shipping_relation,
                [
                    'count' => intval( $count ),
                ],
                [
                    'id' => $shipping_relation_id
                ],
                [ '%d' ],
                [ '%d' ]
            );

        } else {

            $wpdb->insert(
                $wpdb->sellvia_shipping_relation,
                [
                    'post_id'           => $post_id,
                    'skuAttrId'         => trim($skuAttrId),
                    'wh_pr_id'          => intval( $wh_pr_id ),
                    'shipping_group_id' => $shipping_group_id,
                    'count'             => intval( $count ),
                ],
                [ '%d', '%s', '%d', '%d', '%d' ]
            );

            $shipping_relation_id = $wpdb->insert_id;
        }

        return $shipping_relation_id;

    }
}
