<?php

namespace sellvia\Shipping;

use sellvia\Shipping\Shipping;

class ShippingAds extends Shipping {

    public function __construct() {

        parent::__construct();

        if( intval( $this->settings['enable'] ) !== 0 ){
            add_filter( 'ads_basket_total', [ $this, 'ads_basket_total' ], 10, 2 );
            add_filter('ads_shipping_product', [$this, 'ads_shipping_product'], 10, 2);
        }

        add_filter( 'ads_shipping_name_order', [ $this, 'ads_shipping_name_order' ], 10, 3);
    }

    public function ads_shipping_product($shipping, $post_id){

        if( !$this->hasWhShipping( $post_id ) ){
            return $shipping;
        }

        return [
            'fast_us_shipping' => [
                'post_id' => $post_id,
                'code' => 'fast_us_shipping',
                'title' => __( 'Fast US shipping', 'sel'),
                'time' => $this->settings['time'],
                'enable' => 1,
                'cost' => 0,
                '_cost_nc' => 0,
                'cur_cost' => \ads_get_price_cur_current(0.00, 'USD'),
            ]
        ];

    }


    public function ads_basket_total( $basket, $object ){

        $whShippingGroupsCount = [];

        $shipping_total = 0;

        $orders = $object->getItems();

        foreach( $orders as $k => &$order ) {

            $product = $order->getProduct();

            $variations = $order->getVariations();


            if( empty( $variations ) ){
                $variation_code = 'default';
            }else{
                $variation_code = implode(';', array_map(
                    function ( $i ){
                        return $i['prop_id'] . ':' . $i['sku_id'];
                    },
                    $variations
                ) );
            }

            $variation_shipping = $this->getWHShipping( $product->getPostId(), $variation_code );

            if( !empty( $variation_shipping ) ){

                foreach ( $variation_shipping as $vs_item ){

                    if( isset( $whShippingGroupsCount[ $vs_item['shipping_group_id'] ] ) ){
                        $whShippingGroupsCount[ $vs_item['shipping_group_id'] ] += intval( $vs_item['count'] ) * intval( $order->getQuantity() );
                    }else{
                        $whShippingGroupsCount[ $vs_item['shipping_group_id'] ] = intval( $vs_item['count'] ) * intval( $order->getQuantity() );
                    }
                }
            }else{

                $shipping_total += $order->getTotalShippingPrice();
            }
        }

        if( !empty($whShippingGroupsCount) ) {

            $whShippingGroups = $this->getWhShippingGroups( array_keys( $whShippingGroupsCount ) );

            foreach ( $whShippingGroups as $whShippingGroup ){

                $shipping_total += $this->countShippingGroup( $whShippingGroup, $whShippingGroupsCount[ $whShippingGroup['id'] ] );
            }
        }

        $fast_us_shipping = [
            'fast_us_shipping' => [
                'code' => 'fast_us_shipping',
                'enable' => 1,
                'title' => __( 'Fast US shipping', 'sel'),
                'time' => $this->settings['time'],
                'cost' => $shipping_total,
                'min_cost_enabled' => 0.0,
                'comment' => '',
                '_cost_nc' => ads_floatvalue($shipping_total),
                'cur_cost_not_rounded_decimal' => floatval( ads_price_convert_current( $shipping_total, ADS_MAIN_CUR )),
                'cur_cost' => ads_get_price_cur_current( $shipping_total, ADS_MAIN_CUR ),
            ],
        ];

        foreach( $orders as $k => &$order ) {
            $order->setAvailableShipping( $fast_us_shipping );
            $order->setShipping( 'fast_us_shipping' );
            $order->setShippingPrice( 0 );
            $order->setTotalShippingPrice( 0 );
        }

        $object->setItems($orders);

        $object->setAvailableShipping($fast_us_shipping);

        $object->setShippingBasket('fast_us_shipping');

        $object->setShippingTotal( $shipping_total );

        $object->setOrdersShipping($fast_us_shipping);

        $object->applyDiscount();
        $basket['discount']      = $object->getDiscount();
        $basket['discount_info'] = $object->getDiscountInfo();

        return $basket;
    }

    public function ads_shipping_name_order($shipping, $trans_id, $id){

        $map = [
            'fast_us_shipping' => __( 'Fast US shipping', 'sel' ),
        ];

        return $map[$shipping] ?? $shipping;
    }

}
