<?php

namespace sellvia\Shipping;

use WC_Shipping_Method;
use WC_Product_Variation;

class ShippingWooMethod extends WC_Shipping_Method{

    const WOO_TRUE = 'yes';
    const WOO_FALSE = 'no';

    //Settings from ShippingWoo
    protected $shipping_enable;
    protected $shipping_time;

    private $shipping_override;
    private $isSellviaPackage;

    /** @var Shipping $sellvia_shipping_instance */
    private $sellvia_shipping_instance;

    public function __construct( $settings = [], $sellvia_shipping_instance = null ) {
        parent::__construct();

        $this->sellvia_shipping_instance = $sellvia_shipping_instance;

        $this->setSettings( $settings );

        $this->init();
    }

    public function init(){

        $this->id = 'fast_us_shipping';
        $this->method_title = __( 'Sellvia Shipping', 'sel' );
        $this->method_description = __( 'Calculate live shipping rates based on actual Sellvia shipping costs.', 'sel' );
        $this->title = __( 'Sellvia Shipping', 'sel' );

        $this->shipping_override = 1;

        add_action('woocommerce_load_shipping_methods', [$this, 'load_shipping_methods'], 100);

//        add_filter('woocommerce_shipping_methods', [$this, 'add_sellvia_shipping_method']);

        add_filter('woocommerce_cart_shipping_packages', [$this, 'calculate_shipping_rates'], 100);
    }

    public function setSettings( $settings = [] ){

        $this->shipping_enable = isset( $settings['enable'] ) ? intval( $settings['enable'] ) : 0;
        $this->shipping_time = isset( $settings['time'] ) ? strval( $settings['time'] ) : '';
    }

    public function add_sellvia_shipping_method( $methods ){

        return 1 === intval( $this->shipping_override ) && true === $this->isSellviaPackage ? [] : $methods;
    }

    function load_shipping_methods( $package ){

        $this->isSellviaPackage = false;
//        Own settings
//        if (!$package) {
//            WC()->shipping()->register_shipping_method($this);
//
//            return;
//        }

        if( self::WOO_FALSE === $this->enabled ) {
            return;
        }

        if ( isset($package['managed_by_sellvia'] ) && true === $package['managed_by_sellvia'] ) {
            if (1 === intval( $this->shipping_override )) {
                WC()->shipping()->unregister_shipping_methods();
            }

            $this->isSellviaPackage = true;

            WC()->shipping()->register_shipping_method($this);
        }
    }

    public function calculate_shipping_rates( $packages = [] ){

        if ( intval( $this->shipping_enable ) === 0 ) {
            return $packages;
        }

        $whShippingGroupsCount = [];

        $shipping_total = 0;

        $split_variations = [];

        foreach ($packages as &$package) {

            if( !isset( $package['destination'] ) || !isset( $package['destination']['country'] ) || !in_array( $package['destination']['country'], ['US'] ) ) continue;

            foreach ($package['contents'] as $key => $variation ) {
                if ($variation && $variation['data']) {

                    /** @var WC_Product_Variation $productVariation */
                    $productVariation = $variation['data'];

                    $sellvia_post_id = null;

                    //Simple product
                    if( $productVariation->is_type( 'simple' ) ){
                        $product_id = $productVariation->get_id();
                        $sellvia_post_id = get_post_meta( $productVariation->get_id(), 'sellvia_post_id', true );
                        $variation_code = 'default';
                    }
                    // Product variation
                    if( $productVariation->is_type('variation') ) {
                        $product_id = $productVariation->get_parent_id();
                        $sellvia_post_id = get_post_meta( $product_id, 'sellvia_post_id', true );
                        $variation_code = get_post_meta( $productVariation->get_id(), 'adswSKU', true );
                    }

                    if( empty( $product_id ) || empty( $sellvia_post_id ) || empty( $variation_code ) ){
                        continue;
                    }

                    $variation_shipping = $this->sellvia_shipping_instance->getWHShipping( $product_id, $variation_code );

                    if( empty( $variation_shipping ) ) {
                        continue;
                    }

                    foreach ( $variation_shipping as $vs_item ){

                        if( isset( $whShippingGroupsCount[ $vs_item['shipping_group_id'] ] ) ){
                            $whShippingGroupsCount[ $vs_item['shipping_group_id'] ] += intval( $vs_item['count'] ) * intval( $variation['quantity'] );
                        }else{
                            $whShippingGroupsCount[ $vs_item['shipping_group_id'] ] = intval( $vs_item['count'] ) * intval( $variation['quantity'] );
                        }
                    }

                    $split_variations[$key] = $variation;

                    unset($package['contents'][$key]);
                }
            }
        }

        unset($package);

        if( !empty($whShippingGroupsCount) && !empty( $split_variations ) ) {

            $whShippingGroups = $this->sellvia_shipping_instance->getWhShippingGroups( array_keys( $whShippingGroupsCount ) );

            foreach ( $whShippingGroups as $whShippingGroup ){

                $shipping_total += $this->sellvia_shipping_instance->countShippingGroup( $whShippingGroup, $whShippingGroupsCount[ $whShippingGroup['id'] ] );
            }
        }

        if( !empty( $split_variations ) ) {
            $packages[] = array(
                'contents' => $split_variations,
                'contents_cost' => array_sum(wp_list_pluck($split_variations, 'line_total')),
                'applied_coupons' => WC()->cart->get_applied_coupons(),
                'managed_by_sellvia' => true,
                'sellvia_shipping_rate' => $shipping_total,
                'user' => array(
                    'ID' => get_current_user_id(),
                ),
                'destination' => $packages[0]['destination']
            );
        }

        foreach ( $packages as $key => $package ) {
            if( empty( $package['contents'] ) ){
                unset( $packages[$key] );
            } else {
                $package['contents_cost'] = array_sum(wp_list_pluck($package['contents'], 'line_total'));
            }
        }

        return $packages;
    }

    public function calculate_shipping( $package = [] ){

        if (isset($package['managed_by_sellvia']) && $package['managed_by_sellvia'] === true) {

            $this->add_rate([
                'id' => $this->id,
                'label' => 'Fast US shipping',
                'cost' => $package['sellvia_shipping_rate'],
                'calc_tax' => 'per_order',
                'package' => $package
            ]);
        }
    }
}