<?php


namespace sellvia\ShippingBadge;

use sellvia\location\locationAPI;

class ShippingBadge {


    private $enable = false;

    private $add_processing_time = true;

    private $processing_time = 1;

    private $add_arrives_by = true;

    private $free = true;

    private $main_color = '#018ed3';

    private $primary_text_color = '#444444';

    private $secondary_text_color = '#27bf2f';

    private $location = NULL;

    private $fsb_user_time;

    private $processing_days = 3;


    public function __construct() {

        $this->init();
    }
	
	public function actions( $post ) {
		
		if( isset( $post['sellvia_action'] ) ) {
			
			$actions = 'action_' . $post['sellvia_action'];
			$args    = $post['args'] ?? $post;
			$data    = [];
			
			if( is_array( $args ) )
				$data = $args;
			else
				parse_str( $args, $data );
			
			
			if( method_exists( $this, $actions ) )
				return $this->$actions( $data );
		}
		
		return [ 'error' => __( 'Undefined action', 'sel' ) ];
	}

    public function init(){

        $this->settings();

        if (intval($this->enable) !== 1)
            return;

        $this->location = $this->getLocation();

        if( !current_user_can('activate_plugins') ) {
            if (empty($this->location))
                return;

            if (isset($this->location['country']) && $this->location['country'] !== 'US')
                return;
        }

        $this->user_data();

        $theme = wp_get_theme( get_template() );

        if( in_array( $theme->get('Name'), [ "Davinci", "Davinci 2" ] ) ){

            add_action('ads_template_single_sku_after', [$this, 'render'], 10, 1);

        } else {

            add_action('ads_single_product_before_product_actions', [$this, 'render'], 10, 1);
            add_action('woocommerce_before_add_to_cart_button', [$this, 'renderWoo']);
        }

        add_filter( 'store_benefits_days', [$this, 'store_benefits_days'], 10, 1 );

        add_action( 'the_post', function () {
            add_shortcode( 'fast_shipping_badge' , [ $this, 'shortcode' ] );
        }, 1000 );

    }

    public function settings(){

        $settings = sellvia_get_shipping_badge_settings();

        $this->enable               = boolval( $settings['enable'] );
        $this->add_processing_time  = boolval( $settings['add_processing_time'] );
        $this->processing_time      = $this->add_processing_time ? intval( $settings['processing_time'] ) : 0;
        $this->add_arrives_by       = boolval( $settings['add_arrives_by'] );
        $this->free                 = boolval( $settings['free'] );
        $this->main_color           = trim( $settings['main_color'] );
        $this->primary_text_color   = trim($settings['primary_text_color'] );
        $this->secondary_text_color = trim( $settings['secondary_text_color'] );
    }

    public function initJS(){
        wp_enqueue_script('sellvia_shipping_badge_script', SELLVIA_URL . '/assets/js/front/shipping-badge.min.js', ['jquery'], SELLVIA_VERSION);
    }

    public function initCSS(){

    }

    private function user_data(){

        if( ! isset( $_COOKIE[ 'fsb_user_time' ] ) ){

            setcookie( 'fsb_user_time', time() + HOUR_IN_SECONDS * 2, time() + HOUR_IN_SECONDS * 2, '/' );
        }
    }

    public function render( $post_id = null, $show = false ) {

        if( empty( $post_id ) )
            return;

        $sellvia_post_id = get_post_meta( $post_id, 'sellvia_post_id', true  );

        if( empty( $sellvia_post_id ) )
            return;

        $product_show_badge = get_post_meta( $post_id, 'sellvia_shipping_badge_show', true  );

        if( $product_show_badge === "0" && !$show )
            return;

        add_action('wp_enqueue_scripts', [$this, 'initJS']);
        add_action('wp_enqueue_scripts', [$this, 'initCSS']);

        $this->fsb_user_time = isset($_COOKIE['fsb_user_time'] ) ? $_COOKIE['fsb_user_time'] : time() + HOUR_IN_SECONDS * 2;

        $total_seconds = $this->fsb_user_time - time();

        $hours = intdiv ( $total_seconds, 3600 );
        $minutes = intdiv ( $total_seconds - $hours * 3600, 60 );

        $arrivesByBlock = '';

        if( $this->add_arrives_by ){

            $arrivesBy = $this->us_states_delivery();

            $arrivesByBlock = sprintf(
                '<div class="fsbe-arrives-by" style="color:' . $this->primary_text_color .';">'.
                'Arrives by <span>' . date_i18n( 'l, M j', strtotime($arrivesBy) ) . '</span>' .
                '</div>'
            );
        }

        printf (
            '<div class="fsbe-wrap">'.
            '<div class="fsbe-main" style="color:' . $this->main_color . ';">'.
            '<svg width="45" viewBox="0 0 33.2 16.7" style="enable-background:new 0 0 33.2 16.7;" >
                        <path class="st0" style="fill:' . $this->main_color . ';" d="M32.5,10.6L32.5,10.6c0.1-0.7,0.2-1.4,0.3-2.2c0.3-1.8-1.5-1.9-1.5-1.9l-1.2,0c-0.4-0.8-0.8-1.8-1.2-2.7
            C28.2,2.2,27,2.3,27,2.3h-4.1l0.3-1.5C23.3,0.4,22.9,0,22.5,0H2.9h0l0,0C2.5,0,0.8,0,0.5,0S0,0.4,0,0.7s0.2,1.6,3,1.6l0,0l0,0h13.3
            c0.4,0,0.7,0.4,0.6,0.8c-0.1,0.4-0.5,0.8-0.9,0.8H4.9c0,0,0,0-0.1,0v0c-0.4,0-2.1,0-2.4,0c-0.3,0-0.5,0.4-0.5,0.7S2.1,6,4.9,6v0h8.8
            c0.4,0,0.7,0.4,0.6,0.8c-0.1,0.4-0.5,0.8-0.9,0.8H7.2c0,0-0.1,0-0.1,0l0,0c-0.4,0-2.1,0-2.4,0c-0.3,0-0.5,0.4-0.5,0.7
            c0,0.3,0.2,1.6,3,1.6v0c0,0,0,0,0,0h2.2l-0.7,3.9c-0.1,0.5,0.2,0.8,0.7,0.8h1.5c0.2,1.3,1.3,2.2,2.7,2.2c1.4,0,2.8-0.9,3.5-2.2h7.2
            c0.2,1.3,1.3,2.2,2.7,2.2c1.4,0,2.8-0.9,3.5-2.2h1.3c0.5,0,0.9-0.4,1-0.8l0.4-2.2C33.2,11,32.9,10.6,32.5,10.6z M15.7,13.5
            c-0.2,0.9-1,1.6-1.9,1.6c-0.9,0-1.5-0.7-1.3-1.6c0.2-0.9,1-1.6,1.9-1.6C15.3,11.9,15.9,12.6,15.7,13.5z M22.9,6.7
            c-0.4,0-0.7-0.4-0.6-0.8l0.3-1.5c0.1-0.5,0.5-0.8,0.9-0.8h3.1c0.7,0,1,0.9,1,0.9s0.4,1,0.6,1.4c0.2,0.5,0.1,0.8-0.9,0.8H22.9z
             M29.1,13.5c-0.2,0.9-1,1.6-1.9,1.6c-0.9,0-1.4-0.7-1.3-1.6c0.2-0.9,1-1.6,1.9-1.6C28.7,11.9,29.3,12.6,29.1,13.5z"/>
                    </svg> <span>' . ( $this->free ? "FREE " : "" ) . ( $this->processing_days +  $this->processing_time ) . '-Day Delivery</span>'.
            '</div>'.
            '<div class="fsbe-order-within" style="color:' . $this->primary_text_color . ';">'.
            'If you order within <span class="fsbe-time" style="color:' . $this->secondary_text_color . ';">' . ( $hours > 0 ? '<span class="fsbe-hours">'. $hours .' '. _n('hour', 'hours', $hours ) .'</span>' : '' ) . ' ' . '<span class="fsbe-minutes">'. $minutes .' mins</span>' . '</span>'.
            '</div>'.
            $arrivesByBlock .
            '</div>'
        );

        printf('
            <style scoped lang="css">
                .fsbe-wrap {
                    font-size: 15px;
                    line-height: 1.2;
                    margin-top: 30px;
                    margin-bottom: 30px;
                }
                .fsbe-wrap .fsbe-main {
                    font-size: 18px;
                }
                .fsbe-wrap .fsbe-main svg {
                    vertical-align: text-bottom;
                }
                .fsbe-wrap .fsbe-main span{
                    font-style: italic;
                    font-weight: 700;
                }

                .fsbe-wrap .fsbe-order-within{
                    margin-top: .725em;
                }
                .fsbe-wrap .fsbe-arrives-by{
                    margin-top: .5em;
                }
                .fsbe-wrap .fsbe-arrives-b span{
                    font-weight: 700;
                }
            </style>'
        );
    }

    public function renderWoo(){
        global $post;

        if( empty(  $post->ID ))
            return;

        $this->render( $post->ID );
    }


    /**
     * [fast_shipping_badge post_id="5967" force_show="0"]
     *  post_id - product post ID
     *  force_show: 0 - use product settings, 1 - ignore product settings and show badge anyway
     **/
    public function shortcode( $atts ){

        if( empty( $atts['post_id'] ) || !is_numeric($atts['post_id']) )
            return;

        if( empty( $atts['force_show'] ) ){
            $atts['force_show'] = "0";
        }

        ob_start();

        $this->render( $atts['post_id'], $atts['force_show'] === "1" );

        return ob_get_clean();
    }
	
	public function action_get_badge_html( $args ){
		
		if( ! isset( $args['post_id'] ) )
			return[
				'result' => false,
				'error' => 'Post ID not found'
			];
		
		$args['force_show'] = isset( $args['force_show'] ) ? intval( $args['force_show'] ) : 0;
		
		if( empty( $this->location ) || isset( $this->location['country'] ) && $this->location['country'] !== 'US' ) {
			$text = '';
		} else {
			ob_start();
			$this->render( $args['post_id'], $args['force_show'] === 1 );
			$text = ob_get_clean();
		}
		
		return [
			'result' => true,
			'html'   => $text,
		];
	}

    public function getLocation(){

        $locationAPI = new locationAPI();

        return $locationAPI->getCountry();
    }

    private function us_states_delivery(){

        $bdc = new BusinessDaysCalculator();

        $date_gtm = strtotime('now');

        $date_east = $date_gtm - 3600 * 5;

        $date_east_formatted = date_i18n( 'Y-m-d', $date_east, true);

        return $bdc->addBusinessDays( $date_east_formatted, $this->processing_days + $this->processing_time );
    }

    public function store_benefits_days( $date ){

        $date = strtotime( $this->us_states_delivery() );

        return $date;
    }
}