<?php


namespace sellvia\Tracking;

use \sellvia\email\orderShippedNotification as orderShippedNotification;
use \sellvia\email\orderTrackingChangedNotification as orderTrackingChangedNotification;


class TrackingWoo extends Tracking {

    public function __construct(){
        parent::__construct();
        $this->init();
    }

    private function init(){

        if( in_array( SELLVIA_PLUGIN, ['WOO'] ) ) {

            // Add email templates
            add_filter('woocommerce_email_classes', [$this, 'custom_woocommerce_emails']);

            //add woocommerce order action to resend order shipped notification
            add_filter( 'woocommerce_order_actions', [ $this, 'send_order_shipped_notification_option' ], 10, 1 );

            add_action( 'woocommerce_order_action_send_sellvia_order_shipped_notification', [ $this, 'order_shipped_notification_action' ], 10, 1 );
        }
    }

    public function custom_woocommerce_emails( $email_classes ){

        $email_classes[ 'sellvia_order_shipped_notification' ] = new orderShippedNotification();

        $email_classes[ 'sellvia_order_tracking_changed_notification' ] = new orderTrackingChangedNotification();

        return $email_classes;
    }


    public function send_order_shipped_notification_option( $args ){

        global $theorder;
        $order_id = $theorder->get_id();
        $already = get_post_meta( $order_id, 'sellviaSendEmail', true );

        if( ! empty( $already ) && $already == 'send' ) {
            $args['send_sellvia_order_shipped_notification'] = __( 'Resend order shipped notification', 'sel' );
        }

        return $args;
    }


    public function order_shipped_notification_action( $order ){

        wc_switch_to_site_locale();

        do_action( 'woocommerce_before_resend_order_emails', $order );

        $mailer = \WC()->mailer();

        do_action( 'sellvia_order_shipped_notification', $order->get_id(), $order );

        $order->add_order_note( __( 'Order shipped email notification has been sent.', 'sel' ), false, true );

        do_action( 'woocommerce_after_resend_order_email', $order, 'sellvia_order_shipped_notification' );

    }

}