<?php

namespace sellvia;

class Upgrade {

    private $db_version;

    private $running_version;

    private $is_upgrade_completed = false;

    private $updates = [
        '0.2.7' => [
            'update_0_2_7',
        ],
        '0.4.0' => [
            'update_0_4_0',
        ],
        '0.4.3' => [
            'update_0_4_3',
        ],
        '0.5.4' => [
            'update_0_5_4'
        ],
        '0.6.1' => [
            'update_0_6_1'
        ],
        '0.6.6' => [
            'update_0_6_6',
        ]
    ];

    public function __construct() {
        $this->check_version();
    }

    public function check_version() {

        $this->db_version = get_option( 'sellvia-version' );
        $this->running_version = SELLVIA_VERSION;
        // If current version is new and current version has any update run it.
        if ( !empty($this->updates) && version_compare( $this->db_version, $this->running_version, '<' ) && version_compare( $this->db_version, max( array_keys( $this->updates ) ), '<' ) ) {
            $this->update();
        }
    }

    private function update() {

        foreach ( $this->updates as $version => $update_callbacks ) {
            if ( version_compare( $this->db_version, $version, '<' ) ) {
                // Run all callbacks.
                foreach ( $update_callbacks as $update_callback ) {
                    if ( method_exists( $this, $update_callback ) ) {
                        $this->$update_callback();
                    } elseif ( function_exists( $update_callback ) ) {
                        $update_callback();
                    }
                }
            }
        }
        $this->is_upgrade_completed = true;
    }

    private function update_0_2_7() {

        global $wpdb;

        $self_products = $wpdb->get_results(
            "SELECT
                m.post_id, m.meta_value as sellvia_post_id                
                FROM {$wpdb->prefix}postmeta m
                WHERE m.meta_key = 'sellvia_post_id'",
            ARRAY_A);

        if( !empty($self_products) ) {
            foreach ($self_products as $self_product ){
                update_post_meta( $self_product['post_id'], 'sellvia_need_update', 1 );
            }
        }
    }

    private function update_0_4_0(){
        update_option( 'sellvia-images-settings' , [
            'feature_images'   => 1,
            'gallery_images'   => 1,
            'variation_images' => 1,
            'content_images'   => 1,
            'reviews_images'   => 1,
        ], false );
    }

    private function update_0_4_3(){

        update_option( 'sellvia_update_product_settings' , [
            'status'         => 'draft',
            'notify_status'  => 0,
            'variant'        => 'disable',
            'notify_variant' => 0,
            'cost'           => 'update',
            'notify_cost'    => 0,
            'stock'          => 'update',
            'notify_stock'   => 0,
        ], false );
    }

    private function update_0_5_4(){
        update_option( 'sellvia_fulfillment_settings' , [
            'auto' => 1,
        ], false );
    }

    private function update_0_6_1(){

        global $wpdb;

        $create_table_columns = [
            'sellvia_shipping_group' => [
                'retail' => "ALTER TABLE `{$wpdb->prefix}sellvia_shipping_group` ADD `retail` DECIMAL(10,2) DEFAULT 0.00;",
                'retail_details' => "ALTER TABLE `{$wpdb->prefix}sellvia_shipping_group` ADD `retail_details` TEXT DEFAULT NULL;",
            ]
        ];

        self::maybe_add_columns( $create_table_columns );

        $update_table_columns = [
            'sellvia_shipping_group' => [
                'retail_details' => "UPDATE `{$wpdb->prefix}sellvia_shipping_group` SET `retail_details` = 'a:5:{i:1;d:0;i:2;d:0;i:3;d:0;i:4;d:0;i:5;d:0;}';",
            ]
        ];

        self::may_be_update_columns( $update_table_columns );
    }

    private function update_0_6_6(){

        $settings = sellvia_get_images_settings();

        $settings['webp'] = 0;

        update_option( 'sellvia-images-settings' , $settings, false );
    }

    static function maybe_add_columns( $args = [] ){

        if( empty( $args ) ) return;

        global $wpdb;

        foreach( $args as $key => $val ) {

            $result = $wpdb->get_results(
                $wpdb->prepare(
                    "SELECT *
			 	 FROM `information_schema`.`COLUMNS`
			 	 WHERE `TABLE_SCHEMA` = '%s' AND `TABLE_NAME` = '{$wpdb->prefix}{$key}'",
                    DB_NAME
                )
            );

            $col = [];
            if( count($result) > 0 ) foreach( $result as $column ) {
                $col[] = $column->COLUMN_NAME;
            }

            if( count($col) > 0 ) foreach( $val as $k => $v ) {
                if( ! in_array( $k, $col ) )
                    $wpdb->query( $v );
            }
        }
    }

    static function may_be_update_columns( $args = [] ){

        if( empty( $args ) ) return;

        global $wpdb;

        foreach( $args as $key => $val ) {

            $result = $wpdb->get_results(
                $wpdb->prepare(
                    "SELECT *
			 	 FROM `information_schema`.`COLUMNS`
			 	 WHERE `TABLE_SCHEMA` = '%s' AND `TABLE_NAME` = '{$wpdb->prefix}{$key}'",
                    DB_NAME
                )
            );

            $col = [];
            if( count($result) > 0 ) foreach( $result as $column ) {
                $col[] = $column->COLUMN_NAME;
            }

            if( count($col) > 0 ) foreach( $val as $k => $v ) {
                if( in_array( $k, $col ) )
                    $wpdb->query( $v );
            }
        }
    }

//    static function call( $path, $args = [] ) {
//
//        $response = wp_remote_post( SELLVIA_REQUEST_PATH . $path, [
//            'method'    => 'POST',
//            'timeout'   => 120,
//            'sslverify' => false,
//            'headers'   => [],
//            'body'      => json_encode( $args )
//        ] );
//
//        if( is_wp_error( $response ) )
//            return [ 'error' => $response->get_error_message() ];
//
//        if( ! wp_remote_retrieve_response_code( $response ) == '200' )
//            return [ 'error' => __( 'The response from the server has not been received', 'sel' ) ];
//
//        return json_decode( wp_remote_retrieve_body( $response ), true );
//    }

}
