<?php

namespace sellvia\front;

class NiceAttributes{

    private $enable;

    private $text_only;

    private $custom_css;

    private $custom_width;

    private $custom_height;

    private $round_variations;

    private $variations_color_apply;

    public function __construct() {

        $this->init();
    }

    public function init(){

        $this->settings();

        if ( $this->enable ) {

            add_action( 'admin_init', [$this, 'init_taxonomy_hook'], 10, 2 );

            add_action( 'admin_enqueue_scripts', [ $this, 'wp_color_picker' ] );

            add_action( 'woocommerce_product_option_terms', [ $this, 'product_option_terms' ], 10, 3 );

            add_action( 'wp_ajax_woocommerce_save_attributes', [ $this, 'save_attributes' ], 1 );

            add_action( 'woocommerce_product_options_attributes', [ $this, 'admin_attributes_styles'] );

            add_action( 'wp_enqueue_scripts', [ $this, 'enqueue'] );
        }
    }

    private function settings(){

        $settings = sellvia_get_nice_attributes_settings();

        $this->enable    = boolval( $settings['enable'] );
        $this->text_only = boolval( $settings['text_only'] );
        $this->custom_css = trim( $settings['custom_css'] );
        $this->custom_width = is_numeric( trim( $settings['custom_width'] ) ) && intval( trim( $settings['custom_width'] ) ) > 0 ? min( (int) trim( $settings['custom_width'] ), 100 ) : ''; ;
        $this->custom_height = is_numeric( trim( $settings['custom_height'] ) ) && intval( trim( $settings['custom_height'] ) ) > 0 ? min( (int) trim( $settings['custom_height'] ), 100 ) : ''; ;
        $this->round_variations = boolval( $settings['round_variations'] );
        $this->variations_color_apply = boolval( $settings['variations_color_apply'] );
    }

    public function init_taxonomy_hook(){

        if ( isset( $_GET[ 'taxonomy' ] ) && strpos($_GET[ 'taxonomy' ], 'pa_') >= 0 ) {
            add_action(
                $_GET[ 'taxonomy' ] . '_edit_form_fields',
                [$this, 'taxonomy_extra_fields']
            );
        }

        add_action( 'edited_terms', [ $this, 'save_term_taxonomy' ], 10, 2 );

    }

    public function taxonomy_extra_fields( $term ){

        if ( $this->variations_color_apply ) {

            $data = get_option( 'adsw-product-settings' );
            $data = maybe_unserialize( $data );

            if ( isset( $data[ 'attr_options' ][ $term->taxonomy ][ 'product_color' ][ $term->term_id ] ) ) {

                $attr_color = $data[ 'attr_options' ][ $term->taxonomy ][ 'product_color' ][ $term->term_id ];
            } else {

                $attr_color = null;
            }

            $forminputname = 'attr_options[product_color][' . $term->term_id . ']'; ?>

            <tr class="form-field">
                <th scope="row" valign="top"><label for="<?php echo $forminputname; ?>"><?php _e( 'Color code', 'sel' ); ?></label></th>
                <td>
                    <script>jQuery(function($) {jQuery('input[name*="color"]').wpColorPicker();});</script>
                    <style>.wp-picker-container {
                            display: inline-block;
                            outline: 0;
                            position: relative;
                            vertical-align: middle;
                            border: none;
                        }
                        .wp-picker-container .wp-color-result.button {
                            display: flex;
                            flex-direction: row-reverse;
                            width: 127px;
                            height: 38px;
                            border-color: transparent;
                            border-width: 0;
                        }
                        .wp-picker-container input.wp-color-picker {
                            width: 68px;
                            font-size: 13px;
                            font-family: monospace;
                            text-align: center;
                            line-height: 26px;
                            margin: 0;
                            margin-left: 6px;
                            height: 38px;
                            border: 0;
                            box-shadow: none;
                        }
                        .wp-picker-container .wp-color-result-text {
                            font-size: 14px;
                            line-height: 38px;
                            height: 38px;
                        }
                        .wp-picker-container .wp-picker-input-wrap {
                            position: absolute;
                            top: 0;
                            right: 0;
                        }
                        .wp-picker-container  .wp-picker-active .wp-picker-input-wrap > label {
                            margin-top: 0;
                            display: block;
                            max-width: 100%;
                            margin-bottom: 5px;
                        }
                        .wp-picker-container .button.button-small.wp-picker-clear {
                            height: 38px;
                            line-height: 26px;
                            padding: 0 8px 1px;
                            font-size: 13px;
                            box-shadow: none;
                            border: 0;
                        }</style>
                    <input type="text" name="<?php echo $forminputname; ?>" id="<?php echo $forminputname; ?>" value="<?php echo esc_attr( $attr_color ) ? esc_attr( $attr_color ) : ''; ?>" placeholder="#rrggbb">
                </td>
            </tr>
            <?php
        }
    }

    public function admin_attributes_styles(){
        ?>
        <style>
            .adsw-attribute-box{padding:25px 0 10px}
            .adsw-attribute-box .adsw-attribute-image-wrap{
                border-top: 1px solid #eee;
                border-bottom: 1px solid #eee;
                margin-bottom: 10px;
                padding: 10px 0;
            }
            #woocommerce-product-data .adsw-attribute-box .adsw-attribute-image-wrap .wc-radios li{
                padding: 0;
                margin: 0;
            }
            #woocommerce-product-data .adsw-attribute-box .adsw-attribute-image-wrap .form-field + .form-field{
                margin-top: 10px;
            }
        </style>
        <?php
    }

    public function save_term_taxonomy( $term_id, $taxonomy ){

        $product_color = isset( $_POST['attr_options']['product_color'][ $term_id ] ) ? trim( $_POST['attr_options']['product_color'][ $term_id ] ) : '';

        $data = get_option( 'adsw-product-settings', false );
        $data = maybe_unserialize( $data );

        if( empty( $data ) || !is_array( $data ) ) {
            $data = [];
        }

        $data['attr_options'][ $taxonomy ]['product_color'][ $term_id ] = $product_color;

        update_option( 'adsw-product-settings', $data, false );
    }

    public function wp_color_picker(){
        wp_enqueue_script( 'wp-color-picker' );
        wp_enqueue_style( 'wp-color-picker' );
    }

    public function product_option_terms( $attribute_taxonomy, $i, $attribute ) {

        global $post;
        $post_id = null;

        if($post || is_object( $post )){
            $post_id = $post->ID;
        } else if( isset( $_POST['post_id'] ) && $_POST['post_id'] ){
            $post_id = $_POST['post_id'];
        }

        if( !$post_id )
            return null;

        $override_global = get_post_meta($post_id, 'adsw-override-global-attribute-image', true);
        $has_image = get_post_meta($post_id, 'adsw-attribute-image', true);
        $drop_attrib = get_post_meta($post_id, 'adsw-attributes', true);
        $drop_attrib = maybe_unserialize($drop_attrib);
        $drop_attrib = ( !empty( $drop_attrib ) && is_array($drop_attrib) ) ? $drop_attrib : [];

        $niceAttributesColor = get_post_meta($post_id, 'adsw-niceAttributesColor', true);
        $niceAttributesColor = (is_array($niceAttributesColor)) ? $niceAttributesColor : [];

        if( is_object( $attribute_taxonomy ) && isset( $attribute_taxonomy->attribute_name ) ) {

            $name = $post_id . '-' . $attribute_taxonomy->attribute_name;

            /*if( $i == 0 && ! $has_image ) {
                update_post_meta( $post->ID, 'adsw-attribute-image', $name );
                $has_image = $name;
            }*/

            echo '<div class="adsw-attribute-box">';

            echo '<div class="adsw-attribute-image-wrap">';

            printf(
                '<fieldset class="form-field %d_attributeImage_field">
                <ul class="wc-radios">
                    <li>
                        <label>
                            <input name="overrideGlobal" value="%s" type="checkbox" class="checkbox" style="%s" %s> %s
                            <span class="woocommerce-help-tip" data-tip="%s"></span>
                        </label>
                    </li>
                </ul>
            </fieldset>',
                $i,
                $name,
                'width:16px',
                $override_global == $name ? 'checked="checked"' : '',
                __( 'Override global settings of the Sellvia plugin', 'sel' ),
                __('Works only with \'Display product variations as images\' option', 'sel')
            );

            printf(
                '<fieldset class="form-field %d_attributeImage_field">
                <ul class="wc-radios">
                    <li><label><input name="adsw-attribute-image" value="%s" type="checkbox" class="checkbox" style="%s" %s> %s</label></li>
                </ul>
            </fieldset>',
                $i,
                $name,
                'width:16px',
                $has_image == $name ? 'checked="checked"' : '',
                __( 'Display product variations as images', 'sel' )
            );

            echo '</div>';

            printf(
                '<fieldset class="form-field %d__dropdownAttrib">
                <ul class="wc-radios">
                    <li><label><input name="dropdownAttrib[%1$d]" value="%s" type="checkbox" class="checkbox" style="%s" %s> %s</label></li>
                </ul>
            </fieldset>',
                $i,
                'pa_' . $attribute_taxonomy->attribute_name,
                'width:16px',
                in_array( 'pa_' . $attribute_taxonomy->attribute_name, $drop_attrib ) ? 'checked="checked"' : '',
                __( 'Display product variations in dropdown menu', 'sel' )
            );

            printf(
                '<fieldset class="form-field %d__niceAttributesColor">
                <ul class="wc-radios">
                    <li><label><input name="niceAttributesColor[%1$d]" value="%s" type="checkbox" class="checkbox" style="%s" %s> %s</label></li>
                </ul>
            </fieldset>',
                $i,
                'pa_' . $attribute_taxonomy->attribute_name,
                'width:16px',
                in_array( 'pa_' . $attribute_taxonomy->attribute_name, $niceAttributesColor ) ? 'checked="checked"' : '',
                __('Disable Nice Attributes color value', 'sel')
            );

            echo '</div>';
        }
    }

    public function enqueue(){

        wp_enqueue_style( 'sellvia-nice-attributes-css', SELLVIA_URL . '/assets/css/front/nice-attributes.min.css', [], SELLVIA_VERSION );
        wp_enqueue_script( 'sellvia-nice-attributes-js', SELLVIA_URL . '/assets/js/front/nice-attributes.min.js', [
            'jquery',
            'wc-add-to-cart-variation'
        ], SELLVIA_VERSION );
    }

    public function save_attributes() {

        check_ajax_referer( 'save-attributes', 'security' );

        if ( ! current_user_can( 'edit_products' ) ) {
            die( -1 );
        }

        parse_str( $_POST[ 'data' ], $data );
        $post_id = absint( $_POST[ 'post_id' ] );

        if( isset( $data[ 'overrideGlobal' ] ) ) {
            update_post_meta( $post_id, 'adsw-override-global-attribute-image', $data[ 'overrideGlobal' ]);
        } else {
            delete_post_meta( $post_id, 'adsw-override-global-attribute-image' );
        }

        if( isset( $data[ 'adsw-attribute-image' ] ) ) {
            update_post_meta( $post_id, 'adsw-attribute-image', $data[ 'adsw-attribute-image' ]);
        } else {
            delete_post_meta( $post_id, 'adsw-attribute-image' );
        }

        $foo = [];
        if( isset( $data[ 'dropdownAttrib' ] ) ) foreach( $data[ 'dropdownAttrib' ] as $v ) {
            $foo[] = $v;
        }
        update_post_meta( $post_id, 'adsw-attributes', serialize( $foo ) );

        $foo = [];
        if( isset( $data[ 'niceAttributesColor' ] ) ) foreach( $data[ 'niceAttributesColor' ] as $v ) {
            $foo[] = $v;
        }
        update_post_meta( $post_id, 'adsw-niceAttributesColor', $foo );
    }

}
