<?php

namespace sellvia\front;

class Reviews {

    private $enable = false;

    private $enable_popup = false;

    public function __construct() {

        $this->init();
    }

    public function init(){

        if( SELLVIA_PLUGIN === 'WOO' ) {

            $this->settings();

            if ($this->enable) {

                add_filter( 'comments_array', [$this, 'sellvia_comments_array'], 10, 2 );

                add_action( 'wp_enqueue_scripts', [ $this, 'enqueue'] );
            }
        }
    }

    private function settings(){

        $settings = sellvia_get_review_settings();

        $this->enable       = boolval( $settings['enable'] );
        $this->enable_popup = boolval( $settings['enable_popup'] );
    }

    public function enqueue(){

        if( is_singular( 'product' ) ){

            wp_enqueue_style( 'sellvia_review_gallery_style', SELLVIA_URL . '/assets/css/front/review-gallery.min.css', [], SELLVIA_VERSION );

            if($this->enable_popup){

                wp_enqueue_style( 'sellvia_baguetteBox_style', SELLVIA_URL . '/assets/css/front/baguetteBox.min.css', [], SELLVIA_VERSION );

                wp_enqueue_script( 'sellvia_baguetteBox_script', SELLVIA_URL . '/assets/js/front/baguetteBox.min.js', [ 'jquery' ], SELLVIA_VERSION  );

                wp_enqueue_script( 'sellvia_review_gallery_script', SELLVIA_URL . '/assets/js/front/review-gallery.min.js', [ 'jquery', 'sellvia_baguetteBox_script' ], SELLVIA_VERSION  );
            }
        }
    }

    public function sellvia_comments_array( $comments_flat, $post_id ) {

        if( is_array( $comments_flat ) && count( $comments_flat ) ) {

            global $wpdb;

            $ids = [];
            foreach( $comments_flat as $item ) {

                $ids[] = $item->comment_ID;
            }

            $images = $wpdb->get_results(
                "SELECT * FROM {$wpdb->commentmeta}
            WHERE meta_key = 'images' AND comment_id IN (" . implode(',', $ids) . ")"
            );

            $foo = [];

            if( $images ) foreach( $images as $image ) {
                $foo[ $image->comment_id ] = $image->meta_value;
            }

            if( count($foo) ) foreach( $comments_flat as $i => $item ) {

                $layout  = '';

                $thumbsize = apply_filters( 'adsw_rewiew_gallery_thumbsize', 'thumbnail' );

                $gallery = isset( $foo[ $item->comment_ID ] ) ? $this->get_review_gallery( $foo[ $item->comment_ID ], $thumbsize ) : false;

                if ( $gallery ) {

                    $layout .= '<ul class="adsw-gallery">';

                    foreach ( $gallery as $image )
                        $layout .= '<li><a href="' . $image[ 'url' ] .'" data-lightbox="image-' . $item->comment_ID . '">'.
                            '<img src="' . $image[ $thumbsize ] . '" style="max-height: 50px;" /></a></li>';

                    $layout .= '</ul>';
                }

                $comments_flat[ $i ]->comment_content = $item->comment_content . $layout;
            }
        }

        return $comments_flat;
    }

    private function get_review_gallery( $args, $size = 'medium' ) {

        $args = maybe_unserialize( $args );

        if ( ! is_array( $args ) ) {
            return false;
        }

        $foo = [];
        $media = [];

        foreach ( $args as $i => $item ) {

            if ( is_numeric( $item ) ) {
                $media[]           = $item;
                $foo[ $i ][ 'id' ] = intval( $item );
            }
        }

        $media = $this->get_list_images($media, $size);

        foreach ( $foo as $i => $item ) {

            if ( isset( $item[ 'id' ] ) ) {

                if ( isset( $media[ $item[ 'id' ] ] ) ) {
                    $foo[ $i ][$size] = $media[ $item[ 'id' ] ][$size][ 'url' ];
                    $foo[ $i ][ 'url' ]        = $media[ $item[ 'id' ] ][ 'full' ][ 'url' ];
                    $foo[ $i ][ 'id' ]         = $item[ 'id' ];

                } else {
                    unset( $foo[ $i ] );
                }
            }
        }
        return $foo;
    }

    function get_list_images( $args = [], $size = 'thumbnail' ) {

        if ( count( $args ) <= 0 ) {
            return false;
        }

        global $wpdb;

        $fileds = implode( ',', $args );

        $result = $wpdb->get_results(
            "SELECT post_id, meta_value FROM {$wpdb->postmeta} 
          WHERE meta_key = '_wp_attachment_metadata' AND post_id IN ({$fileds}) 
          ORDER BY FIELD(post_id, {$fileds})" );

        if ( ! $result ) {
            return false;
        }

        if ( ! is_array( $size ) ) {
            $size = [ $size ];
        }

        $upload_dir = wp_upload_dir();
        $upload_dir = $upload_dir[ 'baseurl' ];

        $foo = [];

        foreach ( $result as $item ) {

            $f = maybe_unserialize( $item->meta_value );
            if ( ! $f ) {
                continue;
            }

            $folder = substr( $f[ 'file' ], 0, strrpos( $f[ 'file' ], '/' ) );

            $foo[ $item->post_id ][ 'full' ] = [
                'width'  => $f[ 'width' ],
                'height' => $f[ 'height' ],
                'url'    => apply_filters( 'ads_get_list_images_url', $upload_dir . '/' . $f[ 'file' ], $item->post_id )
            ];

            foreach ( $size as $i ) {
                $foo[ $item->post_id ][ $i ] = isset( $f[ 'sizes' ][ $i ] ) ? [
                    'width'  => $f[ 'sizes' ][ $i ][ 'width' ],
                    'height' => $f[ 'sizes' ][ $i ][ 'height' ],
                    'url'    => apply_filters( 'ads_get_list_images_url', $upload_dir . '/' . $folder . '/' . $f[ 'sizes' ][ $i ][ 'file' ], $item->post_id )
                ] : $foo[ $item->post_id ][ 'full' ];
            }
        }

        return $foo;
    }
}
