<?php

namespace sellvia\front;

class SizeChart {

    private $enable = false;

    public function __construct() {

        $this->init();
    }

    private function settings(){

        $settings = sellvia_get_size_chart_settings();

        $this->enable = boolval( $settings['enable'] );
    }

    public function init(){

        if( SELLVIA_PLUGIN === 'WOO' ) {

            $this->settings();

            if ( $this->enable ) {

                add_action( 'woocommerce_before_single_variation', [ $this, 'render'] );

                add_action( 'wp_enqueue_scripts', [ $this, 'enqueue'] );
            }
        }
    }

    public function enqueue(){

        if( is_singular( 'product' ) ) {

            wp_enqueue_style('sellvia_size_chart_style', SELLVIA_URL . '/assets/css/front/size-chart.min.css', [], SELLVIA_VERSION);

            wp_enqueue_script('sellvia_size_chart_script', SELLVIA_URL . '/assets/js/front/size-chart.min.js', ['jquery'], SELLVIA_VERSION);
        }
    }

    public function render(){

        global $product;

        $product_id = $product->get_id();

        $sizeAttr = get_post_meta( $product_id, 'adswSizeAttr', true );
//        $sizeAttr = get_post_meta( $product_id, 'sellvia_sizeAttr', true );

        if( !empty( $sizeAttr ) && is_array( $sizeAttr ) ) { ?>

            <div class="size_chart_cont">
                <a href="" class="size_chart_btn"><?php _e('Size Guide', 'sel'); ?></a>
            </div>
            <div class="chart_modal">
                <div class="chart_modal_inner">
                    <div class="chart_modal_block">
                        <span class="chart_close"></span>
                        <div class="chart_table_block">
                            <table class="size_chart_table">
                                <tr>
                                    <?php foreach ($sizeAttr['title'] as $k => $v) {
                                        echo '<th>' . $v . '</th>';
                                    } ?>
                                </tr>
                                <?php foreach ($sizeAttr['list'] as $k => $v) { ?>
                                    <tr>
                                        <?php foreach ($v as $v2) {
                                            echo '<td>' . $v2 . '</td>';
                                        } ?>
                                    </tr>
                                <?php } ?>
                            </table>
                        </div>
                    </div>
                </div>
            </div>
        <?php }
    }
}
